@@ -25,9 +25,11 @@
 #include "bus.h"
 
 #include <stdio.h>
+#include <stdlib.h>
 
 #include "activation.h"
 #include "connection.h"
+#include "containers.h"
 #include "services.h"
 #include "utils.h"
 #include "policy.h"
@@ -37,6 +39,7 @@
 #include "apparmor.h"
 #include "audit.h"
 #include "dir-watch.h"
+#include <dbus/dbus-auth.h>
 #include <dbus/dbus-list.h>
 #include <dbus/dbus-hash.h>
 #include <dbus/dbus-credentials.h>
@@ -67,11 +70,15 @@ struct BusContext
   BusMatchmaker *matchmaker;
   BusLimits limits;
   DBusRLimit *initial_fd_limit;
+  BusContainers *containers;
   unsigned int fork : 1;
   unsigned int syslog : 1;
   unsigned int keep_umask : 1;
   unsigned int allow_anonymous : 1;
   unsigned int systemd_activation : 1;
+#ifdef DBUS_ENABLE_EMBEDDED_TESTS
+  unsigned int quiet_log : 1;
+#endif
   dbus_bool_t watches_enabled;
 };
 
@@ -90,20 +97,17 @@ server_get_context (DBusServer *server)
   BusContext *context;
   BusServerData *bd;
 
-  if (!dbus_server_allocate_data_slot (&server_data_slot))
-    return NULL;
+  /* this data slot was allocated by the BusContext */
+  _dbus_assert (server_data_slot >= 0);
 
   bd = BUS_SERVER_DATA (server);
-  if (bd == NULL)
-    {
-      dbus_server_free_data_slot (&server_data_slot);
-      return NULL;
-    }
+
+  /* every DBusServer in the dbus-daemon's main loop has gone through
+   * bus_context_setup_server() */
+  _dbus_assert (bd != NULL);
 
   context = bd->context;
 
-  dbus_server_free_data_slot (&server_data_slot);
-
   return context;
 }
 
@@ -172,18 +176,25 @@ new_connection_callback (DBusServer     *server,
                          DBusConnection *new_connection,
                          void           *data)
 {
-  BusContext *context = data;
+  /* If this fails it logs a warning, so we don't need to do that */
+  bus_context_add_incoming_connection (data, new_connection);
+}
 
+dbus_bool_t
+bus_context_add_incoming_connection (BusContext *context,
+                                     DBusConnection *new_connection)
+{
+  /* If this fails it logs a warning, so we don't need to do that */
   if (!bus_connections_setup_connection (context->connections, new_connection))
     {
-      _dbus_verbose ("No memory to setup new connection\n");
-
       /* if we don't do this, it will get unref'd without
        * being disconnected... kind of strange really
        * that we have to do this, people won't get it right
        * in general.
        */
       dbus_connection_close (new_connection);
+      /* on OOM, we won't have ref'd the connection so it will die. */
+      return FALSE;
     }
 
   dbus_connection_set_max_received_size (new_connection,
@@ -201,7 +212,7 @@ new_connection_callback (DBusServer     *server,
   dbus_connection_set_allow_anonymous (new_connection,
                                        context->allow_anonymous);
 
-  /* on OOM, we won't have ref'd the connection so it will die. */
+  return TRUE;
 }
 
 static void
@@ -217,6 +228,25 @@ setup_server (BusContext *context,
               DBusServer *server,
               char      **auth_mechanisms,
               DBusError  *error)
+{
+  if (!bus_context_setup_server (context, server, error))
+    return FALSE;
+
+  if (!dbus_server_set_auth_mechanisms (server, (const char**) auth_mechanisms))
+    {
+      BUS_SET_OOM (error);
+      return FALSE;
+    }
+
+  dbus_server_set_new_connection_function (server, new_connection_callback,
+                                           context, NULL);
+  return TRUE;
+}
+
+dbus_bool_t
+bus_context_setup_server (BusContext                 *context,
+                          DBusServer                 *server,
+                          DBusError                  *error)
 {
   BusServerData *bd;
 
@@ -232,16 +262,6 @@ setup_server (BusContext *context,
 
   bd->context = context;
 
-  if (!dbus_server_set_auth_mechanisms (server, (const char**) auth_mechanisms))
-    {
-      BUS_SET_OOM (error);
-      return FALSE;
-    }
-
-  dbus_server_set_new_connection_function (server,
-                                           new_connection_callback,
-                                           context, NULL);
-
   if (!dbus_server_set_watch_functions (server,
                                         add_server_watch,
                                         remove_server_watch,
@@ -285,6 +305,7 @@ process_config_first_time_only (BusContext       *context,
   DBusList **auth_mechanisms_list;
   int len;
   dbus_bool_t retval;
+  DBusLogFlags log_flags = DBUS_LOG_FLAGS_STDERR;
 
   _DBUS_ASSERT_ERROR_IS_CLEAR (error);
 
@@ -292,7 +313,27 @@ process_config_first_time_only (BusContext       *context,
   auth_mechanisms = NULL;
   pidfile = NULL;
 
-  _dbus_init_system_log (TRUE);
+  if (flags & BUS_CONTEXT_FLAG_SYSLOG_ALWAYS)
+    {
+      context->syslog = TRUE;
+      log_flags |= DBUS_LOG_FLAGS_SYSTEM_LOG;
+
+      if (flags & BUS_CONTEXT_FLAG_SYSLOG_ONLY)
+        log_flags &= ~DBUS_LOG_FLAGS_STDERR;
+    }
+  else if (flags & BUS_CONTEXT_FLAG_SYSLOG_NEVER)
+    {
+      context->syslog = FALSE;
+    }
+  else
+    {
+      context->syslog = bus_config_parser_get_syslog (parser);
+
+      if (context->syslog)
+        log_flags |= DBUS_LOG_FLAGS_SYSTEM_LOG;
+    }
+
+  _dbus_init_system_log ("dbus-daemon", log_flags);
 
   if (flags & BUS_CONTEXT_FLAG_SYSTEMD_ACTIVATION)
     context->systemd_activation = TRUE;
@@ -412,6 +453,26 @@ process_config_first_time_only (BusContext       *context,
       link = _dbus_list_get_first_link (auth_mechanisms_list);
       while (link != NULL)
         {
+          DBusString name;
+          _dbus_string_init_const (&name, link->data);
+          if (!_dbus_auth_is_supported_mechanism (&name))
+            {
+              DBusString list;
+              if (!_dbus_string_init (&list))
+                goto oom;
+
+              if (!_dbus_auth_dump_supported_mechanisms (&list))
+                {
+                  _dbus_string_free (&list);
+                  goto oom;
+                }
+              dbus_set_error (error, DBUS_ERROR_FAILED,
+                              "Unsupported auth mechanism \"%s\" in bus config file detected. Supported mechanisms are \"%s\".",
+                              (char*)link->data,
+                              _dbus_string_get_const_data (&list));
+              _dbus_string_free (&list);
+              goto failed;
+            }
           auth_mechanisms[i] = _dbus_strdup (link->data);
           if (auth_mechanisms[i] == NULL)
             goto oom;
@@ -474,7 +535,6 @@ process_config_first_time_only (BusContext       *context,
     }
 
   context->fork = bus_config_parser_get_fork (parser);
-  context->syslog = bus_config_parser_get_syslog (parser);
   context->keep_umask = bus_config_parser_get_keep_umask (parser);
   context->allow_anonymous = bus_config_parser_get_allow_anonymous (parser);
 
@@ -633,32 +693,6 @@ process_config_every_time (BusContext      *context,
   return retval;
 }
 
-static dbus_bool_t
-list_concat_new (DBusList **a,
-                 DBusList **b,
-                 DBusList **result)
-{
-  DBusList *link;
-
-  *result = NULL;
-
-  for (link = _dbus_list_get_first_link (a); link; link = _dbus_list_get_next_link (a, link))
-    {
-      if (!_dbus_list_append (result, link->data))
-        goto oom;
-    }
-  for (link = _dbus_list_get_first_link (b); link; link = _dbus_list_get_next_link (b, link))
-    {
-      if (!_dbus_list_append (result, link->data))
-        goto oom;
-    }
-
-  return TRUE;
-oom:
-  _dbus_list_clear (result);
-  return FALSE;
-}
-
 static void
 raise_file_descriptor_limit (BusContext      *context)
 {
@@ -704,8 +738,6 @@ process_config_postinit (BusContext      *context,
   DBusHashTable *service_context_table;
   DBusList *watched_dirs = NULL;
 
-  raise_file_descriptor_limit (context);
-
   service_context_table = bus_config_parser_steal_service_context_table (parser);
   if (!bus_registry_set_service_context_table (context->registry,
 					       service_context_table))
@@ -719,9 +751,7 @@ process_config_postinit (BusContext      *context,
   /* We need to monitor both the configuration directories and directories
    * containing .service files.
    */
-  if (!list_concat_new (bus_config_parser_get_conf_dirs (parser),
-                        bus_config_parser_get_service_dirs (parser),
-                        &watched_dirs))
+  if (!bus_config_parser_get_watched_dirs (parser, &watched_dirs))
     {
       BUS_SET_OOM (error);
       return FALSE;
@@ -875,6 +905,14 @@ bus_context_new (const DBusString *config_file,
       goto failed;
     }
 
+  context->containers = bus_containers_new ();
+
+  if (context->containers == NULL)
+    {
+      BUS_SET_OOM (error);
+      goto failed;
+    }
+
   /* check user before we fork */
   if (context->user != NULL)
     {
@@ -931,9 +969,36 @@ bus_context_new (const DBusString *config_file,
       !_dbus_pipe_is_stdout_or_stderr (print_pid_pipe))
     _dbus_pipe_close (print_pid_pipe, NULL);
 
-  if (!bus_selinux_full_init ())
+  /* Raise the file descriptor limits before dropping the privileges
+   * required to do so.
+   */
+  raise_file_descriptor_limit (context);
+
+  /* Here we change our credentials if required,
+   * as soon as we've set up our sockets and pidfile.
+   * This must be done before initializing LSMs, so that the netlink
+   * monitoring thread started by avc_init() will not lose CAP_AUDIT_WRITE
+   * when the main thread calls setuid().
+   * https://bugs.freedesktop.org/show_bug.cgi?id=92832
+   */
+  if (context->user != NULL)
     {
-      bus_context_log (context, DBUS_SYSTEM_LOG_FATAL, "SELinux enabled but D-Bus initialization failed; check system log\n");
+      if (!_dbus_change_to_daemon_user (context->user, error))
+	{
+	  _DBUS_ASSERT_ERROR_IS_SET (error);
+	  goto failed;
+	}
+    }
+
+  /* Auditing should be initialized before LSMs, so that the LSMs are able
+   * to log audit-events that happen during their initialization.
+   */
+  bus_audit_init (context);
+
+  if (!bus_selinux_full_init (context, error))
+    {
+      _DBUS_ASSERT_ERROR_IS_SET (error);
+      goto failed;
     }
 
   if (!bus_apparmor_full_init (error))
@@ -942,6 +1007,13 @@ bus_context_new (const DBusString *config_file,
       goto failed;
     }
 
+  if (bus_selinux_enabled ())
+    {
+      if (context->syslog)
+        bus_context_log (context, DBUS_SYSTEM_LOG_INFO,
+                         "SELinux support is enabled\n");
+    }
+
   if (bus_apparmor_enabled ())
     {
       /* Only print AppArmor mediation message when syslog support is enabled */
@@ -950,6 +1022,11 @@ bus_context_new (const DBusString *config_file,
                          "AppArmor D-Bus mediation is enabled\n");
     }
 
+  /* When SELinux is used, this must happen after bus_selinux_full_init()
+   * so that it has access to the access vector cache, which is required
+   * to process <associate/> elements.
+   * http://lists.freedesktop.org/archives/dbus/2008-October/010491.html
+   */
   if (!process_config_postinit (context, parser, error))
     {
       _DBUS_ASSERT_ERROR_IS_SET (error);
@@ -962,20 +1039,6 @@ bus_context_new (const DBusString *config_file,
       parser = NULL;
     }
 
-  /* Here we change our credentials if required,
-   * as soon as we've set up our sockets and pidfile
-   */
-  if (context->user != NULL)
-    {
-      if (!_dbus_change_to_daemon_user (context->user, error))
-	{
-	  _DBUS_ASSERT_ERROR_IS_SET (error);
-	  goto failed;
-	}
-    }
-
-  bus_audit_init (context);
-
   dbus_server_free_data_slot (&server_data_slot);
 
   return context;
@@ -1007,6 +1070,8 @@ bus_context_reload_config (BusContext *context,
   DBusString config_file;
   dbus_bool_t ret;
 
+  _dbus_daemon_report_reloading ();
+
   /* Flush the user database cache */
   _dbus_flush_caches ();
 
@@ -1037,6 +1102,8 @@ bus_context_reload_config (BusContext *context,
     bus_context_log (context, DBUS_SYSTEM_LOG_INFO, "Unable to reload configuration: %s", error->message);
   if (parser != NULL)
     bus_config_parser_unref (parser);
+
+  _dbus_daemon_report_reloaded ();
   return ret;
 }
 
@@ -1075,6 +1142,9 @@ bus_context_shutdown (BusContext  *context)
 
       link = _dbus_list_get_next_link (&context->servers, link);
     }
+
+  if (context->containers != NULL)
+    bus_containers_stop_listening (context->containers);
 }
 
 BusContext *
@@ -1145,6 +1215,7 @@ bus_context_unref (BusContext *context)
           context->matchmaker = NULL;
         }
 
+      bus_clear_containers (&context->containers);
       dbus_free (context->config_file);
       dbus_free (context->log_prefix);
       dbus_free (context->type);
@@ -1249,10 +1320,10 @@ bus_context_allow_windows_user (BusContext       *context,
                                         windows_sid);
 }
 
-BusPolicy *
-bus_context_get_policy (BusContext *context)
+BusContainers *
+bus_context_get_containers (BusContext *context)
 {
-  return context->policy;
+  return context->containers;
 }
 
 BusClientPolicy*
@@ -1332,31 +1403,43 @@ bus_context_get_reply_timeout (BusContext *context)
   return context->limits.reply_timeout;
 }
 
+int bus_context_get_max_containers (BusContext *context)
+{
+  return context->limits.max_containers;
+}
+
+int bus_context_get_max_containers_per_user (BusContext *context)
+{
+  return context->limits.max_containers_per_user;
+}
+
+int bus_context_get_max_container_metadata_bytes (BusContext *context)
+{
+  return context->limits.max_container_metadata_bytes;
+}
+
+int bus_context_get_max_connections_per_container (BusContext *context)
+{
+  return context->limits.max_connections_per_container;
+}
+
 DBusRLimit *
 bus_context_get_initial_fd_limit (BusContext *context)
 {
   return context->initial_fd_limit;
 }
 
+dbus_bool_t
+bus_context_get_using_syslog (BusContext *context)
+{
+  return context->syslog;
+}
+
 void
 bus_context_log (BusContext *context, DBusSystemLogSeverity severity, const char *msg, ...)
 {
   va_list args;
 
-  if (!context->syslog)
-    {
-      /* we're not syslogging; just output to stderr */
-      va_start (args, msg);
-      vfprintf (stderr, msg, args);
-      fprintf (stderr, "\n");
-      va_end (args);
-
-      if (severity == DBUS_SYSTEM_LOG_FATAL)
-        _dbus_exit (1);
-
-      return;
-    }
-
   va_start (args, msg);
 
   if (context->log_prefix)
@@ -1370,12 +1453,16 @@ bus_context_log (BusContext *context, DBusSystemLogSeverity severity, const char
       if (!_dbus_string_append_printf_valist (&full_msg, msg, args))
         goto oom_out;
 
-      _dbus_system_log (severity, "%s", _dbus_string_get_const_data (&full_msg));
+#ifdef DBUS_ENABLE_EMBEDDED_TESTS
+      if (severity > DBUS_SYSTEM_LOG_WARNING || !context->quiet_log)
+#endif
+        _dbus_log (severity, "%s", _dbus_string_get_const_data (&full_msg));
+
     oom_out:
       _dbus_string_free (&full_msg);
     }
   else
-    _dbus_system_logv (severity, msg, args);
+    _dbus_logv (severity, msg, args);
 
 out:
   va_end (args);
@@ -1393,19 +1480,7 @@ bus_context_log_literal (BusContext            *context,
                          DBusSystemLogSeverity  severity,
                          const char            *msg)
 {
-  if (!context->syslog)
-    {
-      fputs (msg, stderr);
-      fputc ('\n', stderr);
-
-      if (severity == DBUS_SYSTEM_LOG_FATAL)
-        _dbus_exit (1);
-    }
-  else
-    {
-      _dbus_system_log (severity, "%s%s", nonnull (context->log_prefix, ""),
-                        msg);
-    }
+  _dbus_log (severity, "%s%s", nonnull (context->log_prefix, ""), msg);
 }
 
 void
@@ -1503,7 +1578,11 @@ complain_about_message (BusContext     *context,
  *
  * sender is the sender of the message.
  *
- * NULL for proposed_recipient or sender definitely means the bus driver.
+ * NULL for sender definitely means the bus driver.
+ *
+ * NULL for proposed_recipient may mean the bus driver, or may mean
+ * we are checking whether service-activation is allowed as a first
+ * pass before all details of the activated service are known.
  *
  * NULL for addressed_recipient may mean the bus driver, or may mean
  * no destination was specified in the message (e.g. a signal).
@@ -1515,6 +1594,7 @@ bus_context_check_security_policy (BusContext     *context,
                                    DBusConnection *addressed_recipient,
                                    DBusConnection *proposed_recipient,
                                    DBusMessage    *message,
+                                   BusActivationEntry *activation_entry,
                                    DBusError      *error)
 {
   const char *src, *dest;
@@ -1535,6 +1615,7 @@ bus_context_check_security_policy (BusContext     *context,
                 (sender == NULL && !bus_connection_is_active (proposed_recipient)));
   _dbus_assert (type == DBUS_MESSAGE_TYPE_SIGNAL ||
                 addressed_recipient != NULL ||
+                activation_entry != NULL ||
                 strcmp (dest, DBUS_SERVICE_DBUS) == 0);
 
   switch (type)
@@ -1600,7 +1681,9 @@ bus_context_check_security_policy (BusContext     *context,
 				    dbus_message_get_interface (message),
 				    dbus_message_get_member (message),
 				    dbus_message_get_error_name (message),
-				    dest ? dest : DBUS_SERVICE_DBUS, error))
+				    dest ? dest : DBUS_SERVICE_DBUS,
+				    activation_entry,
+				    error))
         {
           if (error != NULL && !dbus_error_is_set (error))
             {
@@ -1629,6 +1712,7 @@ bus_context_check_security_policy (BusContext     *context,
                                      dbus_message_get_error_name (message),
                                      dest ? dest : DBUS_SERVICE_DBUS,
                                      src ? src : DBUS_SERVICE_DBUS,
+                                     activation_entry,
                                      error))
         return FALSE;
 
@@ -1692,7 +1776,7 @@ bus_context_check_security_policy (BusContext     *context,
         }
       else
         {
-          _dbus_assert_not_reached ("a message was somehow sent to an inactive recipient from a source other than the message bus\n");
+          _dbus_assert_not_reached ("a message was somehow sent to an inactive recipient from a source other than the message bus");
           recipient_policy = NULL;
         }
     }
@@ -1761,6 +1845,10 @@ bus_context_check_security_policy (BusContext     *context,
   /* Record that we will allow a reply here in the future (don't
    * bother if the recipient is the bus or this is an eavesdropping
    * connection). Only the addressed recipient may reply.
+   *
+   * This isn't done for activation attempts because they have no addressed
+   * or proposed recipient; when we check whether to actually deliver the
+   * message, later, we'll record the reply expectation at that point.
    */
   if (type == DBUS_MESSAGE_TYPE_METHOD_CALL &&
       sender &&
@@ -1811,3 +1899,23 @@ bus_context_check_all_watches (BusContext *context)
       _dbus_server_toggle_all_watches (server, enabled);
     }
 }
+
+#ifdef DBUS_ENABLE_EMBEDDED_TESTS
+void
+bus_context_quiet_log_begin (BusContext *context)
+{
+  context->quiet_log = TRUE;
+}
+
+void
+bus_context_quiet_log_end (BusContext *context)
+{
+  context->quiet_log = FALSE;
+}
+
+dbus_bool_t
+bus_context_get_quiet_log (BusContext *context)
+{
+  return context->quiet_log;
+}
+#endif
