@@ -28,6 +28,7 @@
 #include "utils.h"
 #include <dbus/dbus-list.h>
 #include <dbus/dbus-internals.h>
+#include <dbus/dbus-test-tap.h>
 #include <string.h>
 
 /**
@@ -121,35 +122,10 @@ bus_config_parser_unref (BusConfigParser *parser)
   _dbus_string_free (&parser->user);
   _dbus_string_free (&parser->service_helper);
   _dbus_string_free (&parser->bus_type);
-
-  _dbus_list_foreach (&parser->service_dirs,
-                      (DBusForeachFunction) dbus_free,
-                      NULL);
-
-  _dbus_list_clear (&parser->service_dirs);
-
+  _dbus_list_clear_full (&parser->service_dirs, dbus_free);
   dbus_free (parser);
 }
 
-dbus_bool_t
-bus_config_parser_check_doctype (BusConfigParser   *parser,
-                                 const char        *doctype,
-                                 DBusError         *error)
-{
-  _DBUS_ASSERT_ERROR_IS_CLEAR (error);
-
-  if (strcmp (doctype, "busconfig") != 0)
-    {
-      dbus_set_error (error,
-                      DBUS_ERROR_FAILED,
-                      "Configuration file has the wrong document type %s",
-                      doctype);
-      return FALSE;
-    }
-  else
-    return TRUE;
-}
-
 dbus_bool_t
 bus_config_parser_start_element (BusConfigParser   *parser,
                                  const char        *element_name,
@@ -162,6 +138,7 @@ bus_config_parser_start_element (BusConfigParser   *parser,
 
   switch (parser->type)
     {
+    case ELEMENT_SERVICEDIR:
     case ELEMENT_SERVICEHELPER:
     case ELEMENT_USER:
     case ELEMENT_CONFIGTYPE:
@@ -185,6 +162,26 @@ bus_config_parser_start_element (BusConfigParser   *parser,
         break;
       }
 
+    case ELEMENT_NONE:
+    case ELEMENT_BUSCONFIG:
+    case ELEMENT_INCLUDE:
+    case ELEMENT_LISTEN:
+    case ELEMENT_AUTH:
+    case ELEMENT_POLICY:
+    case ELEMENT_LIMIT:
+    case ELEMENT_ALLOW:
+    case ELEMENT_DENY:
+    case ELEMENT_FORK:
+    case ELEMENT_PIDFILE:
+    case ELEMENT_INCLUDEDIR:
+    case ELEMENT_SELINUX:
+    case ELEMENT_ASSOCIATE:
+    case ELEMENT_STANDARD_SESSION_SERVICEDIRS:
+    case ELEMENT_KEEP_UMASK:
+    case ELEMENT_SYSLOG:
+    case ELEMENT_ALLOW_ANONYMOUS:
+    case ELEMENT_APPARMOR:
+      /* fall through */
     default:
       {
         /* we really don't care about the others... */
@@ -286,6 +283,28 @@ bus_config_parser_content (BusConfigParser   *parser,
           }
       }
       break;
+
+    case ELEMENT_NONE:
+    case ELEMENT_BUSCONFIG:
+    case ELEMENT_INCLUDE:
+    case ELEMENT_LISTEN:
+    case ELEMENT_AUTH:
+    case ELEMENT_POLICY:
+    case ELEMENT_LIMIT:
+    case ELEMENT_ALLOW:
+    case ELEMENT_DENY:
+    case ELEMENT_FORK:
+    case ELEMENT_PIDFILE:
+    case ELEMENT_INCLUDEDIR:
+    case ELEMENT_SELINUX:
+    case ELEMENT_ASSOCIATE:
+    case ELEMENT_STANDARD_SESSION_SERVICEDIRS:
+    case ELEMENT_STANDARD_SYSTEM_SERVICEDIRS:
+    case ELEMENT_KEEP_UMASK:
+    case ELEMENT_SYSLOG:
+    case ELEMENT_ALLOW_ANONYMOUS:
+    case ELEMENT_APPARMOR:
+      /* fall through */
     default:
       {
         /* we don't care about the others... really */
@@ -324,13 +343,16 @@ bus_config_parser_get_type (BusConfigParser *parser)
   return _dbus_string_get_const_data (&parser->bus_type);
 }
 
+/*
+ * @returns A list of strings, owned by the BusConfigParser
+ */
 DBusList**
-bus_config_parser_get_service_dirs (BusConfigParser *parser)
+bus_config_parser_get_service_paths (BusConfigParser *parser)
 {
   return &parser->service_dirs;
 }
 
-#ifdef DBUS_BUILD_TESTS
+#ifdef DBUS_ENABLE_EMBEDDED_TESTS
 #include <stdio.h>
 #include "test.h"
 
@@ -354,7 +376,7 @@ check_return_values (const DBusString *full_path)
   dbus_error_init (&error);
   retval = FALSE;
 
-  printf ("Testing values from: %s\n", _dbus_string_get_const_data (full_path));
+  _dbus_test_diag ("Testing values from: %s", _dbus_string_get_const_data (full_path));
 
   parser = bus_config_load (full_path, TRUE, NULL, &error);
   if (parser == NULL)
@@ -370,41 +392,41 @@ check_return_values (const DBusString *full_path)
   user = bus_config_parser_get_user (parser);
   if (user == NULL)
     {
-      _dbus_warn ("User was NULL!\n");
+      _dbus_warn ("User was NULL!");
       goto finish;
     }
 #if 0
   /* the username can be configured in configure.in so this test doesn't work */
   if (strcmp (user, "dbus") != 0)
     {
-      _dbus_warn ("User was invalid; '%s'!\n", user);
+      _dbus_warn ("User was invalid; '%s'!", user);
       goto finish;
     }
-  printf ("    <user>dbus</user> OKAY!\n");  
+  _dbus_test_diag ("    <user>dbus</user> OKAY!");
 #endif
   
   /* check type return value is okay */
   type = bus_config_parser_get_type (parser);
   if (type == NULL)
     {
-      _dbus_warn ("Type was NULL!\n");
+      _dbus_warn ("Type was NULL!");
       goto finish;
     }
   if (strcmp (type, "system") != 0)
     {
-      _dbus_warn ("Type was invalid; '%s'!\n", user);
+      _dbus_warn ("Type was invalid; '%s'!", user);
       goto finish;
     }
-  printf ("    <type>system</type> OKAY!\n");
+  _dbus_test_diag ("    <type>system</type> OKAY!");
 
   /* check dirs return value is okay */
-  dirs = bus_config_parser_get_service_dirs (parser);
+  dirs = bus_config_parser_get_service_paths (parser);
   if (dirs == NULL)
     {
-      _dbus_warn ("Service dirs are NULL!\n");
+      _dbus_warn ("Service dirs are NULL!");
       goto finish;
     }
-  printf ("    <standard_system_service_dirs/> OKAY!\n");
+  _dbus_test_diag ("    <standard_system_service_dirs/> OKAY!");
   /* NOTE: We tested the specific return values in the config-parser tests */
 
   /* woohoo! */
@@ -440,7 +462,7 @@ do_load (const DBusString *full_path,
         }
       else if (validity == VALID)
         {
-          _dbus_warn ("Failed to load valid file but still had memory: %s\n",
+          _dbus_warn ("Failed to load valid file but still had memory: %s",
                       error.message);
 
           dbus_error_free (&error);
@@ -460,7 +482,7 @@ do_load (const DBusString *full_path,
 
       if (validity == INVALID)
         {
-          _dbus_warn ("Accepted invalid file\n");
+          _dbus_warn ("Accepted invalid file");
           return FALSE;
         }
 
@@ -497,26 +519,26 @@ process_test_valid_subdir (const DBusString *test_base_dir,
   dir = NULL;
 
   if (!_dbus_string_init (&test_directory))
-    _dbus_assert_not_reached ("didn't allocate test_directory\n");
+    _dbus_test_fatal ("didn't allocate test_directory");
 
   _dbus_string_init_const (&filename, subdir);
 
   if (!_dbus_string_copy (test_base_dir, 0,
                           &test_directory, 0))
-    _dbus_assert_not_reached ("couldn't copy test_base_dir to test_directory");
+    _dbus_test_fatal ("couldn't copy test_base_dir to test_directory");
 
   if (!_dbus_concat_dir_and_file (&test_directory, &filename))
-    _dbus_assert_not_reached ("couldn't allocate full path");
+    _dbus_test_fatal ("couldn't allocate full path");
 
   _dbus_string_free (&filename);
   if (!_dbus_string_init (&filename))
-    _dbus_assert_not_reached ("didn't allocate filename string\n");
+    _dbus_test_fatal ("didn't allocate filename string");
 
   dbus_error_init (&error);
   dir = _dbus_directory_open (&test_directory, &error);
   if (dir == NULL)
     {
-      _dbus_warn ("Could not open %s: %s\n",
+      _dbus_warn ("Could not open %s: %s",
                   _dbus_string_get_const_data (&test_directory),
                   error.message);
       dbus_error_free (&error);
@@ -524,11 +546,11 @@ process_test_valid_subdir (const DBusString *test_base_dir,
     }
 
   if (validity == VALID)
-    printf ("Testing valid files:\n");
+    _dbus_test_diag ("Testing valid files:");
   else if (validity == INVALID)
-    printf ("Testing invalid files:\n");
+    _dbus_test_diag ("Testing invalid files:");
   else
-    printf ("Testing unknown files:\n");
+    _dbus_test_diag ("Testing unknown files:");
 
  next:
   while (_dbus_directory_get_next_file (dir, &filename, &error))
@@ -537,13 +559,13 @@ process_test_valid_subdir (const DBusString *test_base_dir,
       LoaderOomData d;
 
       if (!_dbus_string_init (&full_path))
-        _dbus_assert_not_reached ("couldn't init string");
+        _dbus_test_fatal ("couldn't init string");
 
       if (!_dbus_string_copy (&test_directory, 0, &full_path, 0))
-        _dbus_assert_not_reached ("couldn't copy dir to full_path");
+        _dbus_test_fatal ("couldn't copy dir to full_path");
 
       if (!_dbus_concat_dir_and_file (&full_path, &filename))
-        _dbus_assert_not_reached ("couldn't concat file to dir");
+        _dbus_test_fatal ("couldn't concat file to dir");
 
       if (!_dbus_string_ends_with_c_str (&full_path, ".conf"))
         {
@@ -553,7 +575,7 @@ process_test_valid_subdir (const DBusString *test_base_dir,
           goto next;
         }
 
-      printf ("    %s\n", _dbus_string_get_const_data (&filename));
+      _dbus_test_diag ("    %s", _dbus_string_get_const_data (&filename));
 
       _dbus_verbose (" expecting %s\n",
                      validity == VALID ? "valid" :
@@ -569,14 +591,14 @@ process_test_valid_subdir (const DBusString *test_base_dir,
        */
       /* if (!_dbus_test_oom_handling ("config-loader", check_loader_oom_func, &d)) */
       if (!check_loader_oom_func (&d))
-        _dbus_assert_not_reached ("test failed");
-      
+        _dbus_test_fatal ("test failed");
+
       _dbus_string_free (&full_path);
     }
 
   if (dbus_error_is_set (&error))
     {
-      _dbus_warn ("Could not get next file in %s: %s\n",
+      _dbus_warn ("Could not get next file in %s: %s",
                   _dbus_string_get_const_data (&test_directory),
                   error.message);
       dbus_error_free (&error);
@@ -649,14 +671,14 @@ check_file_valid (DBusString *full_path,
   dbus_bool_t retval;
 
   if (validity == VALID)
-    printf ("Testing valid file:\n");
+    _dbus_test_diag ("Testing valid file:");
   else if (validity == INVALID)
-    printf ("Testing invalid file:\n");
+    _dbus_test_diag ("Testing invalid file:");
   else
-    printf ("Testing unknown file:\n");
+    _dbus_test_diag ("Testing unknown file:");
 
   /* print the filename, just so we match the other output */
-  printf ("    %s\n", _dbus_string_get_const_data (full_path));
+  _dbus_test_diag ("    %s", _dbus_string_get_const_data (full_path));
 
   /* only test one file */
   retval = do_load (full_path, validity, TRUE);
@@ -675,7 +697,7 @@ bus_config_parser_trivial_test (const DBusString *test_data_dir)
   if (test_data_dir == NULL ||
       _dbus_string_get_length (test_data_dir) == 0)
     {
-      printf ("No test data\n");
+      _dbus_test_diag ("No test data");
       return TRUE;
     }
   
@@ -684,6 +706,13 @@ bus_config_parser_trivial_test (const DBusString *test_data_dir)
   if (!process_test_valid_subdir (test_data_dir, "valid-config-files", VALID))
     goto finish;
 
+#ifndef DBUS_WIN
+  /* We already test default_session_servicedirs and default_system_servicedirs
+   * in bus_config_parser_test() */
+  if (!process_test_valid_subdir (test_data_dir, "valid-config-files-system", VALID))
+    goto finish;
+#endif
+
   /* we don't process all the invalid files, as the trivial parser can't hope
    * to validate them all for all different syntaxes. We just check one broken
    * file to see if junk is received */
@@ -693,14 +722,16 @@ bus_config_parser_trivial_test (const DBusString *test_data_dir)
     goto finish;
   _dbus_string_free (&full_path);
 
+#ifndef DBUS_WIN
   /* just test if the check_file_valid works okay and we got sane values */
-  if (!make_full_path (test_data_dir, "valid-config-files", "system.conf", &full_path))
+  if (!make_full_path (test_data_dir, "valid-config-files-system", "system.conf", &full_path))
     goto finish;
   if (!check_file_valid (&full_path, VALID))
     goto finish;
   /* check to see if we got the correct values from the parser */
   if (!check_return_values (&full_path))
     goto finish;
+#endif
 
   /* woot! */
   retval = TRUE;
@@ -712,5 +743,5 @@ finish:
   return retval;
 }
 
-#endif /* DBUS_BUILD_TESTS */
+#endif /* DBUS_ENABLE_EMBEDDED_TESTS */
 
