@@ -66,6 +66,12 @@ typedef struct
   
 } BusDesktopFileParser;
 
+#define BUS_DESKTOP_FILE_PARSER_INIT \
+  { \
+    _DBUS_STRING_INIT_INVALID, \
+    NULL, 0, 0, 0, 0 \
+  }
+
 #define VALID_KEY_CHAR 1
 #define VALID_LOCALE_CHAR 2
 static unsigned char valid[256] = { 
@@ -88,15 +94,14 @@ static unsigned char valid[256] = {
 };
 
 static void report_error (BusDesktopFileParser *parser,
-			  char                 *message,
+			  const char           *message,
 			  const char           *error_name,
 			  DBusError            *error);
 
 static void
-parser_free (BusDesktopFileParser *parser)
+parser_clear (BusDesktopFileParser *parser)
 {
-  bus_desktop_file_free (parser->desktop_file);
-  
+  bus_clear_desktop_file (&parser->desktop_file);
   _dbus_string_free (&parser->data);
 }
 
@@ -302,7 +307,7 @@ new_section (BusDesktopFile *desktop_file,
 
 static BusDesktopFileSection* 
 open_section (BusDesktopFileParser *parser,
-              char                 *name)
+              const char           *name)
 {  
   BusDesktopFileSection *section;
 
@@ -376,17 +381,28 @@ parse_comment_or_blank (BusDesktopFileParser *parser)
 }
 
 static dbus_bool_t
-is_valid_section_name (const char *name)
+is_valid_section_name (const DBusString *section_name)
 {
-  /* 5. Group names may contain all ASCII characters except for control characters and '[' and ']'. */
+  int i;
+  int len;
+  const unsigned char *data;
 
-  while (*name)
+  len = _dbus_string_get_length (section_name);
+  data = _dbus_string_get_const_udata_len (section_name, 0, len);
+
+  /* 5. Group names may contain all ASCII characters except for control characters and '[' and ']'.
+   *
+   * We don't use isprint() here because it's locale-dependent. ASCII
+   * characters <= 0x1f and 0x7f are control characters, and bytes with
+   * values >= 0x80 aren't ASCII. 0x20 is a space, which we must allow,
+   * not least because DBUS_SERVICE_SECTION contains one. */
+
+  for (i = 0; i < len; i++)
     {
-      if (!((*name >= 'A' && *name <= 'Z') || (*name >= 'a' || *name <= 'z') ||
-	    *name == '\n' || *name == '\t'))
+      unsigned char c = data[i];
+
+      if (c <= 0x1f || c >= 0x7f || c  == '[' || c == ']')
 	return FALSE;
-      
-      name++;
     }
 
   return TRUE;
@@ -396,7 +412,7 @@ static dbus_bool_t
 parse_section_start (BusDesktopFileParser *parser, DBusError *error)
 {
   int line_end, eol_len;
-  char *section_name;
+  DBusString section_name;
 
   _DBUS_ASSERT_ERROR_IS_CLEAR (error);
     
@@ -407,32 +423,34 @@ parse_section_start (BusDesktopFileParser *parser, DBusError *error)
       _dbus_string_get_byte (&parser->data, line_end - 1) != ']')
     {
       report_error (parser, "Invalid syntax for section header", BUS_DESKTOP_PARSE_ERROR_INVALID_SYNTAX, error);
-      parser_free (parser);
       return FALSE;
     }
 
-  section_name = unescape_string (parser,
-                                  &parser->data, parser->pos + 1, line_end - 1,
-                                  error);
-
-  if (section_name == NULL)
+  if (!_dbus_string_init (&section_name))
     {
-      parser_free (parser);
+      BUS_SET_OOM (error);
       return FALSE;
     }
 
-  if (!is_valid_section_name (section_name))
+  if (!_dbus_string_copy_len (&parser->data, parser->pos + 1,
+                              line_end - parser->pos - 2,
+                              &section_name, 0))
+    {
+      _dbus_string_free (&section_name);
+      BUS_SET_OOM (error);
+      return FALSE;
+    }
+
+  if (!is_valid_section_name (&section_name))
     {
       report_error (parser, "Invalid characters in section name", BUS_DESKTOP_PARSE_ERROR_INVALID_CHARS, error);
-      parser_free (parser);
-      dbus_free (section_name);
+      _dbus_string_free (&section_name);
       return FALSE;
     }
 
-  if (open_section (parser, section_name) == NULL)
+  if (open_section (parser, _dbus_string_get_const_data (&section_name)) == NULL)
     {
-      dbus_free (section_name);
-      parser_free (parser);
+      _dbus_string_free (&section_name);
       BUS_SET_OOM (error);
       return FALSE;
     }
@@ -444,7 +462,7 @@ parse_section_start (BusDesktopFileParser *parser, DBusError *error)
   
   parser->line_num += 1;
 
-  dbus_free (section_name);
+  _dbus_string_free (&section_name);
   
   return TRUE;
 }
@@ -475,7 +493,6 @@ parse_key_value (BusDesktopFileParser *parser, DBusError *error)
   if (key_start == key_end)
     {
       report_error (parser, "Empty key name", BUS_DESKTOP_PARSE_ERROR_INVALID_SYNTAX, error);
-      parser_free (parser);
       return FALSE;
     }
 
@@ -499,14 +516,12 @@ parse_key_value (BusDesktopFileParser *parser, DBusError *error)
   if (p < line_end && _dbus_string_get_byte (&parser->data, p) != '=')
     {
       report_error (parser, "Invalid characters in key name", BUS_DESKTOP_PARSE_ERROR_INVALID_CHARS, error);
-      parser_free (parser);
       return FALSE;
     }
 
   if (p == line_end)
     {
       report_error (parser, "No '=' in key/value pair", BUS_DESKTOP_PARSE_ERROR_INVALID_SYNTAX, error);
-      parser_free (parser);
       return FALSE;
     }
 
@@ -522,7 +537,6 @@ parse_key_value (BusDesktopFileParser *parser, DBusError *error)
   value = unescape_string (parser, &parser->data, value_start, line_end, error);
   if (value == NULL)
     {
-      parser_free (parser);
       return FALSE;
     }
 
@@ -530,7 +544,6 @@ parse_key_value (BusDesktopFileParser *parser, DBusError *error)
   if (line == NULL)
     {
       dbus_free (value);
-      parser_free (parser);
       BUS_SET_OOM (error);
       return FALSE;
     }
@@ -538,7 +551,6 @@ parse_key_value (BusDesktopFileParser *parser, DBusError *error)
   if (!_dbus_string_init (&key))
     {
       dbus_free (value);
-      parser_free (parser);
       BUS_SET_OOM (error);
       return FALSE;
     }
@@ -548,7 +560,6 @@ parse_key_value (BusDesktopFileParser *parser, DBusError *error)
     {
       _dbus_string_free (&key);
       dbus_free (value);
-      parser_free (parser);
       BUS_SET_OOM (error);
       return FALSE;
     }
@@ -557,7 +568,6 @@ parse_key_value (BusDesktopFileParser *parser, DBusError *error)
     {
       _dbus_string_free (&key);
       dbus_free (value);
-      parser_free (parser);
       BUS_SET_OOM (error);
       return FALSE;
     }
@@ -579,7 +589,7 @@ parse_key_value (BusDesktopFileParser *parser, DBusError *error)
 
 static void
 report_error (BusDesktopFileParser *parser,
-	      char                 *message,
+	      const char           *message,
 	      const char           *error_name,
 	      DBusError            *error)
 {
@@ -598,34 +608,13 @@ report_error (BusDesktopFileParser *parser,
                     "Error at line %d: %s\n", parser->line_num, message);
 }
 
-#if 0
-static void
-dump_desktop_file (BusDesktopFile *file)
-{
-  int i;
-
-  for (i = 0; i < file->n_sections; i++)
-    {
-      int j;
-      
-      printf ("[%s]\n", file->sections[i].section_name);
-
-      for (j = 0; j < file->sections[i].n_lines; j++)
-	{
-	  printf ("%s=%s\n", file->sections[i].lines[j].key,
-		  file->sections[i].lines[j].value);
-	}
-    }
-}
-#endif
-
 BusDesktopFile*
 bus_desktop_file_load (DBusString *filename,
 		       DBusError  *error)
 {
-  DBusString str;
-  BusDesktopFileParser parser;
+  BusDesktopFileParser parser = BUS_DESKTOP_FILE_PARSER_INIT;
   DBusStat sb;
+  BusDesktopFile *result = NULL;
 
   _DBUS_ASSERT_ERROR_IS_CLEAR (error);
   
@@ -633,44 +622,39 @@ bus_desktop_file_load (DBusString *filename,
    * that we do something silly, we still handle doing it below.
    */
   if (!_dbus_stat (filename, &sb, error))
-    return NULL;
+    goto out;
 
   if (sb.size > _DBUS_ONE_KILOBYTE * 128)
     {
       dbus_set_error (error, DBUS_ERROR_FAILED,
                       "Desktop file size (%ld bytes) is too large", (long) sb.size);
-      return NULL;
+      goto out;
     }
   
-  if (!_dbus_string_init (&str))
+  if (!_dbus_string_init (&parser.data))
     {
       BUS_SET_OOM (error);
-      return NULL;
-    }
-  
-  if (!_dbus_file_get_contents (&str, filename, error))
-    {
-      _dbus_string_free (&str);
-      return NULL;
+      goto out;
     }
 
-  if (!_dbus_string_validate_utf8 (&str, 0, _dbus_string_get_length (&str)))
+  if (!_dbus_file_get_contents (&parser.data, filename, error))
+    goto out;
+
+  if (!_dbus_string_validate_utf8 (&parser.data, 0,
+                                   _dbus_string_get_length (&parser.data)))
     {
-      _dbus_string_free (&str);
       dbus_set_error (error, DBUS_ERROR_FAILED,
                       "invalid UTF-8");   
-      return NULL;
+      goto out;
     }
   
   parser.desktop_file = dbus_new0 (BusDesktopFile, 1);
   if (parser.desktop_file == NULL)
     {
-      _dbus_string_free (&str);
       BUS_SET_OOM (error);
-      return NULL;
+      goto out;
     }
   
-  parser.data = str;
   parser.line_num = 1;
   parser.pos = 0;
   parser.len = _dbus_string_get_length (&parser.data);
@@ -681,25 +665,32 @@ bus_desktop_file_load (DBusString *filename,
       if (_dbus_string_get_byte (&parser.data, parser.pos) == '[')
 	{
 	  if (!parse_section_start (&parser, error))
-            {
-              return NULL;
-            }
+            goto out;
 	}
       else if (is_blank_line (&parser) ||
 	       _dbus_string_get_byte (&parser.data, parser.pos) == '#')
 	parse_comment_or_blank (&parser);
+      else if (parser.current_section < 0)
+	{
+           dbus_set_error(error, DBUS_ERROR_FAILED,
+                          "invalid service file: key=value before [Section]");
+           goto out;
+	}
       else
 	{
 	  if (!parse_key_value (&parser, error))
-            {
-              return NULL;
-            }
+            goto out;
 	}
     }
 
-  _dbus_string_free (&parser.data);
+  /* Transfer ownership on success */
+  result = parser.desktop_file;
+  parser.desktop_file = NULL;
 
-  return parser.desktop_file;
+out:
+  _DBUS_ASSERT_ERROR_XOR_BOOL (error, result != NULL);
+  parser_clear (&parser);
+  return result;
 }
 
 static BusDesktopFileSection *
