@@ -28,6 +28,7 @@
 #include <dbus/dbus-userdb.h>
 #endif
 #include "selinux.h"
+#include "audit.h"
 #include "services.h"
 #include "policy.h"
 #include "utils.h"
@@ -44,15 +45,13 @@
 #include <syslog.h>
 #include <selinux/selinux.h>
 #include <selinux/avc.h>
-#include <selinux/av_permissions.h>
-#include <selinux/flask.h>
 #include <signal.h>
 #include <stdarg.h>
 #include <stdio.h>
 #include <grp.h>
+#include <dbus/dbus-watch.h>
 #endif /* HAVE_SELINUX */
 #ifdef HAVE_LIBAUDIT
-#include <cap-ng.h>
 #include <libaudit.h>
 #endif /* HAVE_LIBAUDIT */
 
@@ -66,45 +65,20 @@ static dbus_bool_t selinux_enabled = FALSE;
 /* Store an avc_entry_ref to speed AVC decisions. */
 static struct avc_entry_ref aeref;
 
+/* Store the avc netlink fd. */
+static int avc_netlink_fd = -1;
+
+/* Watch to listen for SELinux status changes via netlink. */
+static DBusWatch *avc_netlink_watch_obj = NULL;
+static DBusLoop *avc_netlink_loop_obj = NULL;
+
 /* Store the SID of the bus itself to use as the default. */
 static security_id_t bus_sid = SECSID_WILD;
 
-/* Thread to listen for SELinux status changes via netlink. */
-static pthread_t avc_notify_thread;
-
 /* Prototypes for AVC callback functions.  */
-static void log_callback (const char *fmt, ...);
-static void log_audit_callback (void *data, security_class_t class, char *buf, size_t bufleft);
-static void *avc_create_thread (void (*run) (void));
-static void avc_stop_thread (void *thread);
-static void *avc_alloc_lock (void);
-static void avc_get_lock (void *lock);
-static void avc_release_lock (void *lock);
-static void avc_free_lock (void *lock);
+static int log_callback (int type, const char *fmt, ...) _DBUS_GNUC_PRINTF (2, 3);
+static int log_audit_callback (void *data, security_class_t class, char *buf, size_t bufleft);
 
-/* AVC callback structures for use in avc_init.  */
-static const struct avc_memory_callback mem_cb =
-{
-  .func_malloc = dbus_malloc,
-  .func_free = dbus_free
-};
-static const struct avc_log_callback log_cb =
-{
-  .func_log = log_callback,
-  .func_audit = log_audit_callback
-};
-static const struct avc_thread_callback thread_cb =
-{
-  .func_create_thread = avc_create_thread,
-  .func_stop_thread = avc_stop_thread
-};
-static const struct avc_lock_callback lock_cb =
-{
-  .func_alloc_lock = avc_alloc_lock,
-  .func_get_lock = avc_get_lock,
-  .func_release_lock = avc_release_lock,
-  .func_free_lock = avc_free_lock
-};
 #endif /* HAVE_SELINUX */
 
 /**
@@ -117,55 +91,43 @@ static const struct avc_lock_callback lock_cb =
  */
 #ifdef HAVE_SELINUX
 
-#ifdef HAVE_LIBAUDIT
-static int audit_fd = -1;
-#endif
-
-void
-bus_selinux_audit_init(void)
-{
-#ifdef HAVE_LIBAUDIT  
-  audit_fd = audit_open ();
-
-  if (audit_fd < 0)
-    {
-      /* If kernel doesn't support audit, bail out */
-      if (errno == EINVAL || errno == EPROTONOSUPPORT || errno == EAFNOSUPPORT)
-        return;
-      /* If user bus, bail out */
-      if (errno == EPERM && getuid() != 0)
-        return;
-      _dbus_warn ("Failed opening connection to the audit subsystem");
-    }
-#endif /* HAVE_LIBAUDIT */
-}
-
-static void 
-log_callback (const char *fmt, ...) 
+static int
+log_callback (int type, const char *fmt, ...)
 {
   va_list ap;
+#ifdef HAVE_LIBAUDIT
+  int audit_fd;
+#endif
 
   va_start(ap, fmt);
 
 #ifdef HAVE_LIBAUDIT
+  audit_fd = bus_audit_get_fd ();
+
   if (audit_fd >= 0)
   {
-    capng_get_caps_process();
-    if (capng_have_capability(CAPNG_EFFECTIVE, CAP_AUDIT_WRITE))
-    {
-      char buf[PATH_MAX*2];
-    
-      /* FIXME: need to change this to show real user */
-      vsnprintf(buf, sizeof(buf), fmt, ap);
-      audit_log_user_avc_message(audit_fd, AUDIT_USER_AVC, buf, NULL, NULL,
-                               NULL, getuid());
-      return;
-    }
+    /* This should really be a DBusString, but DBusString allocates
+     * memory dynamically; before switching it, we need to check with
+     * SELinux people that it would be OK for this to fall back to
+     * syslog if OOM, like the equivalent AppArmor code does. */
+    char buf[PATH_MAX*2];
+
+    /* FIXME: need to change this to show real user */
+    vsnprintf(buf, sizeof(buf), fmt, ap);
+    audit_log_user_avc_message(audit_fd, AUDIT_USER_AVC, buf, NULL, NULL,
+                             NULL, getuid());
+    goto out;
   }
 #endif /* HAVE_LIBAUDIT */
-  
-  vsyslog (LOG_INFO, fmt, ap);
+
+  vsyslog (LOG_USER | LOG_INFO, fmt, ap);
+
+#ifdef HAVE_LIBAUDIT
+out:
+#endif
   va_end(ap);
+
+  return 0;
 }
 
 /**
@@ -173,20 +135,16 @@ log_callback (const char *fmt, ...)
  * this could have changed.  Send a SIGHUP to reload all configs.
  */
 static int
-policy_reload_callback (u_int32_t event, security_id_t ssid, 
-                        security_id_t tsid, security_class_t tclass, 
-                        access_vector_t perms, access_vector_t *out_retained)
+policy_reload_callback (int seqno)
 {
-  if (event == AVC_CALLBACK_RESET)
-    return raise (SIGHUP);
-  
-  return 0;
+  _dbus_verbose ("SELinux policy reload callback called, sending SIGHUP\n");
+  return raise (SIGHUP);
 }
 
 /**
  * Log any auxiliary data 
  */
-static void
+static int
 log_audit_callback (void *data, security_class_t class, char *buf, size_t bufleft)
 {
   DBusString *audmsg = data;
@@ -204,73 +162,20 @@ log_audit_callback (void *data, security_class_t class, char *buf, size_t buflef
       if (bufleft > (size_t) _dbus_string_get_length(&s))
         _dbus_string_copy_to_buffer_with_nul (&s, buf, bufleft);
     }
+
+  return 0;
 }
 
-/**
- * Create thread to notify the AVC of enforcing and policy reload
- * changes via netlink.
- *
- * @param run the thread run function
- * @return pointer to the thread
- */
-static void *
-avc_create_thread (void (*run) (void))
+static dbus_bool_t
+handle_avc_netlink_watch (DBusWatch *passed_watch, unsigned int flags, void *data)
 {
-  int rc;
-
-  rc = pthread_create (&avc_notify_thread, NULL, (void *(*) (void *)) run, NULL);
-  if (rc != 0)
+  if (avc_netlink_check_nb () < 0)
     {
-      _dbus_warn ("Failed to start AVC thread: %s\n", _dbus_strerror (rc));
-      exit (1);
+      _dbus_warn ("Failed to check the netlink socket for pending messages and process them: %s", _dbus_strerror (errno));
+      return FALSE;
     }
-  return &avc_notify_thread;
-}
 
-/* Stop AVC netlink thread.  */
-static void
-avc_stop_thread (void *thread)
-{
-  pthread_cancel (*(pthread_t *) thread);
-}
-
-/* Allocate a new AVC lock.  */
-static void *
-avc_alloc_lock (void)
-{
-  pthread_mutex_t *avc_mutex;
-
-  avc_mutex = dbus_new (pthread_mutex_t, 1);
-  if (avc_mutex == NULL)
-    {
-      _dbus_warn ("Could not create mutex: %s\n", _dbus_strerror (errno));
-      exit (1);
-    }
-  pthread_mutex_init (avc_mutex, NULL);
-
-  return avc_mutex;
-}
-
-/* Acquire an AVC lock.  */
-static void
-avc_get_lock (void *lock)
-{
-  pthread_mutex_lock (lock);
-}
-
-/* Release an AVC lock.  */
-static void
-avc_release_lock (void *lock)
-{
-  pthread_mutex_unlock (lock);
-}
-
-/* Free an AVC lock.  */
-static void
-avc_free_lock (void *lock)
-{
-  pthread_mutex_destroy (lock);
-  dbus_free (lock);
+  return TRUE;
 }
 #endif /* HAVE_SELINUX */
 
@@ -288,6 +193,19 @@ bus_selinux_enabled (void)
 #endif /* HAVE_SELINUX */
 }
 
+BusSELinuxID*
+bus_selinux_get_self (void)
+{
+#ifdef HAVE_SELINUX
+  if(bus_selinux_enabled ())
+    return BUS_SID_FROM_SELINUX (bus_sid);
+  else
+    return NULL;
+#else
+  return NULL;
+#endif /* HAVE_SELINUX */
+}
+
 /**
  * Do early initialization; determine whether SELinux is enabled.
  */
@@ -302,7 +220,7 @@ bus_selinux_pre_init (void)
   r = is_selinux_enabled ();
   if (r < 0)
     {
-      _dbus_warn ("Could not tell if SELinux is enabled: %s\n",
+      _dbus_warn ("Could not tell if SELinux is enabled: %s",
                   _dbus_strerror (errno));
       return FALSE;
     }
@@ -314,12 +232,31 @@ bus_selinux_pre_init (void)
 #endif
 }
 
+/*
+ * Private Flask definitions; the order of these constants must
+ * exactly match that of the structure array below!
+ */
+/* security dbus class constants */
+#define SECCLASS_DBUS       1
+
+/* dbus's per access vector constants */
+#define DBUS__ACQUIRE_SVC   1
+#define DBUS__SEND_MSG      2
+
+#ifdef HAVE_SELINUX
+static struct security_class_mapping dbus_map[] = {
+  { "dbus", { "acquire_svc", "send_msg", NULL } },
+  { NULL }
+};
+#endif /* HAVE_SELINUX */
+
 /**
- * Initialize the user space access vector cache (AVC) for D-Bus and set up
+ * Establish dynamic object class and permission mapping and
+ * initialize the user space access vector cache (AVC) for D-Bus and set up
  * logging callbacks.
  */
 dbus_bool_t
-bus_selinux_full_init (void)
+bus_selinux_full_init (BusContext *context, DBusError *error)
 {
 #ifdef HAVE_SELINUX
   char *bus_context;
@@ -334,82 +271,107 @@ bus_selinux_full_init (void)
 
   _dbus_verbose ("SELinux is enabled in this kernel.\n");
 
-  avc_entry_ref_init (&aeref);
-  if (avc_init ("avc", &mem_cb, &log_cb, &thread_cb, &lock_cb) < 0)
+  if (selinux_set_mapping (dbus_map) < 0)
     {
-      _dbus_warn ("Failed to start Access Vector Cache (AVC).\n");
+      _dbus_warn ("Failed to set up security class mapping (selinux_set_mapping():%s).",
+                   strerror (errno));
+      return FALSE; 
+    }
+
+  avc_entry_ref_init (&aeref);
+  if (avc_open (NULL, 0) < 0)
+    {
+      dbus_set_error (error, DBUS_ERROR_FAILED,
+                      "Failed to start Access Vector Cache (AVC): %s",
+                      _dbus_strerror (errno));
       return FALSE;
     }
   else
     {
-      openlog ("dbus", LOG_PERROR, LOG_USER);
       _dbus_verbose ("Access Vector Cache (AVC) started.\n");
     }
 
-  if (avc_add_callback (policy_reload_callback, AVC_CALLBACK_RESET,
-                       NULL, NULL, 0, 0) < 0)
+  avc_netlink_fd = avc_netlink_acquire_fd ();
+  if (avc_netlink_fd < 0)
     {
-      _dbus_warn ("Failed to add policy reload callback: %s\n",
-                  _dbus_strerror (errno));
-      avc_destroy ();
-      return FALSE;
+       dbus_set_error (error, DBUS_ERROR_FAILED,
+                      "Cannot acquire AVC netlink fd: %s",
+                      _dbus_strerror (errno));
+       goto error;
     }
 
+  _dbus_fd_set_close_on_exec (avc_netlink_fd);
+
+  avc_netlink_loop_obj = bus_context_get_loop (context);
+  /* avc_netlink_loop_obj is a global variable */
+  _dbus_loop_ref (avc_netlink_loop_obj);
+
+  avc_netlink_watch_obj = _dbus_watch_new (avc_netlink_fd, DBUS_WATCH_READABLE, TRUE,
+                                           handle_avc_netlink_watch, NULL, NULL);
+  if (avc_netlink_watch_obj == NULL)
+    {
+      BUS_SET_OOM (error);
+      goto error;
+    }
+
+  if (!_dbus_loop_add_watch (avc_netlink_loop_obj, avc_netlink_watch_obj))
+    {
+      _dbus_watch_invalidate (avc_netlink_watch_obj);
+      _dbus_clear_watch (&avc_netlink_watch_obj);
+      avc_netlink_watch_obj = NULL;
+      BUS_SET_OOM (error);
+      goto error;
+    }
+
+  selinux_set_callback (SELINUX_CB_POLICYLOAD, (union selinux_callback) policy_reload_callback);
+  selinux_set_callback (SELINUX_CB_AUDIT, (union selinux_callback) log_audit_callback);
+  selinux_set_callback (SELINUX_CB_LOG, (union selinux_callback) log_callback);
+
   bus_context = NULL;
   bus_sid = SECSID_WILD;
 
   if (getcon (&bus_context) < 0)
     {
-      _dbus_verbose ("Error getting context of bus: %s\n",
-                     _dbus_strerror (errno));
-      return FALSE;
+      dbus_set_error (error, DBUS_ERROR_FAILED,
+                      "Error getting context of bus: %s",
+                      _dbus_strerror (errno));
+      goto error;
     }
       
   if (avc_context_to_sid (bus_context, &bus_sid) < 0)
     {
-      _dbus_verbose ("Error getting SID from bus context: %s\n",
-                     _dbus_strerror (errno));
+      dbus_set_error (error, DBUS_ERROR_FAILED,
+                      "Error getting SID from bus context: %s",
+                      _dbus_strerror (errno));
       freecon (bus_context);
-      return FALSE;
+      goto error;
     }
 
   freecon (bus_context);
+
+  return TRUE;
+
+error:
+  if (avc_netlink_watch_obj)
+    {
+      _dbus_loop_remove_watch (avc_netlink_loop_obj, avc_netlink_watch_obj);
+      _dbus_watch_invalidate (avc_netlink_watch_obj);
+      _dbus_clear_watch (&avc_netlink_watch_obj);
+    }
+  _dbus_clear_loop (&avc_netlink_loop_obj);
+  if (avc_netlink_fd >= 0)
+    {
+      avc_netlink_release_fd ();
+      avc_netlink_fd = -1;
+    }
+  avc_destroy ();
+  _DBUS_ASSERT_ERROR_IS_SET (error);
+  return FALSE;
   
 #endif /* HAVE_SELINUX */
   return TRUE;
 }
 
-/**
- * Decrement SID reference count.
- * 
- * @param sid the SID to decrement
- */
-void
-bus_selinux_id_unref (BusSELinuxID *sid)
-{
-#ifdef HAVE_SELINUX
-  if (!selinux_enabled)
-    return;
-
-  _dbus_assert (sid != NULL);
-  
-  sidput (SELINUX_SID_FROM_BUS (sid));
-#endif /* HAVE_SELINUX */
-}
-
-void
-bus_selinux_id_ref (BusSELinuxID *sid)
-{
-#ifdef HAVE_SELINUX
-  if (!selinux_enabled)
-    return;
-
-  _dbus_assert (sid != NULL);
-  
-  sidget (SELINUX_SID_FROM_BUS (sid));
-#endif /* HAVE_SELINUX */
-}
-
 /**
  * Determine if the SELinux security policy allows the given sender
  * security context to go to the given recipient security context.
@@ -441,7 +403,7 @@ bus_selinux_check (BusSELinuxID        *sender_sid,
   if (avc_has_perm (SELINUX_SID_FROM_BUS (sender_sid),
                     override_sid ?
                     SELINUX_SID_FROM_BUS (override_sid) :
-                    SELINUX_SID_FROM_BUS (bus_sid), 
+                    bus_sid,
                     target_class, requested, &aeref, auxdata) < 0)
     {
     switch (errno)
@@ -544,6 +506,7 @@ bus_selinux_allows_send (DBusConnection     *sender,
 			 const char         *member,
 			 const char         *error_name,
 			 const char         *destination,
+			 BusActivationEntry *activation_entry,
 			 DBusError          *error)
 {
 #ifdef HAVE_SELINUX
@@ -557,6 +520,10 @@ bus_selinux_allows_send (DBusConnection     *sender,
   if (!selinux_enabled)
     return TRUE;
 
+  /* We do not mediate activation attempts yet. */
+  if (activation_entry)
+    return TRUE;
+
   if (!sender || !dbus_connection_get_unix_process_id (sender, &spid))
     spid = 0;
   if (!proposed_recipient || !dbus_connection_get_unix_process_id (proposed_recipient, &tpid))
@@ -624,7 +591,8 @@ bus_selinux_allows_send (DBusConnection     *sender,
     }
 
   sender_sid = bus_connection_get_selinux_id (sender);
-  /* A NULL proposed_recipient means the bus itself. */
+
+  /* A NULL proposed_recipient with no activation entry means the bus itself. */
   if (proposed_recipient)
     recipient_sid = bus_connection_get_selinux_id (proposed_recipient);
   else
@@ -760,7 +728,7 @@ bus_selinux_init_connection_id (DBusConnection *connection,
                         "Error getting SID from context \"%s\": %s\n",
 			con, _dbus_strerror (errno));
       
-      _dbus_warn ("Error getting SID from context \"%s\": %s\n",
+      _dbus_warn ("Error getting SID from context \"%s\": %s",
 		  con, _dbus_strerror (errno));
       
       freecon (con);
@@ -774,21 +742,6 @@ bus_selinux_init_connection_id (DBusConnection *connection,
 #endif /* HAVE_SELINUX */
 }
 
-
-/**
- * Function for freeing hash table data.  These SIDs
- * should no longer be referenced.
- */
-static void
-bus_selinux_id_table_free_value (BusSELinuxID *sid)
-{
-#ifdef HAVE_SELINUX
-  /* NULL sometimes due to how DBusHashTable works */
-  if (sid)
-    bus_selinux_id_unref (sid);
-#endif /* HAVE_SELINUX */
-}
-
 /**
  * Creates a new table mapping service names to security ID.
  * A security ID is a "compiled" security context, a security
@@ -800,8 +753,7 @@ DBusHashTable*
 bus_selinux_id_table_new (void)
 {
   return _dbus_hash_table_new (DBUS_HASH_STRING,
-                               (DBusFreeFunction) dbus_free,
-                               (DBusFreeFunction) bus_selinux_id_table_free_value);
+                               (DBusFreeFunction) dbus_free, NULL);
 }
 
 /** 
@@ -841,7 +793,7 @@ bus_selinux_id_table_insert (DBusHashTable *service_table,
           return FALSE;
 	}
 
-      _dbus_warn ("Error getting SID from context \"%s\": %s\n",
+      _dbus_warn ("Error getting SID from context \"%s\": %s",
 		  (char *) service_context,
                   _dbus_strerror (errno));
       goto out;
@@ -863,9 +815,6 @@ bus_selinux_id_table_insert (DBusHashTable *service_table,
   retval = TRUE;
   
  out:
-  if (sid != SECSID_WILD)
-    sidput (sid);
-
   if (key)
     dbus_free (key);
 
@@ -937,8 +886,7 @@ bus_selinux_get_policy_root (void)
 void
 bus_selinux_id_table_print (DBusHashTable *service_table)
 {
-#ifdef DBUS_ENABLE_VERBOSE_MODE
-#ifdef HAVE_SELINUX
+#if defined (DBUS_ENABLE_VERBOSE_MODE) && defined (HAVE_SELINUX)
   DBusHashIter iter;
 
   if (!selinux_enabled)
@@ -954,19 +902,18 @@ bus_selinux_id_table_print (DBusHashTable *service_table)
       _dbus_verbose ("The context is %s\n", sid->ctx);
       _dbus_verbose ("The refcount is %d\n", sid->refcnt);
     }
-#endif /* HAVE_SELINUX */
-#endif /* DBUS_ENABLE_VERBOSE_MODE */
+#endif /* DBUS_ENABLE_VERBOSE_MODE && HAVE_SELINUX */
 }
 
 
-#ifdef DBUS_ENABLE_VERBOSE_MODE
-#ifdef HAVE_SELINUX
 /**
  * Print out some AVC statistics.
  */
+#ifdef HAVE_SELINUX
 static void
 bus_avc_print_stats (void)
 {
+#ifdef DBUS_ENABLE_VERBOSE_MODE
   struct avc_cache_stats cstats;
 
   if (!selinux_enabled)
@@ -984,10 +931,9 @@ bus_avc_print_stats (void)
   _dbus_verbose ("CAV hits: %d\n", cstats.cav_hits);
   _dbus_verbose ("CAV probes: %d\n", cstats.cav_probes);
   _dbus_verbose ("CAV misses: %d\n", cstats.cav_misses);
+#endif /* DBUS_ENABLE_VERBOSE_MODE */
 }
 #endif /* HAVE_SELINUX */
-#endif /* DBUS_ENABLE_VERBOSE_MODE */
-
 
 /**
  * Destroy the AVC before we terminate.
@@ -1001,94 +947,27 @@ bus_selinux_shutdown (void)
 
   _dbus_verbose ("AVC shutdown\n");
 
+  if (avc_netlink_watch_obj)
+    {
+      _dbus_loop_remove_watch (avc_netlink_loop_obj, avc_netlink_watch_obj);
+      _dbus_watch_invalidate (avc_netlink_watch_obj);
+      _dbus_clear_watch (&avc_netlink_watch_obj);
+    }
+  _dbus_clear_loop (&avc_netlink_loop_obj);
+
+  if (avc_netlink_fd >= 0)
+    {
+      avc_netlink_release_fd ();
+      avc_netlink_fd = -1;
+    }
+
   if (bus_sid != SECSID_WILD)
     {
-      sidput (bus_sid);
       bus_sid = SECSID_WILD;
 
-#ifdef DBUS_ENABLE_VERBOSE_MODE
- 
-      if (_dbus_is_verbose()) 
-        bus_avc_print_stats ();
- 
-#endif /* DBUS_ENABLE_VERBOSE_MODE */
+      bus_avc_print_stats ();
 
       avc_destroy ();
-#ifdef HAVE_LIBAUDIT
-      audit_close (audit_fd);
-#endif /* HAVE_LIBAUDIT */
     }
 #endif /* HAVE_SELINUX */
 }
-
-/* The !HAVE_LIBAUDIT case lives in dbus-sysdeps-util-unix.c */
-#ifdef HAVE_LIBAUDIT
-/**
- * Changes the user and group the bus is running as.
- *
- * @param user the user to become
- * @param error return location for errors
- * @returns #FALSE on failure
- */
-dbus_bool_t
-_dbus_change_to_daemon_user  (const char    *user,
-                              DBusError     *error)
-{
-  dbus_uid_t uid;
-  dbus_gid_t gid;
-  DBusString u;
-
-  _dbus_string_init_const (&u, user);
-
-  if (!_dbus_get_user_id_and_primary_group (&u, &uid, &gid))
-    {
-      dbus_set_error (error, DBUS_ERROR_FAILED,
-                      "User '%s' does not appear to exist?",
-                      user);
-      return FALSE;
-    }
-
-  /* If we were root */
-  if (_dbus_geteuid () == 0)
-    {
-      int rc;
-
-      capng_clear (CAPNG_SELECT_BOTH);
-      capng_update (CAPNG_ADD, CAPNG_EFFECTIVE | CAPNG_PERMITTED,
-                    CAP_AUDIT_WRITE);
-      rc = capng_change_id (uid, gid, 0);
-      if (rc)
-        {
-          switch (rc) {
-            default:
-              dbus_set_error (error, DBUS_ERROR_FAILED,
-                              "Failed to drop capabilities: %s\n",
-                              _dbus_strerror (errno));
-              break;
-            case -4:
-              dbus_set_error (error, _dbus_error_from_errno (errno),
-                              "Failed to set GID to %lu: %s", gid,
-                              _dbus_strerror (errno));
-              break;
-            case -5:
-              _dbus_warn ("Failed to drop supplementary groups: %s\n",
-                          _dbus_strerror (errno));
-              break;
-            case -6:
-              dbus_set_error (error, _dbus_error_from_errno (errno),
-                              "Failed to set UID to %lu: %s", uid,
-                              _dbus_strerror (errno));
-              break;
-            case -7:
-              dbus_set_error (error, _dbus_error_from_errno (errno),
-                              "Failed to unset keep-capabilities: %s\n",
-                              _dbus_strerror (errno));
-              break;
-          }
-          return FALSE;
-        }
-    }
-
- return TRUE;
-}
-#endif
