@@ -21,6 +21,7 @@
  *
  */
 #include <config.h>
+#include <stdlib.h>
 #include <string.h>
 #include "dbus-credentials.h"
 #include "dbus-internals.h"
@@ -48,8 +49,11 @@
 struct DBusCredentials {
   int refcount;
   dbus_uid_t unix_uid;
-  dbus_pid_t unix_pid;
+  dbus_gid_t *unix_gids;
+  size_t n_unix_gids;
+  dbus_pid_t pid;
   char *windows_sid;
+  char *linux_security_label;
   void *adt_audit_data;
   dbus_int32_t adt_audit_data_size;
 };
@@ -77,8 +81,11 @@ _dbus_credentials_new (void)
   
   creds->refcount = 1;
   creds->unix_uid = DBUS_UID_UNSET;
-  creds->unix_pid = DBUS_PID_UNSET;
+  creds->unix_gids = NULL;
+  creds->n_unix_gids = 0;
+  creds->pid = DBUS_PID_UNSET;
   creds->windows_sid = NULL;
+  creds->linux_security_label = NULL;
   creds->adt_audit_data = NULL;
   creds->adt_audit_data_size = 0;
 
@@ -86,7 +93,7 @@ _dbus_credentials_new (void)
 }
 
 /**
- * Creates a new object with credentials (user ID and process ID) from the current process.
+ * Creates a new object with the most important credentials (user ID and process ID) from the current process.
  * @returns the new object or #NULL if no memory
  */
 DBusCredentials*
@@ -132,7 +139,9 @@ _dbus_credentials_unref (DBusCredentials    *credentials)
   credentials->refcount -= 1;
   if (credentials->refcount == 0)
     {
+      dbus_free (credentials->unix_gids);
       dbus_free (credentials->windows_sid);
+      dbus_free (credentials->linux_security_label);
       dbus_free (credentials->adt_audit_data);
       dbus_free (credentials);
     }
@@ -146,10 +155,10 @@ _dbus_credentials_unref (DBusCredentials    *credentials)
  * @returns #FALSE if no memory
  */
 dbus_bool_t
-_dbus_credentials_add_unix_pid (DBusCredentials    *credentials,
-                                dbus_pid_t          pid)
+_dbus_credentials_add_pid (DBusCredentials    *credentials,
+                           dbus_pid_t          pid)
 {
-  credentials->unix_pid = pid;
+  credentials->pid = pid;
   return TRUE;
 }
 
@@ -169,6 +178,63 @@ _dbus_credentials_add_unix_uid(DBusCredentials    *credentials,
 
 }
 
+static int
+cmp_gidp (const void *a_, const void *b_)
+{
+  const dbus_gid_t *a = a_;
+  const dbus_gid_t *b = b_;
+
+  if (*a < *b)
+    return -1;
+
+  if (*a > *b)
+    return 1;
+
+  return 0;
+}
+
+/**
+ * Add UNIX group IDs to the credentials, replacing any group IDs that
+ * might already have been present.
+ *
+ * @param credentials the object
+ * @param gids the group IDs, which will be freed by the DBusCredentials object
+ * @param n_gids the number of group IDs
+ */
+void
+_dbus_credentials_take_unix_gids (DBusCredentials *credentials,
+                                  dbus_gid_t      *gids,
+                                  size_t           n_gids)
+{
+  /* So we can compare arrays via a simple memcmp */
+  qsort (gids, n_gids, sizeof (dbus_gid_t), cmp_gidp);
+
+  dbus_free (credentials->unix_gids);
+  credentials->unix_gids = gids;
+  credentials->n_unix_gids = n_gids;
+}
+
+/**
+ * Get the Unix group IDs.
+ *
+ * @param credentials the object
+ * @param gids the group IDs, which will be freed by the DBusCredentials object
+ * @param n_gids the number of group IDs
+ */
+dbus_bool_t
+_dbus_credentials_get_unix_gids (DBusCredentials   *credentials,
+                                 const dbus_gid_t **gids,
+                                 size_t            *n_gids)
+{
+  if (gids != NULL)
+    *gids = credentials->unix_gids;
+
+  if (n_gids != NULL)
+    *n_gids = credentials->n_unix_gids;
+
+  return (credentials->unix_gids != NULL);
+}
+
 /**
  * Add a Windows user SID to the credentials.
  *
@@ -192,6 +258,30 @@ _dbus_credentials_add_windows_sid (DBusCredentials    *credentials,
   return TRUE;
 }
 
+/**
+ * Add a Linux security label, as used by LSMs such as SELinux, Smack and
+ * AppArmor, to the credentials.
+ *
+ * @param credentials the object
+ * @param label the label
+ * @returns #FALSE if no memory
+ */
+dbus_bool_t
+_dbus_credentials_add_linux_security_label (DBusCredentials    *credentials,
+                                            const char         *label)
+{
+  char *copy;
+
+  copy = _dbus_strdup (label);
+  if (copy == NULL)
+    return FALSE;
+
+  dbus_free (credentials->linux_security_label);
+  credentials->linux_security_label = copy;
+
+  return TRUE;
+}
+
 /**
  * Add ADT audit data to the credentials.
  *
@@ -231,17 +321,21 @@ _dbus_credentials_include (DBusCredentials    *credentials,
   switch (type)
     {
     case DBUS_CREDENTIAL_UNIX_PROCESS_ID:
-      return credentials->unix_pid != DBUS_PID_UNSET;
+      return credentials->pid != DBUS_PID_UNSET;
     case DBUS_CREDENTIAL_UNIX_USER_ID:
       return credentials->unix_uid != DBUS_UID_UNSET;
+    case DBUS_CREDENTIAL_UNIX_GROUP_IDS:
+      return credentials->unix_gids != NULL;
     case DBUS_CREDENTIAL_WINDOWS_SID:
       return credentials->windows_sid != NULL;
+    case DBUS_CREDENTIAL_LINUX_SECURITY_LABEL:
+      return credentials->linux_security_label != NULL;
     case DBUS_CREDENTIAL_ADT_AUDIT_DATA_ID:
       return credentials->adt_audit_data != NULL;
+    default:
+      _dbus_assert_not_reached ("Unknown credential enum value");
+      return FALSE;
     }
-
-  _dbus_assert_not_reached ("Unknown credential enum value");
-  return FALSE;
 }
 
 /**
@@ -252,9 +346,9 @@ _dbus_credentials_include (DBusCredentials    *credentials,
  * @returns UNIX process ID
  */
 dbus_pid_t
-_dbus_credentials_get_unix_pid (DBusCredentials    *credentials)
+_dbus_credentials_get_pid (DBusCredentials    *credentials)
 {
-  return credentials->unix_pid;
+  return credentials->pid;
 }
 
 /**
@@ -283,6 +377,19 @@ _dbus_credentials_get_windows_sid (DBusCredentials    *credentials)
   return credentials->windows_sid;
 }
 
+/**
+ * Gets the Linux security label (as used by LSMs) from the credentials,
+ * or #NULL if the credentials object doesn't contain a security label.
+ *
+ * @param credentials the object
+ * @returns the security label
+ */
+const char *
+_dbus_credentials_get_linux_security_label (DBusCredentials *credentials)
+{
+  return credentials->linux_security_label;
+}
+
 /**
  * Gets the ADT audit data in the credentials, or #NULL if
  * the credentials object doesn't contain ADT audit data.
@@ -322,13 +429,21 @@ _dbus_credentials_are_superset (DBusCredentials    *credentials,
                                 DBusCredentials    *possible_subset)
 {
   return
-    (possible_subset->unix_pid == DBUS_PID_UNSET ||
-     possible_subset->unix_pid == credentials->unix_pid) &&
+    (possible_subset->pid == DBUS_PID_UNSET ||
+     possible_subset->pid == credentials->pid) &&
     (possible_subset->unix_uid == DBUS_UID_UNSET ||
      possible_subset->unix_uid == credentials->unix_uid) &&
+    (possible_subset->unix_gids == NULL ||
+     (possible_subset->n_unix_gids == credentials->n_unix_gids &&
+      memcmp (possible_subset->unix_gids, credentials->unix_gids,
+              sizeof (dbus_gid_t) * credentials->n_unix_gids) == 0)) &&
     (possible_subset->windows_sid == NULL ||
      (credentials->windows_sid && strcmp (possible_subset->windows_sid,
                                           credentials->windows_sid) == 0)) &&
+    (possible_subset->linux_security_label == NULL ||
+     (credentials->linux_security_label != NULL &&
+      strcmp (possible_subset->linux_security_label,
+              credentials->linux_security_label) == 0)) &&
     (possible_subset->adt_audit_data == NULL ||
      (credentials->adt_audit_data && memcmp (possible_subset->adt_audit_data,
                                              credentials->adt_audit_data,
@@ -345,9 +460,12 @@ dbus_bool_t
 _dbus_credentials_are_empty (DBusCredentials    *credentials)
 {
   return
-    credentials->unix_pid == DBUS_PID_UNSET &&
+    credentials->pid == DBUS_PID_UNSET &&
     credentials->unix_uid == DBUS_UID_UNSET &&
+    credentials->unix_gids == NULL &&
+    credentials->n_unix_gids == 0 &&
     credentials->windows_sid == NULL &&
+    credentials->linux_security_label == NULL &&
     credentials->adt_audit_data == NULL;
 }
 
@@ -384,9 +502,15 @@ _dbus_credentials_add_credentials (DBusCredentials    *credentials,
     _dbus_credentials_add_credential (credentials,
                                       DBUS_CREDENTIAL_UNIX_USER_ID,
                                       other_credentials) &&
+    _dbus_credentials_add_credential (credentials,
+                                      DBUS_CREDENTIAL_UNIX_GROUP_IDS,
+                                      other_credentials) &&
     _dbus_credentials_add_credential (credentials,
                                       DBUS_CREDENTIAL_ADT_AUDIT_DATA_ID,
                                       other_credentials) &&
+    _dbus_credentials_add_credential (credentials,
+                                      DBUS_CREDENTIAL_LINUX_SECURITY_LABEL,
+                                      other_credentials) &&
     _dbus_credentials_add_credential (credentials,
                                       DBUS_CREDENTIAL_WINDOWS_SID,
                                       other_credentials);
@@ -410,9 +534,9 @@ _dbus_credentials_add_credential (DBusCredentials    *credentials,
                                   DBusCredentials    *other_credentials)
 {
   if (which == DBUS_CREDENTIAL_UNIX_PROCESS_ID &&
-      other_credentials->unix_pid != DBUS_PID_UNSET)
+      other_credentials->pid != DBUS_PID_UNSET)
     {
-      if (!_dbus_credentials_add_unix_pid (credentials, other_credentials->unix_pid))
+      if (!_dbus_credentials_add_pid (credentials, other_credentials->pid))
         return FALSE;
     }
   else if (which == DBUS_CREDENTIAL_UNIX_USER_ID &&
@@ -421,12 +545,35 @@ _dbus_credentials_add_credential (DBusCredentials    *credentials,
       if (!_dbus_credentials_add_unix_uid (credentials, other_credentials->unix_uid))
         return FALSE;
     }
+  else if (which == DBUS_CREDENTIAL_UNIX_GROUP_IDS &&
+           other_credentials->unix_gids != NULL)
+    {
+      dbus_gid_t *gids;
+
+      gids = dbus_new (dbus_gid_t, other_credentials->n_unix_gids);
+
+      if (gids == NULL)
+        return FALSE;
+
+      memcpy (gids, other_credentials->unix_gids,
+              sizeof (dbus_gid_t) * other_credentials->n_unix_gids);
+
+      _dbus_credentials_take_unix_gids (credentials, gids,
+                                        other_credentials->n_unix_gids);
+    }
   else if (which == DBUS_CREDENTIAL_WINDOWS_SID &&
            other_credentials->windows_sid != NULL)
     {
       if (!_dbus_credentials_add_windows_sid (credentials, other_credentials->windows_sid))
         return FALSE;
     } 
+  else if (which == DBUS_CREDENTIAL_LINUX_SECURITY_LABEL &&
+           other_credentials->linux_security_label != NULL)
+    {
+      if (!_dbus_credentials_add_linux_security_label (credentials,
+            other_credentials->linux_security_label))
+        return FALSE;
+    }
   else if (which == DBUS_CREDENTIAL_ADT_AUDIT_DATA_ID &&
            other_credentials->adt_audit_data != NULL) 
     {
@@ -445,10 +592,15 @@ _dbus_credentials_add_credential (DBusCredentials    *credentials,
 void
 _dbus_credentials_clear (DBusCredentials    *credentials)
 {
-  credentials->unix_pid = DBUS_PID_UNSET;
+  credentials->pid = DBUS_PID_UNSET;
   credentials->unix_uid = DBUS_UID_UNSET;
+  dbus_free (credentials->unix_gids);
+  credentials->unix_gids = NULL;
+  credentials->n_unix_gids = 0;
   dbus_free (credentials->windows_sid);
   credentials->windows_sid = NULL;
+  dbus_free (credentials->linux_security_label);
+  credentials->linux_security_label = NULL;
   dbus_free (credentials->adt_audit_data);
   credentials->adt_audit_data = NULL;
   credentials->adt_audit_data_size = 0;
@@ -523,22 +675,43 @@ _dbus_credentials_to_string_append (DBusCredentials    *credentials,
         goto oom;
       join = TRUE;
     }
-  if (credentials->unix_pid != DBUS_PID_UNSET)
+  if (credentials->pid != DBUS_PID_UNSET)
     {
-      if (!_dbus_string_append_printf (string, "%spid=" DBUS_PID_FORMAT, join ? " " : "", credentials->unix_pid))
+      if (!_dbus_string_append_printf (string, "%spid=" DBUS_PID_FORMAT, join ? " " : "", credentials->pid))
         goto oom;
       join = TRUE;
     }
-  else
-    join = FALSE;
+
+  if (credentials->unix_gids != NULL)
+    {
+      size_t i;
+
+      for (i = 0; i < credentials->n_unix_gids; i++)
+        {
+          if (!_dbus_string_append_printf (string, "%sgid=" DBUS_GID_FORMAT,
+                                           join ? " " : "",
+                                           credentials->unix_gids[i]))
+            goto oom;
+
+          join = TRUE;
+        }
+    }
+
   if (credentials->windows_sid != NULL)
     {
       if (!_dbus_string_append_printf (string, "%ssid=%s", join ? " " : "", credentials->windows_sid))
         goto oom;
       join = TRUE;
     }
-  else
-    join = FALSE;
+
+  if (credentials->linux_security_label != NULL)
+    {
+      if (!_dbus_string_append_printf (string, "%slsm='%s'",
+                                       join ? " " : "",
+                                       credentials->linux_security_label))
+        goto oom;
+      join = TRUE;
+    }
 
   return TRUE;
 oom:
