@@ -27,6 +27,7 @@
 #include <dbus/dbus-string.h>
 #include <dbus/dbus-list.h>
 #include <dbus/dbus-sysdeps.h>
+#include <dbus/dbus-test-tap.h>
 
 /**
  * @defgroup DBusKeyring keyring class
@@ -250,7 +251,7 @@ _dbus_keyring_unlock (DBusKeyring *keyring)
 
   if (!_dbus_delete_file (&keyring->filename_lock, &error))
     {
-      _dbus_warn ("Failed to delete lock file: %s\n",
+      _dbus_warn ("Failed to delete lock file: %s",
                   error.message);
       dbus_error_free (&error);
     }
@@ -309,9 +310,7 @@ add_new_key (DBusKey  **keys_p,
 
   s = (const unsigned char*) _dbus_string_get_const_data (&bytes);
       
-  id = s[0] | (s[1] << 8) | (s[2] << 16) | (s[3] << 24);
-  if (id < 0)
-    id = - id;
+  id = s[0] | (s[1] << 8) | (s[2] << 16) | ((s[3] & 0x7f) << 24);
   _dbus_assert (id >= 0);
 
   if (find_key_by_id (keys, n_keys, id) != NULL)
@@ -452,7 +451,7 @@ _dbus_keyring_reload (DBusKeyring *keyring,
   if (!_dbus_string_validate_ascii (&contents, 0,
                                     _dbus_string_get_length (&contents)))
     {
-      _dbus_warn ("Secret keyring file contains non-ASCII! Ignoring existing contents\n");
+      _dbus_warn ("Secret keyring file contains non-ASCII! Ignoring existing contents");
       _dbus_string_set_length (&contents, 0);
     }
 
@@ -621,7 +620,7 @@ _dbus_keyring_reload (DBusKeyring *keyring,
     {
       if (error && error->name)
         _dbus_verbose ("error is %s: %s\n", error->name, error->message);
-      _dbus_warn ("returning %d but error pointer %p name %s\n",
+      _dbus_warn ("returning %d but error pointer %p name %s",
                   retval, error, error->name ? error->name : "(none)");
       _dbus_assert_not_reached ("didn't handle errors properly");
     }
@@ -807,7 +806,7 @@ _dbus_keyring_new_for_credentials (DBusCredentials  *credentials,
    * unless someone else manages to create it
    */
   dbus_error_init (&tmp_error);
-  if (!_dbus_create_directory (&keyring->directory,
+  if (!_dbus_ensure_directory (&keyring->directory,
                                &tmp_error))
     {
       _dbus_verbose ("Creating keyring directory: %s\n",
@@ -1063,9 +1062,9 @@ _dbus_keyring_test (void)
   _dbus_assert (!_dbus_keyring_validate_context (&context));
   
   if (!_dbus_string_init (&context))
-    _dbus_assert_not_reached ("no memory");
+    _dbus_test_fatal ("no memory");
   if (!_dbus_string_append_byte (&context, '\0'))
-    _dbus_assert_not_reached ("no memory");
+    _dbus_test_fatal ("no memory");
   _dbus_assert (!_dbus_keyring_validate_context (&context));
   _dbus_string_free (&context);
 
@@ -1128,7 +1127,7 @@ _dbus_keyring_test (void)
       ++i;
     }
 
-  printf (" %d keys in test\n", ring1->n_keys);
+  _dbus_test_diag (" %d keys in test", ring1->n_keys);
 
   /* Test ref/unref */
   _dbus_keyring_ref (ring1);
