@@ -22,13 +22,14 @@
  */
 
 #include <config.h>
-#ifdef DBUS_BUILD_TESTS
+#ifdef DBUS_ENABLE_EMBEDDED_TESTS
 
 #ifndef DOXYGEN_SHOULD_SKIP_THIS
 
 #include "dbus-internals.h"
 #include "dbus-marshal-validate.h"
 #include "dbus-marshal-recursive.h"
+#include <dbus/dbus-test-tap.h>
 
 #include "dbus-test.h"
 #include <stdio.h>
@@ -56,19 +57,15 @@ run_validity_tests (const ValidityTest *tests,
       v = (*func) (&str, 0, _dbus_string_get_length (&str));
 
       if (v != tests[i].expected)
-        {
-          _dbus_warn ("Improper validation result %d for '%s'\n",
-                      v, tests[i].data);
-          _dbus_assert_not_reached ("test failed");
-        }
-
-      ++i;
+        _dbus_test_fatal ("Improper validation result %d for '%s'", v, tests[i].data);
     }
 }
 
 static const ValidityTest signature_tests[] = {
   { "", DBUS_VALID },
+  { "sss", DBUS_VALID },
   { "i", DBUS_VALID },
+  { "b", DBUS_VALID },
   { "ai", DBUS_VALID },
   { "(i)", DBUS_VALID },
   { "w", DBUS_INVALID_UNKNOWN_TYPECODE },
@@ -83,7 +80,7 @@ static const ValidityTest signature_tests[] = {
     DBUS_INVALID_EXCEEDED_MAXIMUM_STRUCT_RECURSION },
   { ")", DBUS_INVALID_STRUCT_ENDED_BUT_NOT_STARTED },
   { "i)", DBUS_INVALID_STRUCT_ENDED_BUT_NOT_STARTED },
-  { "a)", DBUS_INVALID_STRUCT_ENDED_BUT_NOT_STARTED },
+  { "a)", DBUS_INVALID_MISSING_ARRAY_ELEMENT_TYPE },
   { "(", DBUS_INVALID_STRUCT_STARTED_BUT_NOT_ENDED },
   { "(i", DBUS_INVALID_STRUCT_STARTED_BUT_NOT_ENDED },
   { "(iiiii", DBUS_INVALID_STRUCT_STARTED_BUT_NOT_ENDED },
@@ -97,9 +94,15 @@ static const ValidityTest signature_tests[] = {
   { "a{ia}", DBUS_INVALID_MISSING_ARRAY_ELEMENT_TYPE },
   { "a{}", DBUS_INVALID_DICT_ENTRY_HAS_NO_FIELDS },
   { "a{aii}", DBUS_INVALID_DICT_KEY_MUST_BE_BASIC_TYPE },
-  /* { "a{i}", DBUS_INVALID_DICT_ENTRY_HAS_ONLY_ONE_FIELD }, */
-  /* { "{is}", DBUS_INVALID_DICT_ENTRY_NOT_INSIDE_ARRAY }, */
-  /* { "a{isi}", DBUS_INVALID_DICT_ENTRY_HAS_TOO_MANY_FIELDS }, */
+  { " ", DBUS_INVALID_UNKNOWN_TYPECODE },
+  { "not a valid signature", DBUS_INVALID_UNKNOWN_TYPECODE },
+  { "123", DBUS_INVALID_UNKNOWN_TYPECODE },
+  { ".", DBUS_INVALID_UNKNOWN_TYPECODE },
+  /* https://bugs.freedesktop.org/show_bug.cgi?id=17803 */
+  { "a{(ii)i}", DBUS_INVALID_DICT_KEY_MUST_BE_BASIC_TYPE },
+  { "a{i}", DBUS_INVALID_DICT_ENTRY_HAS_ONLY_ONE_FIELD },
+  { "{is}", DBUS_INVALID_DICT_ENTRY_NOT_INSIDE_ARRAY },
+  { "a{isi}", DBUS_INVALID_DICT_ENTRY_HAS_TOO_MANY_FIELDS },
 };
 
 dbus_bool_t
@@ -118,7 +121,7 @@ _dbus_marshal_validate_test (void)
     "bar",
     "bar/baz",
     "/foo/bar/",
-    "/foo/"
+    "/foo/",
     "foo/",
     "boo//blah",
     "//",
@@ -215,22 +218,6 @@ _dbus_marshal_validate_test (void)
     "foo bar"
   };
 
-  const char *valid_signatures[] = {
-    "",
-    "sss",
-    "i",
-    "b"
-  };
-
-  const char *invalid_signatures[] = {
-    " ",
-    "not a valid signature",
-    "123",
-    ".",
-    "(",
-    "a{(ii)i}" /* https://bugs.freedesktop.org/show_bug.cgi?id=17803 */
-  };
-
   /* Signature with reason */
 
   run_validity_tests (signature_tests, _DBUS_N_ELEMENTS (signature_tests),
@@ -244,10 +231,7 @@ _dbus_marshal_validate_test (void)
 
       if (!_dbus_validate_path (&str, 0,
                                 _dbus_string_get_length (&str)))
-        {
-          _dbus_warn ("Path \"%s\" should have been valid\n", valid_paths[i]);
-          _dbus_assert_not_reached ("invalid path");
-        }
+        _dbus_test_fatal ("Path \"%s\" should have been valid", valid_paths[i]);
 
       ++i;
     }
@@ -259,10 +243,7 @@ _dbus_marshal_validate_test (void)
 
       if (_dbus_validate_path (&str, 0,
                                _dbus_string_get_length (&str)))
-        {
-          _dbus_warn ("Path \"%s\" should have been invalid\n", invalid_paths[i]);
-          _dbus_assert_not_reached ("valid path");
-        }
+        _dbus_test_fatal ("Path \"%s\" should have been invalid", invalid_paths[i]);
 
       ++i;
     }
@@ -275,10 +256,7 @@ _dbus_marshal_validate_test (void)
 
       if (!_dbus_validate_interface (&str, 0,
                                      _dbus_string_get_length (&str)))
-        {
-          _dbus_warn ("Interface \"%s\" should have been valid\n", valid_interfaces[i]);
-          _dbus_assert_not_reached ("invalid interface");
-        }
+        _dbus_test_fatal ("Interface \"%s\" should have been valid", valid_interfaces[i]);
 
       ++i;
     }
@@ -290,10 +268,7 @@ _dbus_marshal_validate_test (void)
 
       if (_dbus_validate_interface (&str, 0,
                                     _dbus_string_get_length (&str)))
-        {
-          _dbus_warn ("Interface \"%s\" should have been invalid\n", invalid_interfaces[i]);
-          _dbus_assert_not_reached ("valid interface");
-        }
+        _dbus_test_fatal ("Interface \"%s\" should have been invalid", invalid_interfaces[i]);
 
       ++i;
     }
@@ -308,10 +283,7 @@ _dbus_marshal_validate_test (void)
 
       if (!_dbus_validate_bus_name (&str, 0,
                                    _dbus_string_get_length (&str)))
-        {
-          _dbus_warn ("Bus name \"%s\" should have been valid\n", valid_interfaces[i]);
-          _dbus_assert_not_reached ("invalid bus name");
-        }
+        _dbus_test_fatal ("Bus name \"%s\" should have been valid", valid_interfaces[i]);
 
       ++i;
     }
@@ -325,10 +297,7 @@ _dbus_marshal_validate_test (void)
 
           if (_dbus_validate_bus_name (&str, 0,
                                        _dbus_string_get_length (&str)))
-            {
-              _dbus_warn ("Bus name \"%s\" should have been invalid\n", invalid_interfaces[i]);
-              _dbus_assert_not_reached ("valid bus name");
-            }
+            _dbus_test_fatal ("Bus name \"%s\" should have been invalid", invalid_interfaces[i]);
         }
 
       ++i;
@@ -342,10 +311,7 @@ _dbus_marshal_validate_test (void)
 
       if (!_dbus_validate_bus_name (&str, 0,
                                     _dbus_string_get_length (&str)))
-        {
-          _dbus_warn ("Bus name \"%s\" should have been valid\n", valid_unique_names[i]);
-          _dbus_assert_not_reached ("invalid unique name");
-        }
+        _dbus_test_fatal ("Bus name \"%s\" should have been valid", valid_unique_names[i]);
 
       ++i;
     }
@@ -357,10 +323,7 @@ _dbus_marshal_validate_test (void)
 
       if (_dbus_validate_bus_name (&str, 0,
                                    _dbus_string_get_length (&str)))
-        {
-          _dbus_warn ("Bus name \"%s\" should have been invalid\n", invalid_unique_names[i]);
-          _dbus_assert_not_reached ("valid unique name");
-        }
+        _dbus_test_fatal ("Bus name \"%s\" should have been invalid", invalid_unique_names[i]);
 
       ++i;
     }
@@ -375,10 +338,7 @@ _dbus_marshal_validate_test (void)
 
       if (!_dbus_validate_error_name (&str, 0,
                                       _dbus_string_get_length (&str)))
-        {
-          _dbus_warn ("Error name \"%s\" should have been valid\n", valid_interfaces[i]);
-          _dbus_assert_not_reached ("invalid error name");
-        }
+        _dbus_test_fatal ("Error name \"%s\" should have been valid", valid_interfaces[i]);
 
       ++i;
     }
@@ -392,10 +352,7 @@ _dbus_marshal_validate_test (void)
 
           if (_dbus_validate_error_name (&str, 0,
                                          _dbus_string_get_length (&str)))
-            {
-              _dbus_warn ("Error name \"%s\" should have been invalid\n", invalid_interfaces[i]);
-              _dbus_assert_not_reached ("valid error name");
-            }
+            _dbus_test_fatal ("Error name \"%s\" should have been invalid", invalid_interfaces[i]);
         }
 
       ++i;
@@ -409,10 +366,7 @@ _dbus_marshal_validate_test (void)
 
       if (!_dbus_validate_member (&str, 0,
                                   _dbus_string_get_length (&str)))
-        {
-          _dbus_warn ("Member \"%s\" should have been valid\n", valid_members[i]);
-          _dbus_assert_not_reached ("invalid member");
-        }
+        _dbus_test_fatal ("Member \"%s\" should have been valid", valid_members[i]);
 
       ++i;
     }
@@ -424,41 +378,7 @@ _dbus_marshal_validate_test (void)
 
       if (_dbus_validate_member (&str, 0,
                                  _dbus_string_get_length (&str)))
-        {
-          _dbus_warn ("Member \"%s\" should have been invalid\n", invalid_members[i]);
-          _dbus_assert_not_reached ("valid member");
-        }
-
-      ++i;
-    }
-
-  /* Signature validation */
-  i = 0;
-  while (i < (int) _DBUS_N_ELEMENTS (valid_signatures))
-    {
-      _dbus_string_init_const (&str, valid_signatures[i]);
-
-      if (!_dbus_validate_signature (&str, 0,
-                                     _dbus_string_get_length (&str)))
-        {
-          _dbus_warn ("Signature \"%s\" should have been valid\n", valid_signatures[i]);
-          _dbus_assert_not_reached ("invalid signature");
-        }
-
-      ++i;
-    }
-
-  i = 0;
-  while (i < (int) _DBUS_N_ELEMENTS (invalid_signatures))
-    {
-      _dbus_string_init_const (&str, invalid_signatures[i]);
-
-      if (_dbus_validate_signature (&str, 0,
-                                    _dbus_string_get_length (&str)))
-        {
-          _dbus_warn ("Signature \"%s\" should have been invalid\n", invalid_signatures[i]);
-          _dbus_assert_not_reached ("valid signature");
-        }
+        _dbus_test_fatal ("Member \"%s\" should have been invalid", invalid_members[i]);
 
       ++i;
     }
@@ -466,53 +386,49 @@ _dbus_marshal_validate_test (void)
   /* Validate claimed length longer than real length */
   _dbus_string_init_const (&str, "abc.efg");
   if (_dbus_validate_bus_name (&str, 0, 8))
-    _dbus_assert_not_reached ("validated too-long string");
+    _dbus_test_fatal ("validated too-long string");
   if (_dbus_validate_interface (&str, 0, 8))
-    _dbus_assert_not_reached ("validated too-long string");
+    _dbus_test_fatal ("validated too-long string");
   if (_dbus_validate_error_name (&str, 0, 8))
-    _dbus_assert_not_reached ("validated too-long string");
+    _dbus_test_fatal ("validated too-long string");
 
   _dbus_string_init_const (&str, "abc");
   if (_dbus_validate_member (&str, 0, 4))
-    _dbus_assert_not_reached ("validated too-long string");
-
-  _dbus_string_init_const (&str, "sss");
-  if (_dbus_validate_signature (&str, 0, 4))
-    _dbus_assert_not_reached ("validated too-long signature");
+    _dbus_test_fatal ("validated too-long string");
 
   /* Validate string exceeding max name length */
   if (!_dbus_string_init (&str))
-    _dbus_assert_not_reached ("no memory");
+    _dbus_test_fatal ("no memory");
 
   while (_dbus_string_get_length (&str) <= DBUS_MAXIMUM_NAME_LENGTH)
     if (!_dbus_string_append (&str, "abc.def"))
-      _dbus_assert_not_reached ("no memory");
+      _dbus_test_fatal ("no memory");
 
   if (_dbus_validate_bus_name (&str, 0, _dbus_string_get_length (&str)))
-    _dbus_assert_not_reached ("validated overmax string");
+    _dbus_test_fatal ("validated overmax string");
   if (_dbus_validate_interface (&str, 0, _dbus_string_get_length (&str)))
-    _dbus_assert_not_reached ("validated overmax string");
+    _dbus_test_fatal ("validated overmax string");
   if (_dbus_validate_error_name (&str, 0, _dbus_string_get_length (&str)))
-    _dbus_assert_not_reached ("validated overmax string");
+    _dbus_test_fatal ("validated overmax string");
 
   /* overlong member */
   _dbus_string_set_length (&str, 0);
   while (_dbus_string_get_length (&str) <= DBUS_MAXIMUM_NAME_LENGTH)
     if (!_dbus_string_append (&str, "abc"))
-      _dbus_assert_not_reached ("no memory");
+      _dbus_test_fatal ("no memory");
 
   if (_dbus_validate_member (&str, 0, _dbus_string_get_length (&str)))
-    _dbus_assert_not_reached ("validated overmax string");
+    _dbus_test_fatal ("validated overmax string");
 
   /* overlong unique name */
   _dbus_string_set_length (&str, 0);
   _dbus_string_append (&str, ":");
   while (_dbus_string_get_length (&str) <= DBUS_MAXIMUM_NAME_LENGTH)
     if (!_dbus_string_append (&str, "abc"))
-      _dbus_assert_not_reached ("no memory");
+      _dbus_test_fatal ("no memory");
 
   if (_dbus_validate_bus_name (&str, 0, _dbus_string_get_length (&str)))
-    _dbus_assert_not_reached ("validated overmax string");
+    _dbus_test_fatal ("validated overmax string");
 
   _dbus_string_free (&str);
 
@@ -524,12 +440,11 @@ _dbus_marshal_validate_test (void)
     DBusString body;
 
     if (!_dbus_string_init (&signature) || !_dbus_string_init (&body))
-      _dbus_assert_not_reached ("oom");
+      _dbus_test_fatal ("oom");
 
     sequence = 0;
-    while (dbus_internal_do_not_use_generate_bodies (sequence,
-                                                     DBUS_LITTLE_ENDIAN,
-                                                     &signature, &body))
+    while (_dbus_test_generate_bodies (sequence, DBUS_LITTLE_ENDIAN,
+                                       &signature, &body))
       {
         DBusValidity validity;
 
@@ -539,11 +454,11 @@ _dbus_marshal_validate_test (void)
                                                     _dbus_string_get_length (&body));
         if (validity != DBUS_VALID)
           {
-            _dbus_warn ("invalid code %d expected valid on sequence %d little endian\n",
+            _dbus_warn ("invalid code %d expected valid on sequence %d little endian",
                         validity, sequence);
             _dbus_verbose_bytes_of_string (&signature, 0, _dbus_string_get_length (&signature));
             _dbus_verbose_bytes_of_string (&body, 0, _dbus_string_get_length (&body));
-            _dbus_assert_not_reached ("test failed");
+            _dbus_test_fatal ("test failed");
           }
 
         _dbus_string_set_length (&signature, 0);
@@ -552,9 +467,8 @@ _dbus_marshal_validate_test (void)
       }
                                                      
     sequence = 0;
-    while (dbus_internal_do_not_use_generate_bodies (sequence,
-                                                     DBUS_BIG_ENDIAN,
-                                                     &signature, &body))
+    while (_dbus_test_generate_bodies (sequence, DBUS_BIG_ENDIAN,
+                                       &signature, &body))
       {
         DBusValidity validity;
 
@@ -564,11 +478,11 @@ _dbus_marshal_validate_test (void)
                                                     _dbus_string_get_length (&body));
         if (validity != DBUS_VALID)
           {
-            _dbus_warn ("invalid code %d expected valid on sequence %d big endian\n",
+            _dbus_warn ("invalid code %d expected valid on sequence %d big endian",
                         validity, sequence);
             _dbus_verbose_bytes_of_string (&signature, 0, _dbus_string_get_length (&signature));
             _dbus_verbose_bytes_of_string (&body, 0, _dbus_string_get_length (&body));
-            _dbus_assert_not_reached ("test failed");
+            _dbus_test_fatal ("test failed");
           }
 
         _dbus_string_set_length (&signature, 0);
@@ -585,4 +499,4 @@ _dbus_marshal_validate_test (void)
 
 #endif /* !DOXYGEN_SHOULD_SKIP_THIS */
 
-#endif /* DBUS_BUILD_TESTS */
+#endif /* DBUS_ENABLE_EMBEDDED_TESTS */
