@@ -24,10 +24,12 @@
 
 #ifndef DOXYGEN_SHOULD_SKIP_THIS
 
-#ifdef DBUS_BUILD_TESTS
+#ifdef DBUS_ENABLE_EMBEDDED_TESTS
 #include "dbus-message-factory.h"
 #include "dbus-message-private.h"
+#include "dbus-signature.h"
 #include "dbus-test.h"
+#include <dbus/dbus-test-tap.h>
 #include <stdio.h>
 
 typedef enum
@@ -102,9 +104,9 @@ static void
 set_reply_serial (DBusMessage *message)
 {
   if (message == NULL)
-    _dbus_assert_not_reached ("oom");
+    _dbus_test_fatal ("oom");
   if (!dbus_message_set_reply_serial (message, 100))
-    _dbus_assert_not_reached ("oom");
+    _dbus_test_fatal ("oom");
 }
 
 static dbus_bool_t
@@ -135,17 +137,17 @@ generate_trivial_inner (DBusMessageDataIter *iter,
 
       if (!dbus_message_set_error_name (message,
                                         "org.freedesktop.TestErrorName"))
-        _dbus_assert_not_reached ("oom");
-      
+        _dbus_test_fatal ("oom");
+
       {
-        DBusMessageIter iter;
+        DBusMessageIter iter2;
         const char *v_STRING = "This is an error";
         
-        dbus_message_iter_init_append (message, &iter);
-        if (!dbus_message_iter_append_basic (&iter,
+        dbus_message_iter_init_append (message, &iter2);
+        if (!dbus_message_iter_append_basic (&iter2,
                                              DBUS_TYPE_STRING,
                                              &v_STRING))
-          _dbus_assert_not_reached ("oom");
+          _dbus_test_fatal ("oom");
       }
       
       set_reply_serial (message);
@@ -155,7 +157,7 @@ generate_trivial_inner (DBusMessageDataIter *iter,
     }
   
   if (message == NULL)
-    _dbus_assert_not_reached ("oom");
+    _dbus_test_fatal ("oom");
 
   *message_p = message;
   
@@ -169,6 +171,7 @@ generate_many_bodies_inner (DBusMessageDataIter *iter,
   DBusMessage *message;
   DBusString signature;
   DBusString body;
+  char byte_order;
 
   /* Keeping this small makes things go faster */
   message = dbus_message_new_method_call ("o.z.F",
@@ -176,16 +179,17 @@ generate_many_bodies_inner (DBusMessageDataIter *iter,
                                           "o.z.B",
                                           "Nah");
   if (message == NULL)
-    _dbus_assert_not_reached ("oom");
+    _dbus_test_fatal ("oom");
+
+  byte_order = _dbus_header_get_byte_order (&message->header);
 
   set_reply_serial (message);
 
   if (!_dbus_string_init (&signature) || !_dbus_string_init (&body))
-    _dbus_assert_not_reached ("oom");
-  
-  if (dbus_internal_do_not_use_generate_bodies (iter_get_sequence (iter),
-                                                message->byte_order,
-                                                &signature, &body))
+    _dbus_test_fatal ("oom");
+
+  if (_dbus_test_generate_bodies (iter_get_sequence (iter), byte_order,
+                                  &signature, &body))
     {
       const char *v_SIGNATURE;
 
@@ -194,14 +198,14 @@ generate_many_bodies_inner (DBusMessageDataIter *iter,
                                          DBUS_HEADER_FIELD_SIGNATURE,
                                          DBUS_TYPE_SIGNATURE,
                                          &v_SIGNATURE))
-        _dbus_assert_not_reached ("oom");
+        _dbus_test_fatal ("oom");
 
       if (!_dbus_string_move (&body, 0, &message->body, 0))
-        _dbus_assert_not_reached ("oom");
+        _dbus_test_fatal ("oom");
 
       _dbus_marshal_set_uint32 (&message->header.data, BODY_LENGTH_OFFSET,
                                 _dbus_string_get_length (&message->body),
-                                message->byte_order);
+                                byte_order);
       
       *message_p = message;
     }
@@ -230,11 +234,11 @@ generate_from_message (DBusString            *data,
   /* move for efficiency, since we'll nuke the message anyway */
   if (!_dbus_string_move (&message->header.data, 0,
                           data, 0))
-    _dbus_assert_not_reached ("oom");
+    _dbus_test_fatal ("oom");
 
   if (!_dbus_string_copy (&message->body, 0,
                           data, _dbus_string_get_length (data)))
-    _dbus_assert_not_reached ("oom");
+    _dbus_test_fatal ("oom");
 }
 
 static dbus_bool_t
@@ -288,7 +292,7 @@ simple_method_call (void)
                                           "o.b.Z",
                                           "Fro");
   if (message == NULL)
-    _dbus_assert_not_reached ("oom");
+    _dbus_test_fatal ("oom");
   return message;
 }
 
@@ -300,7 +304,7 @@ simple_signal (void)
                                      "o.b.Z",
                                      "Fro");
   if (message == NULL)
-    _dbus_assert_not_reached ("oom");
+    _dbus_test_fatal ("oom");
   return message;
 }
 
@@ -310,7 +314,7 @@ simple_method_return (void)
   DBusMessage *message;
   message =  dbus_message_new (DBUS_MESSAGE_TYPE_METHOD_RETURN);
   if (message == NULL)
-    _dbus_assert_not_reached ("oom");
+    _dbus_test_fatal ("oom");
 
   set_reply_serial (message);
   
@@ -323,11 +327,11 @@ simple_error (void)
   DBusMessage *message;
   message =  dbus_message_new (DBUS_MESSAGE_TYPE_ERROR);
   if (message == NULL)
-    _dbus_assert_not_reached ("oom");
+    _dbus_test_fatal ("oom");
 
   if (!dbus_message_set_error_name (message, "foo.bar"))
-    _dbus_assert_not_reached ("oom");
-  
+    _dbus_test_fatal ("oom");
+
   set_reply_serial (message);
   
   return message;
@@ -358,19 +362,24 @@ message_with_nesting_levels (int levels)
   dbus_message_iter_init_append (message, &parents[i]);
   while (i < levels)
     {
-      dbus_message_iter_open_container (&parents[i], DBUS_TYPE_VARIANT,
-                                        i == (levels - 1) ?
-                                        DBUS_TYPE_INT32_AS_STRING :
-                                        DBUS_TYPE_VARIANT_AS_STRING,
-                                        &children[i]);
+      if (!dbus_message_iter_open_container (&parents[i], DBUS_TYPE_VARIANT,
+                                             i == (levels - 1) ?
+                                             DBUS_TYPE_INT32_AS_STRING :
+                                             DBUS_TYPE_VARIANT_AS_STRING,
+                                             &children[i]))
+        _dbus_test_fatal ("oom");
       ++i;
       parents[i] = children[i-1];
     }
   --i;
-  dbus_message_iter_append_basic (&children[i], DBUS_TYPE_INT32, &v_INT32);
+
+  if (!dbus_message_iter_append_basic (&children[i], DBUS_TYPE_INT32, &v_INT32))
+    _dbus_test_fatal ("oom");
+
   while (i >= 0)
     {
-      dbus_message_iter_close_container (&parents[i], &children[i]);
+      if (!dbus_message_iter_close_container (&parents[i], &children[i]))
+        _dbus_test_fatal ("oom");
       --i;
     }
 
@@ -405,8 +414,8 @@ generate_special (DBusMessageDataIter   *iter,
                                      DBUS_TYPE_INT32, &v_INT32,
                                      DBUS_TYPE_INT32, &v_INT32,
                                      DBUS_TYPE_INVALID))
-        _dbus_assert_not_reached ("oom");
-                                     
+        _dbus_test_fatal ("oom");
+
       _dbus_header_get_field_raw (&message->header,
                                   DBUS_HEADER_FIELD_SIGNATURE,
                                   NULL, &pos);
@@ -429,7 +438,7 @@ generate_special (DBusMessageDataIter   *iter,
                                      DBUS_TYPE_INT32, &v_INT32,
                                      DBUS_TYPE_INT32, &v_INT32,
                                      DBUS_TYPE_INVALID))
-        _dbus_assert_not_reached ("oom");
+        _dbus_test_fatal ("oom");
 
       i = 0;
       while (i < (DBUS_MAXIMUM_TYPE_RECURSION_DEPTH + 1))
@@ -444,8 +453,8 @@ generate_special (DBusMessageDataIter   *iter,
                                          DBUS_HEADER_FIELD_SIGNATURE,
                                          DBUS_TYPE_SIGNATURE,
                                          &v_STRING))
-        _dbus_assert_not_reached ("oom");
-      
+        _dbus_test_fatal ("oom");
+
       _dbus_header_get_field_raw (&message->header,
                                   DBUS_HEADER_FIELD_SIGNATURE,
                                   NULL, &pos);
@@ -465,7 +474,7 @@ generate_special (DBusMessageDataIter   *iter,
                                      DBUS_TYPE_INT32, &v_INT32,
                                      DBUS_TYPE_INT32, &v_INT32,
                                      DBUS_TYPE_INVALID))
-        _dbus_assert_not_reached ("oom");
+        _dbus_test_fatal ("oom");
 
       i = 0;
       while (i <= (DBUS_MAXIMUM_TYPE_RECURSION_DEPTH + 1))
@@ -489,8 +498,8 @@ generate_special (DBusMessageDataIter   *iter,
                                          DBUS_HEADER_FIELD_SIGNATURE,
                                          DBUS_TYPE_SIGNATURE,
                                          &v_STRING))
-        _dbus_assert_not_reached ("oom");
-      
+        _dbus_test_fatal ("oom");
+
       _dbus_header_get_field_raw (&message->header,
                                   DBUS_HEADER_FIELD_SIGNATURE,
                                   NULL, &pos);
@@ -506,8 +515,8 @@ generate_special (DBusMessageDataIter   *iter,
                                      DBUS_TYPE_INT32, &v_INT32,
                                      DBUS_TYPE_INT32, &v_INT32,
                                      DBUS_TYPE_INVALID))
-        _dbus_assert_not_reached ("oom");
-                                     
+        _dbus_test_fatal ("oom");
+
       _dbus_header_get_field_raw (&message->header,
                                   DBUS_HEADER_FIELD_SIGNATURE,
                                   NULL, &pos);
@@ -525,8 +534,8 @@ generate_special (DBusMessageDataIter   *iter,
                                      DBUS_TYPE_INT32, &v_INT32,
                                      DBUS_TYPE_INT32, &v_INT32,
                                      DBUS_TYPE_INVALID))
-        _dbus_assert_not_reached ("oom");
-                                     
+        _dbus_test_fatal ("oom");
+
       _dbus_header_get_field_raw (&message->header,
                                   DBUS_HEADER_FIELD_SIGNATURE,
                                   NULL, &pos);
@@ -544,8 +553,8 @@ generate_special (DBusMessageDataIter   *iter,
                                      DBUS_TYPE_INT32, &v_INT32,
                                      DBUS_TYPE_INT32, &v_INT32,
                                      DBUS_TYPE_INVALID))
-        _dbus_assert_not_reached ("oom");
-                                     
+        _dbus_test_fatal ("oom");
+
       _dbus_header_get_field_raw (&message->header,
                                   DBUS_HEADER_FIELD_SIGNATURE,
                                   NULL, &pos);
@@ -577,15 +586,18 @@ generate_special (DBusMessageDataIter   *iter,
     }
   else if (item_seq == 8)
     {
+      char byte_order;
+
       message = simple_method_call ();
+      byte_order = _dbus_header_get_byte_order (&message->header);
       generate_from_message (data, expected_validity, message);
       
       _dbus_marshal_set_uint32 (data, BODY_LENGTH_OFFSET,
                                 DBUS_MAXIMUM_MESSAGE_LENGTH / 2 + 4,
-                                message->byte_order);
+                                byte_order);
       _dbus_marshal_set_uint32 (data, FIELDS_ARRAY_LENGTH_OFFSET,
                                 DBUS_MAXIMUM_MESSAGE_LENGTH / 2 + 4,
-                                message->byte_order);
+                                byte_order);
       *expected_validity = DBUS_INVALID_MESSAGE_TOO_LONG;
     }
   else if (item_seq == 9)
@@ -596,8 +608,8 @@ generate_special (DBusMessageDataIter   *iter,
       if (!_dbus_header_set_field_basic (&message->header,
                                          DBUS_HEADER_FIELD_SENDER,
                                          DBUS_TYPE_STRING, &v_STRING))
-        _dbus_assert_not_reached ("oom");
-      
+        _dbus_test_fatal ("oom");
+
       generate_from_message (data, expected_validity, message);
 
       *expected_validity = DBUS_INVALID_BAD_SENDER;
@@ -607,8 +619,8 @@ generate_special (DBusMessageDataIter   *iter,
       message = simple_method_call ();
 
       if (!dbus_message_set_interface (message, DBUS_INTERFACE_LOCAL))
-        _dbus_assert_not_reached ("oom");
-      
+        _dbus_test_fatal ("oom");
+
       generate_from_message (data, expected_validity, message);
 
       *expected_validity = DBUS_INVALID_USES_LOCAL_INTERFACE;
@@ -618,8 +630,8 @@ generate_special (DBusMessageDataIter   *iter,
       message = simple_method_call ();
 
       if (!dbus_message_set_path (message, DBUS_PATH_LOCAL))
-        _dbus_assert_not_reached ("oom");
-      
+        _dbus_test_fatal ("oom");
+
       generate_from_message (data, expected_validity, message);
 
       *expected_validity = DBUS_INVALID_USES_LOCAL_PATH;
@@ -630,8 +642,8 @@ generate_special (DBusMessageDataIter   *iter,
       message = simple_method_call ();
 
       if (!dbus_message_set_interface (message, NULL))
-        _dbus_assert_not_reached ("oom");
-      
+        _dbus_test_fatal ("oom");
+
       generate_from_message (data, expected_validity, message);
       
       *expected_validity = DBUS_VALID;
@@ -642,8 +654,8 @@ generate_special (DBusMessageDataIter   *iter,
       message = simple_signal ();
 
       if (!dbus_message_set_interface (message, NULL))
-        _dbus_assert_not_reached ("oom");
-      
+        _dbus_test_fatal ("oom");
+
       generate_from_message (data, expected_validity, message);
       
       *expected_validity = DBUS_INVALID_MISSING_INTERFACE;
@@ -653,8 +665,8 @@ generate_special (DBusMessageDataIter   *iter,
       message = simple_method_return ();
 
       if (!_dbus_header_delete_field (&message->header, DBUS_HEADER_FIELD_REPLY_SERIAL))
-        _dbus_assert_not_reached ("oom");
-      
+        _dbus_test_fatal ("oom");
+
       generate_from_message (data, expected_validity, message);
       
       *expected_validity = DBUS_INVALID_MISSING_REPLY_SERIAL;
@@ -664,8 +676,8 @@ generate_special (DBusMessageDataIter   *iter,
       message = simple_error ();
 
       if (!dbus_message_set_error_name (message, NULL))
-        _dbus_assert_not_reached ("oom");
-      
+        _dbus_test_fatal ("oom");
+
       generate_from_message (data, expected_validity, message);
       
       *expected_validity = DBUS_INVALID_MISSING_ERROR_NAME;
@@ -683,7 +695,7 @@ generate_special (DBusMessageDataIter   *iter,
                                      DBUS_TYPE_INT32, &v_INT32,
                                      DBUS_TYPE_INT32, &v_INT32,
                                      DBUS_TYPE_INVALID))
-        _dbus_assert_not_reached ("oom");
+        _dbus_test_fatal ("oom");
 
       i = 0;
       while (i <= (DBUS_MAXIMUM_TYPE_RECURSION_DEPTH*3 + 3))
@@ -713,8 +725,8 @@ generate_special (DBusMessageDataIter   *iter,
                                          DBUS_HEADER_FIELD_SIGNATURE,
                                          DBUS_TYPE_SIGNATURE,
                                          &v_STRING))
-        _dbus_assert_not_reached ("oom");
-      
+        _dbus_test_fatal ("oom");
+
       _dbus_header_get_field_raw (&message->header,
                                   DBUS_HEADER_FIELD_SIGNATURE,
                                   NULL, &pos);
@@ -730,8 +742,8 @@ generate_special (DBusMessageDataIter   *iter,
                                      DBUS_TYPE_INT32, &v_INT32,
                                      DBUS_TYPE_INT32, &v_INT32,
                                      DBUS_TYPE_INVALID))
-        _dbus_assert_not_reached ("oom");
-                                     
+        _dbus_test_fatal ("oom");
+
       _dbus_header_get_field_raw (&message->header,
                                   DBUS_HEADER_FIELD_SIGNATURE,
                                   NULL, &pos);
@@ -750,8 +762,8 @@ generate_special (DBusMessageDataIter   *iter,
                                      DBUS_TYPE_INT32, &v_INT32,
                                      DBUS_TYPE_INT32, &v_INT32,
                                      DBUS_TYPE_INVALID))
-        _dbus_assert_not_reached ("oom");
-                                     
+        _dbus_test_fatal ("oom");
+
       _dbus_header_get_field_raw (&message->header,
                                   DBUS_HEADER_FIELD_SIGNATURE,
                                   NULL, &pos);
@@ -769,8 +781,8 @@ generate_special (DBusMessageDataIter   *iter,
                                      DBUS_TYPE_INT32, &v_INT32,
                                      DBUS_TYPE_INT32, &v_INT32,
                                      DBUS_TYPE_INVALID))
-        _dbus_assert_not_reached ("oom");
-                                     
+        _dbus_test_fatal ("oom");
+
       _dbus_header_get_field_raw (&message->header,
                                   DBUS_HEADER_FIELD_SIGNATURE,
                                   NULL, &pos);
@@ -852,7 +864,7 @@ generate_wrong_length (DBusMessageDataIter *iter,
   else
     {      
       if (!_dbus_string_lengthen (data, adjust))
-        _dbus_assert_not_reached ("oom");
+        _dbus_test_fatal ("oom");
       *expected_validity = DBUS_INVALID_TOO_MUCH_DATA;
     }
 
@@ -935,6 +947,9 @@ generate_byte_changed (DBusMessageDataIter *iter,
   return TRUE;
 }
 
+#if 0
+/* This is really expensive and doesn't add too much coverage */
+
 static dbus_bool_t
 find_next_typecode (DBusMessageDataIter *iter,
                     DBusString          *data,
@@ -978,7 +993,7 @@ find_next_typecode (DBusMessageDataIter *iter,
 
       _dbus_assert (byte_seq < _dbus_string_get_length (data));
 
-      if (_dbus_type_is_valid (_dbus_string_get_byte (data, byte_seq)))
+      if (dbus_type_is_valid (_dbus_string_get_byte (data, byte_seq)))
         break;
       else
         iter_next (iter);
@@ -1017,7 +1032,7 @@ static const int typecodes[] = {
   DBUS_TYPE_UNIX_FD,
   255 /* random invalid typecode */
 };
-  
+
 static dbus_bool_t
 generate_typecode_changed (DBusMessageDataIter *iter,
                            DBusString          *data,
@@ -1066,7 +1081,7 @@ generate_typecode_changed (DBusMessageDataIter *iter,
   _dbus_assert (iter->depth == (base_depth + 0));
 
 #if 0
-  printf ("Changing byte %d in message %d to %c\n",
+  _dbus_test_diag ("Changing byte %d in message %d to %c",
           byte_seq, iter_get_sequence (iter), typecodes[typecode_seq]);
 #endif
   
@@ -1074,6 +1089,7 @@ generate_typecode_changed (DBusMessageDataIter *iter,
   *expected_validity = DBUS_VALIDITY_UNKNOWN;
   return TRUE;
 }
+#endif
 
 typedef struct
 {
@@ -1183,15 +1199,15 @@ generate_uint32_changed (DBusMessageDataIter *iter,
     }
 
 #if 0
-  printf ("body %d change %d pos %d ",
+  _dbus_test_diag ("body %d change %d pos %d ",
           body_seq, change_seq, byte_seq);
 
   if (change->type == CHANGE_TYPE_ADJUST)
-    printf ("adjust by %d", (int) change->value);
+    _dbus_test_diag ("adjust by %d", (int) change->value);
   else
-    printf ("set to %u", change->value);
+    _dbus_test_diag ("set to %u", change->value);
   
-  printf (" \t%u -> %u\n",
+  _dbus_test_diag (" \t%u -> %u",
           _dbus_marshal_read_uint32 (data, byte_seq, byte_order, NULL),
           v_UINT32);
 #endif
@@ -1263,15 +1279,14 @@ _dbus_message_data_iter_get_and_next (DBusMessageDataIter *iter,
   
   if (iter_first_in_series (iter))
     {
-      printf (" testing message loading: %s ", generators[generator].name);
-      fflush (stdout);
+      _dbus_test_diag (" testing message loading: %s ", generators[generator].name);
     }
   
   func = generators[generator].func;
 
   if (!_dbus_string_init (&data->data))
-    _dbus_assert_not_reached ("oom");
-  
+    _dbus_test_fatal ("oom");
+
   if ((*func)(iter, &data->data, &data->expected_validity))
     ;
   else
@@ -1280,7 +1295,7 @@ _dbus_message_data_iter_get_and_next (DBusMessageDataIter *iter,
       iter_unrecurse (iter);
       iter_next (iter); /* next generator */
       _dbus_string_free (&data->data);
-      printf ("%d test loads cumulative\n", iter->count);
+      _dbus_test_diag ("%d test loads cumulative", iter->count);
       goto restart;
     }
   iter_unrecurse (iter);
@@ -1291,4 +1306,4 @@ _dbus_message_data_iter_get_and_next (DBusMessageDataIter *iter,
 
 #endif /* !DOXYGEN_SHOULD_SKIP_THIS */
 
-#endif /* DBUS_BUILD_TESTS */
+#endif /* DBUS_ENABLE_EMBEDDED_TESTS */
