@@ -42,6 +42,7 @@
 #include "dbus-protocol.h"
 
 #define WIN32_LEAN_AND_MEAN
+#include <windows.h>
 //#define STRICT
 //#include <windows.h>
 //#undef STRICT
@@ -59,28 +60,25 @@
  */
 struct DBusBabysitter
   {
-    int refcount;
+    DBusAtomic refcount;
 
     HANDLE start_sync_event;
-#ifdef DBUS_BUILD_TESTS
 
-    HANDLE end_sync_event;
-#endif
-
-    char *executable;
-    DBusSpawnChildSetupFunc child_setup;
-    void *user_data;
+    char *log_name;
 
     int argc;
     char **argv;
     char **envp;
 
+    HANDLE thread_handle;
     HANDLE child_handle;
-    int socket_to_babysitter;	/* Connection to the babysitter thread */
-    int socket_to_main;
+    DBusSocket socket_to_babysitter;	/* Connection to the babysitter thread */
+    DBusSocket socket_to_main;
 
     DBusWatchList *watches;
     DBusWatch *sitter_watch;
+    DBusBabysitterFinishedFunc finished_cb;
+    void *finished_data;
 
     dbus_bool_t have_spawn_errno;
     int spawn_errno;
@@ -88,16 +86,33 @@ struct DBusBabysitter
     int child_status;
   };
 
+static void
+_dbus_babysitter_trace_ref (DBusBabysitter *sitter,
+    int old_refcount,
+    int new_refcount,
+    const char *why)
+{
+#ifdef DBUS_ENABLE_VERBOSE_MODE
+  static int enabled = -1;
+
+  _dbus_trace_ref ("DBusBabysitter", sitter, old_refcount, new_refcount, why,
+      "DBUS_BABYSITTER_TRACE", &enabled);
+#endif
+}
+
 static DBusBabysitter*
 _dbus_babysitter_new (void)
 {
   DBusBabysitter *sitter;
+  dbus_int32_t old_refcount;
 
   sitter = dbus_new0 (DBusBabysitter, 1);
   if (sitter == NULL)
     return NULL;
 
-  sitter->refcount = 1;
+  old_refcount = _dbus_atomic_inc (&sitter->refcount);
+
+  _dbus_babysitter_trace_ref (sitter, old_refcount, old_refcount+1, __FUNCTION__);
 
   sitter->start_sync_event = CreateEvent (NULL, FALSE, FALSE, NULL);
   if (sitter->start_sync_event == NULL)
@@ -106,18 +121,9 @@ _dbus_babysitter_new (void)
       return NULL;
     }
 
-#ifdef DBUS_BUILD_TESTS
-  sitter->end_sync_event = CreateEvent (NULL, FALSE, FALSE, NULL);
-  if (sitter->end_sync_event == NULL)
-    {
-      _dbus_babysitter_unref (sitter);
-      return NULL;
-    }
-#endif
-
   sitter->child_handle = NULL;
 
-  sitter->socket_to_babysitter = sitter->socket_to_main = -1;
+  sitter->socket_to_babysitter = sitter->socket_to_main = _dbus_socket_get_invalid ();
 
   sitter->argc = 0;
   sitter->argv = NULL;
@@ -145,15 +151,38 @@ _dbus_babysitter_new (void)
 DBusBabysitter *
 _dbus_babysitter_ref (DBusBabysitter *sitter)
 {
+  dbus_int32_t old_refcount;
   PING();
   _dbus_assert (sitter != NULL);
-  _dbus_assert (sitter->refcount > 0);
 
-  sitter->refcount += 1;
+  old_refcount = _dbus_atomic_inc (&sitter->refcount);
+  _dbus_assert (old_refcount > 0);
+  _dbus_babysitter_trace_ref (sitter, old_refcount, old_refcount+1, __FUNCTION__);
 
   return sitter;
 }
 
+static void
+close_socket_to_babysitter (DBusBabysitter *sitter)
+{
+  _dbus_verbose ("Closing babysitter\n");
+
+  if (sitter->sitter_watch != NULL)
+    {
+      _dbus_assert (sitter->watches != NULL);
+      _dbus_watch_list_remove_watch (sitter->watches,  sitter->sitter_watch);
+      _dbus_watch_invalidate (sitter->sitter_watch);
+      _dbus_watch_unref (sitter->sitter_watch);
+      sitter->sitter_watch = NULL;
+    }
+
+  if (sitter->socket_to_babysitter.sock != INVALID_SOCKET)
+    {
+      _dbus_close_socket (sitter->socket_to_babysitter, NULL);
+      sitter->socket_to_babysitter.sock = INVALID_SOCKET;
+    }
+}
+
 /**
  * Decrement the reference count on the babysitter object.
  *
@@ -163,25 +192,23 @@ void
 _dbus_babysitter_unref (DBusBabysitter *sitter)
 {
   int i;
+  dbus_int32_t old_refcount;
 
   PING();
   _dbus_assert (sitter != NULL);
-  _dbus_assert (sitter->refcount > 0);
 
-  sitter->refcount -= 1;
+  old_refcount = _dbus_atomic_dec (&sitter->refcount);
+  _dbus_assert (old_refcount > 0);
+  _dbus_babysitter_trace_ref (sitter, old_refcount, old_refcount-1, __FUNCTION__);
 
-  if (sitter->refcount == 0)
+  if (old_refcount == 1)
     {
-      if (sitter->socket_to_babysitter != -1)
-        {
-          _dbus_close_socket (sitter->socket_to_babysitter, NULL);
-          sitter->socket_to_babysitter = -1;
-        }
+      close_socket_to_babysitter (sitter);
 
-      if (sitter->socket_to_main != -1)
+      if (sitter->socket_to_main.sock != INVALID_SOCKET)
         {
           _dbus_close_socket (sitter->socket_to_main, NULL);
-          sitter->socket_to_main = -1;
+          sitter->socket_to_main.sock = INVALID_SOCKET;
         }
 
       PING();
@@ -230,15 +257,13 @@ _dbus_babysitter_unref (DBusBabysitter *sitter)
           sitter->start_sync_event = NULL;
         }
 
-#ifdef DBUS_BUILD_TESTS
-      if (sitter->end_sync_event != NULL)
+      if (sitter->thread_handle)
         {
-          CloseHandle (sitter->end_sync_event);
-          sitter->end_sync_event = NULL;
+          CloseHandle (sitter->thread_handle);
+          sitter->thread_handle = NULL;
         }
-#endif
 
-      dbus_free (sitter->executable);
+      dbus_free (sitter->log_name);
 
       dbus_free (sitter);
     }
@@ -317,7 +342,7 @@ _dbus_babysitter_set_child_exit_error (DBusBabysitter *sitter,
       char *emsg = _dbus_win_error_string (sitter->spawn_errno);
       dbus_set_error (error, DBUS_ERROR_SPAWN_EXEC_FAILED,
                       "Failed to execute program %s: %s",
-                      sitter->executable, emsg);
+                      sitter->log_name, emsg);
       _dbus_win_free_error_string (emsg);
     }
   else if (sitter->have_child_status)
@@ -325,14 +350,14 @@ _dbus_babysitter_set_child_exit_error (DBusBabysitter *sitter,
       PING();
       dbus_set_error (error, DBUS_ERROR_SPAWN_CHILD_EXITED,
                       "Process %s exited with status %d",
-                      sitter->executable, sitter->child_status);
+                      sitter->log_name, sitter->child_status);
     }
   else
     {
       PING();
       dbus_set_error (error, DBUS_ERROR_FAILED,
                       "Process %s exited, status unknown",
-                      sitter->executable);
+                      sitter->log_name);
     }
   PING();
 }
@@ -372,17 +397,23 @@ handle_watch (DBusWatch       *watch,
    */
 
   PING();
-  _dbus_close_socket (sitter->socket_to_babysitter, NULL);
+  close_socket_to_babysitter (sitter);
   PING();
-  sitter->socket_to_babysitter = -1;
+
+  if (_dbus_babysitter_get_child_exited (sitter) &&
+      sitter->finished_cb != NULL)
+    {
+      sitter->finished_cb (sitter, sitter->finished_data);
+      sitter->finished_cb = NULL;
+    }
 
   return TRUE;
 }
 
 /* protect_argv lifted from GLib, relicensed by author, Tor Lillqvist */
 static int
-protect_argv (char  **argv,
-              char ***new_argv)
+protect_argv (char  * const *argv,
+              char        ***new_argv)
 {
   int i;
   int argc = 0;
@@ -409,7 +440,7 @@ protect_argv (char  **argv,
    */
   for (i = 0; i < argc; i++)
     {
-      char *p = argv[i];
+      const char *p = argv[i];
       char *q;
       int len = 0;
       int need_dblquotes = FALSE;
@@ -421,7 +452,7 @@ protect_argv (char  **argv,
             len++;
           else if (*p == '\\')
             {
-              char *pp = p;
+              const char *pp = p;
               while (*pp && *pp == '\\')
                 pp++;
               if (*pp == '"')
@@ -448,7 +479,7 @@ protect_argv (char  **argv,
             *q++ = '\\';
           else if (*p == '\\')
             {
-              char *pp = p;
+              const char *pp = p;
               while (*pp && *pp == '\\')
                 pp++;
               if (*pp == '"')
@@ -477,8 +508,7 @@ compose_string (char **strings, char separator)
   int n = 0;
   char *buf;
   char *p;
-  const char *ptr;
-  
+
   if (!strings || !strings[0])
     return 0;
   for (i = 0; strings[i]; i++)
@@ -513,8 +543,10 @@ build_env_string (char** envp)
   return compose_string (envp, '\0');
 }
 
-static HANDLE
-spawn_program (char* name, char** argv, char** envp)
+HANDLE
+_dbus_spawn_program (const char *name,
+                     char      **argv,
+                     char      **envp)
 {
   PROCESS_INFORMATION pi = { NULL, 0, 0, 0 };
   STARTUPINFOA si;
@@ -536,6 +568,15 @@ spawn_program (char* name, char** argv, char** envp)
 
   memset (&si, 0, sizeof (si));
   si.cb = sizeof (si);
+
+#ifdef DBUS_ENABLE_VERBOSE_MODE
+  {
+    char *s = compose_string (envp, ';');
+    _dbus_verbose ("spawning '%s'' with args: '%s' env: '%s'\n", name, arg_string, s);
+    free (s);
+  }
+#endif
+
 #ifdef DBUS_WINCE
   result = CreateProcessA (name, arg_string, NULL, NULL, FALSE, 0,
 #else
@@ -557,25 +598,22 @@ spawn_program (char* name, char** argv, char** envp)
 static DWORD __stdcall
 babysitter (void *parameter)
 {
+  int ret = 0;
   DBusBabysitter *sitter = (DBusBabysitter *) parameter;
-  int fd;
-  PING();
-  _dbus_babysitter_ref (sitter);
+  HANDLE handle;
 
-  if (sitter->child_setup)
+  PING();
+  _dbus_verbose ("babysitter: spawning %s\n", sitter->log_name);
+
+  PING();
+  handle = _dbus_spawn_program (sitter->log_name, sitter->argv, sitter->envp);
+
+  PING();
+  if (handle != INVALID_HANDLE_VALUE)
     {
-      PING();
-      (*sitter->child_setup) (sitter->user_data);
+      sitter->child_handle = handle;
     }
-
-  _dbus_verbose ("babysitter: spawning %s\n", sitter->executable);
-
-  PING();
-  sitter->child_handle = spawn_program (sitter->executable,
-					sitter->argv, sitter->envp);
-
-  PING();
-  if (sitter->child_handle == (HANDLE) -1)
+  else
     {
       sitter->child_handle = NULL;
       sitter->have_spawn_errno = TRUE;
@@ -587,49 +625,50 @@ babysitter (void *parameter)
 
   if (sitter->child_handle != NULL)
     {
-      int ret;
       DWORD status;
 
       PING();
+      // wait until process finished
       WaitForSingleObject (sitter->child_handle, INFINITE);
 
       PING();
       ret = GetExitCodeProcess (sitter->child_handle, &status);
-
-      sitter->child_status = status;
-      sitter->have_child_status = TRUE;
+      if (ret)
+        {
+          sitter->child_status = status;
+          sitter->have_child_status = TRUE;
+        }
 
       CloseHandle (sitter->child_handle);
       sitter->child_handle = NULL;
     }
 
-#ifdef DBUS_BUILD_TESTS
-  SetEvent (sitter->end_sync_event);
-#endif
-
   PING();
-  send (sitter->socket_to_main, " ", 1, 0);
+  send (sitter->socket_to_main.sock, " ", 1, 0);
 
   _dbus_babysitter_unref (sitter);
 
-  return 0;
+  return ret ? 0 : 1;
 }
 
 dbus_bool_t
 _dbus_spawn_async_with_babysitter (DBusBabysitter           **sitter_p,
-                                   char                     **argv,
+                                   const char                *log_name,
+                                   char              * const *argv,
                                    char                     **envp,
-                                   DBusSpawnChildSetupFunc    child_setup,
-                                   void                      *user_data,
+                                   DBusSpawnFlags             flags _DBUS_GNUC_UNUSED,
+                                   DBusSpawnChildSetupFunc    child_setup _DBUS_GNUC_UNUSED,
+                                   void                      *user_data _DBUS_GNUC_UNUSED,
                                    DBusError                 *error)
 {
   DBusBabysitter *sitter;
-  HANDLE sitter_thread;
   DWORD sitter_thread_id;
   
   _DBUS_ASSERT_ERROR_IS_CLEAR (error);
+  _dbus_assert (argv[0] != NULL);
 
-  *sitter_p = NULL;
+  if (sitter_p != NULL)
+    *sitter_p = NULL;
 
   PING();
   sitter = _dbus_babysitter_new ();
@@ -639,20 +678,26 @@ _dbus_spawn_async_with_babysitter (DBusBabysitter           **sitter_p,
       return FALSE;
     }
 
-  sitter->child_setup = child_setup;
-  sitter->user_data = user_data;
+  sitter->log_name = _dbus_strdup (log_name);
+  if (sitter->log_name == NULL && log_name != NULL)
+    {
+      _DBUS_SET_OOM (error);
+      goto out0;
+    }
 
-  sitter->executable = _dbus_strdup (argv[0]);
-  if (sitter->executable == NULL)
+  if (sitter->log_name == NULL)
+    sitter->log_name = _dbus_strdup (argv[0]);
+
+  if (sitter->log_name == NULL)
     {
       _DBUS_SET_OOM (error);
       goto out0;
     }
 
   PING();
-  if (!_dbus_full_duplex_pipe (&sitter->socket_to_babysitter,
-                               &sitter->socket_to_main,
-                               FALSE, error))
+  if (!_dbus_socketpair (&sitter->socket_to_babysitter,
+                         &sitter->socket_to_main,
+                         FALSE, error))
     goto out0;
 
   sitter->sitter_watch = _dbus_watch_new (sitter->socket_to_babysitter,
@@ -668,6 +713,12 @@ _dbus_spawn_async_with_babysitter (DBusBabysitter           **sitter_p,
   PING();
   if (!_dbus_watch_list_add_watch (sitter->watches,  sitter->sitter_watch))
     {
+      /* we need to free it early so the destructor won't try to remove it
+       * without it having been added, which DBusLoop doesn't allow */
+      _dbus_watch_invalidate (sitter->sitter_watch);
+      _dbus_watch_unref (sitter->sitter_watch);
+      sitter->sitter_watch = NULL;
+
       _DBUS_SET_OOM (error);
       goto out0;
     }
@@ -681,17 +732,16 @@ _dbus_spawn_async_with_babysitter (DBusBabysitter           **sitter_p,
   sitter->envp = envp;
 
   PING();
-  sitter_thread = (HANDLE) CreateThread (NULL, 0, babysitter,
-                  sitter, 0, &sitter_thread_id);
+  sitter->thread_handle = (HANDLE) CreateThread (NULL, 0, babysitter,
+                  _dbus_babysitter_ref (sitter), 0, &sitter_thread_id);
 
-  if (sitter_thread == 0)
+  if (sitter->thread_handle == NULL)
     {
       PING();
       dbus_set_error_const (error, DBUS_ERROR_SPAWN_FORK_FAILED,
                             "Failed to create new thread");
       goto out0;
     }
-  CloseHandle (sitter_thread);
 
   PING();
   WaitForSingleObject (sitter->start_sync_event, INFINITE);
@@ -713,229 +763,22 @@ out0:
   return FALSE;
 }
 
-#ifdef DBUS_BUILD_TESTS
+void
+_dbus_babysitter_set_result_function  (DBusBabysitter             *sitter,
+                                       DBusBabysitterFinishedFunc  finished,
+                                       void                       *user_data)
+{
+  sitter->finished_cb = finished;
+  sitter->finished_data = user_data;
+}
 
 #define LIVE_CHILDREN(sitter) ((sitter)->child_handle != NULL)
 
-static void
+void
 _dbus_babysitter_block_for_child_exit (DBusBabysitter *sitter)
 {
-  if (sitter->child_handle == NULL)
-    return;
-
-  WaitForSingleObject (sitter->end_sync_event, INFINITE);
+  /* The thread terminates after the child does. We want to wait for the thread,
+   * not just the child, to avoid data races and ensure that it has freed all
+   * its memory. */
+  WaitForSingleObject (sitter->thread_handle, INFINITE);
 }
-
-static dbus_bool_t
-check_spawn_nonexistent (void *data)
-{
-  char *argv[4] = { NULL, NULL, NULL, NULL };
-  DBusBabysitter *sitter;
-  DBusError error;
-
-  sitter = NULL;
-
-  dbus_error_init (&error);
-
-  /*** Test launching nonexistent binary */
-
-  argv[0] = "/this/does/not/exist/32542sdgafgafdg";
-  if (_dbus_spawn_async_with_babysitter (&sitter, argv, NULL,
-                                         NULL, NULL,
-                                         &error))
-    {
-      _dbus_babysitter_block_for_child_exit (sitter);
-      _dbus_babysitter_set_child_exit_error (sitter, &error);
-    }
-
-  if (sitter)
-    _dbus_babysitter_unref (sitter);
-
-  if (!dbus_error_is_set (&error))
-    {
-      _dbus_warn ("Did not get an error launching nonexistent executable\n");
-      return FALSE;
-    }
-
-  if (!(dbus_error_has_name (&error, DBUS_ERROR_NO_MEMORY) ||
-        dbus_error_has_name (&error, DBUS_ERROR_SPAWN_EXEC_FAILED)))
-    {
-      _dbus_warn ("Not expecting error when launching nonexistent executable: %s: %s\n",
-                  error.name, error.message);
-      dbus_error_free (&error);
-      return FALSE;
-    }
-
-  dbus_error_free (&error);
-
-  return TRUE;
-}
-
-static dbus_bool_t
-check_spawn_segfault (void *data)
-{
-  char *argv[4] = { NULL, NULL, NULL, NULL };
-  DBusBabysitter *sitter;
-  DBusError error;
-
-  sitter = NULL;
-
-  dbus_error_init (&error);
-
-  /*** Test launching segfault binary */
-
-  argv[0] = TEST_SEGFAULT_BINARY;
-  if (_dbus_spawn_async_with_babysitter (&sitter, argv, NULL,
-                                         NULL, NULL,
-                                         &error))
-    {
-      _dbus_babysitter_block_for_child_exit (sitter);
-      _dbus_babysitter_set_child_exit_error (sitter, &error);
-    }
-
-  if (sitter)
-    _dbus_babysitter_unref (sitter);
-
-  if (!dbus_error_is_set (&error))
-    {
-      _dbus_warn ("Did not get an error launching segfaulting binary\n");
-      return FALSE;
-    }
-
-  if (!(dbus_error_has_name (&error, DBUS_ERROR_NO_MEMORY) ||
-        dbus_error_has_name (&error, DBUS_ERROR_SPAWN_CHILD_EXITED)))
-    {
-      _dbus_warn ("Not expecting error when launching segfaulting executable: %s: %s\n",
-                  error.name, error.message);
-      dbus_error_free (&error);
-      return FALSE;
-    }
-
-  dbus_error_free (&error);
-
-  return TRUE;
-}
-
-static dbus_bool_t
-check_spawn_exit (void *data)
-{
-  char *argv[4] = { NULL, NULL, NULL, NULL };
-  DBusBabysitter *sitter;
-  DBusError error;
-
-  sitter = NULL;
-
-  dbus_error_init (&error);
-
-  /*** Test launching exit failure binary */
-
-  argv[0] = TEST_EXIT_BINARY;
-  if (_dbus_spawn_async_with_babysitter (&sitter, argv, NULL,
-                                         NULL, NULL,
-                                         &error))
-    {
-      _dbus_babysitter_block_for_child_exit (sitter);
-      _dbus_babysitter_set_child_exit_error (sitter, &error);
-    }
-
-  if (sitter)
-    _dbus_babysitter_unref (sitter);
-
-  if (!dbus_error_is_set (&error))
-    {
-      _dbus_warn ("Did not get an error launching binary that exited with failure code\n");
-      return FALSE;
-    }
-
-  if (!(dbus_error_has_name (&error, DBUS_ERROR_NO_MEMORY) ||
-        dbus_error_has_name (&error, DBUS_ERROR_SPAWN_CHILD_EXITED)))
-    {
-      _dbus_warn ("Not expecting error when launching exiting executable: %s: %s\n",
-                  error.name, error.message);
-      dbus_error_free (&error);
-      return FALSE;
-    }
-
-  dbus_error_free (&error);
-
-  return TRUE;
-}
-
-static dbus_bool_t
-check_spawn_and_kill (void *data)
-{
-  char *argv[4] = { NULL, NULL, NULL, NULL };
-  DBusBabysitter *sitter;
-  DBusError error;
-
-  sitter = NULL;
-
-  dbus_error_init (&error);
-
-  /*** Test launching sleeping binary then killing it */
-
-  argv[0] = TEST_SLEEP_FOREVER_BINARY;
-  if (_dbus_spawn_async_with_babysitter (&sitter, argv, NULL,
-                                         NULL, NULL,
-                                         &error))
-    {
-      _dbus_babysitter_kill_child (sitter);
-
-      _dbus_babysitter_block_for_child_exit (sitter);
-
-      _dbus_babysitter_set_child_exit_error (sitter, &error);
-    }
-
-  if (sitter)
-    _dbus_babysitter_unref (sitter);
-
-  if (!dbus_error_is_set (&error))
-    {
-      _dbus_warn ("Did not get an error after killing spawned binary\n");
-      return FALSE;
-    }
-
-  if (!(dbus_error_has_name (&error, DBUS_ERROR_NO_MEMORY) ||
-        dbus_error_has_name (&error, DBUS_ERROR_SPAWN_CHILD_EXITED)))
-    {
-      _dbus_warn ("Not expecting error when killing executable: %s: %s\n",
-                  error.name, error.message);
-      dbus_error_free (&error);
-      return FALSE;
-    }
-
-  dbus_error_free (&error);
-
-  return TRUE;
-}
-
-dbus_bool_t
-_dbus_spawn_test (const char *test_data_dir)
-{
-  if (!_dbus_test_oom_handling ("spawn_nonexistent",
-                                check_spawn_nonexistent,
-                                NULL))
-    return FALSE;
-
-  /* Don't run the obnoxious segfault test by default,
-   * it's a pain to have to click all those error boxes.
-   */
-  if (getenv ("DO_SEGFAULT_TEST"))
-    if (!_dbus_test_oom_handling ("spawn_segfault",
-                                  check_spawn_segfault,
-                                  NULL))
-      return FALSE;
-
-  if (!_dbus_test_oom_handling ("spawn_exit",
-                                check_spawn_exit,
-                                NULL))
-    return FALSE;
-
-  if (!_dbus_test_oom_handling ("spawn_and_kill",
-                                check_spawn_and_kill,
-                                NULL))
-    return FALSE;
-
-  return TRUE;
-}
-#endif
