@@ -25,12 +25,19 @@
 #ifndef DBUS_SYSDEPS_H
 #define DBUS_SYSDEPS_H
 
+#ifndef VERSION
+#warning Please include config.h before dbus-sysdeps.h
 #include "config.h"
+#endif
 
 #ifdef HAVE_STDINT_H
 #include <stdint.h>
 #endif
 
+#ifdef HAVE_INTTYPES_H
+#include <inttypes.h>
+#endif
+
 #include <dbus/dbus-errors.h>
 #include <dbus/dbus-file.h>
 #include <dbus/dbus-string.h>
@@ -41,6 +48,32 @@
 #include <string.h>
 #include <stdarg.h>
 
+#if !defined(BROKEN_POLL) && (defined(__APPLE__) || defined(__INTERIX))
+  /* Following libcurl's example, we blacklist poll() on Darwin
+   * (macOS, iOS, etc.) and Interix due to a history of implementation
+   * issues.
+   * https://github.com/curl/curl/blob/master/m4/curl-functions.m4
+   *
+   * On unspecified older macOS versions, poll() failed if given a
+   * device node to poll.
+   *
+   * On macOS < 10.9, poll() with nfds=0 failed instead of waiting for
+   * the timeout and then succeeding.
+   *
+   * On macOS >= 10.12, poll() with nfds=0 succeeded immediately
+   * instead of waiting for the timeout, resulting in busy-looping.
+   *
+   * On Interix, poll() apparently only works for files in /proc.
+   *
+   * The "legacy" build flavour in our CI machinery defines BROKEN_POLL
+   * on whatever platform is in use (normally Linux) to force use of the
+   * same select()-based poll() emulation that we use for macOS, Interix,
+   * and any platform that lacks a real poll(), so that we can test it
+   * more regularly.
+   */
+# define BROKEN_POLL
+#endif
+
 /* AIX sys/poll.h does #define events reqevents, and other
  * wonderousness, so must include sys/poll before declaring
  * DBusPollFD
@@ -55,6 +88,10 @@
 #include "dbus-sysdeps-wince-glue.h"
 #endif
 
+#ifdef DBUS_WIN
+#include <ws2tcpip.h>
+#endif
+
 DBUS_BEGIN_DECLS
 
 #ifdef DBUS_WIN
@@ -81,11 +118,13 @@ typedef struct DBusPipe DBusPipe;
  * @{
  */
 
+DBUS_PRIVATE_EXPORT
 void _dbus_abort (void) _DBUS_GNUC_NORETURN;
 
+dbus_bool_t _dbus_check_setuid (void);
+DBUS_PRIVATE_EXPORT
 const char* _dbus_getenv (const char *varname);
-dbus_bool_t _dbus_setenv (const char *varname,
-			  const char *value);
+DBUS_PRIVATE_EXPORT
 dbus_bool_t _dbus_clearenv (void);
 char **     _dbus_get_environment (void);
 
@@ -110,29 +149,78 @@ typedef unsigned long dbus_gid_t;
 /** an appropriate printf format for dbus_gid_t */
 #define DBUS_GID_FORMAT "%lu"
 
-
 /**
  * Socket interface
- *
- *  @todo Use for the file descriptors a struct
- *           - struct DBusSocket{ int d; }; -
- *        instead of int to get type-safety which 
- *        will be checked by the compiler.
- * 
  */
+#ifdef DBUS_WIN
 
-dbus_bool_t _dbus_open_tcp_socket  (int              *fd,
+typedef struct { SOCKET sock; } DBusSocket;
+# define DBUS_SOCKET_FORMAT "Iu"
+# define DBUS_SOCKET_INIT { INVALID_SOCKET }
+
+_DBUS_WARN_UNUSED_RESULT
+static inline SOCKET
+_dbus_socket_printable (DBusSocket s) { return s.sock; }
+
+_DBUS_WARN_UNUSED_RESULT
+static inline dbus_bool_t
+_dbus_socket_is_valid (DBusSocket s) { return s.sock != INVALID_SOCKET; }
+
+static inline void
+_dbus_socket_invalidate (DBusSocket *s) { s->sock = INVALID_SOCKET; }
+
+_DBUS_WARN_UNUSED_RESULT
+static inline int
+_dbus_socket_get_int (DBusSocket s) { return (int)s.sock; }
+
+#else /* not DBUS_WIN */
+
+typedef struct { int fd; } DBusSocket;
+# define DBUS_SOCKET_FORMAT "d"
+# define DBUS_SOCKET_INIT { -1 }
+
+_DBUS_WARN_UNUSED_RESULT
+static inline int
+_dbus_socket_printable (DBusSocket s) { return s.fd; }
+
+_DBUS_WARN_UNUSED_RESULT
+static inline dbus_bool_t
+_dbus_socket_is_valid (DBusSocket s) { return s.fd >= 0; }
+
+static inline void
+_dbus_socket_invalidate (DBusSocket *s) { s->fd = -1; }
+
+_DBUS_WARN_UNUSED_RESULT
+static inline int
+_dbus_socket_get_int (DBusSocket s) { return s.fd; }
+
+#endif /* not DBUS_WIN */
+
+_DBUS_WARN_UNUSED_RESULT
+static inline DBusSocket
+_dbus_socket_get_invalid (void)
+{
+  DBusSocket s = DBUS_SOCKET_INIT;
+
+  return s;
+}
+
+dbus_bool_t _dbus_set_socket_nonblocking (DBusSocket      fd,
+                                          DBusError      *error);
+
+DBUS_PRIVATE_EXPORT
+dbus_bool_t _dbus_close_socket     (DBusSocket        fd,
                                     DBusError        *error);
-dbus_bool_t _dbus_close_socket     (int               fd,
-                                    DBusError        *error);
-int         _dbus_read_socket      (int               fd,
+DBUS_PRIVATE_EXPORT
+int         _dbus_read_socket      (DBusSocket        fd,
                                     DBusString       *buffer,
                                     int               count);
-int         _dbus_write_socket     (int               fd,
+DBUS_PRIVATE_EXPORT
+int         _dbus_write_socket     (DBusSocket        fd,
                                     const DBusString *buffer,
                                     int               start,
                                     int               len);
-int         _dbus_write_socket_two (int               fd,
+int         _dbus_write_socket_two (DBusSocket        fd,
                                     const DBusString *buffer1,
                                     int               start1,
                                     int               len1,
@@ -140,18 +228,19 @@ int         _dbus_write_socket_two (int               fd,
                                     int               start2,
                                     int               len2);
 
-int _dbus_read_socket_with_unix_fds      (int               fd,
+int _dbus_read_socket_with_unix_fds      (DBusSocket        fd,
                                           DBusString       *buffer,
                                           int               count,
                                           int              *fds,
-                                          int              *n_fds);
-int _dbus_write_socket_with_unix_fds     (int               fd,
+                                          unsigned int     *n_fds);
+DBUS_PRIVATE_EXPORT
+int _dbus_write_socket_with_unix_fds     (DBusSocket        fd,
                                           const DBusString *buffer,
                                           int               start,
                                           int               len,
                                           const int        *fds,
                                           int               n_fds);
-int _dbus_write_socket_with_unix_fds_two (int               fd,
+int _dbus_write_socket_with_unix_fds_two (DBusSocket        fd,
                                           const DBusString *buffer1,
                                           int               start1,
                                           int               len1,
@@ -161,35 +250,43 @@ int _dbus_write_socket_with_unix_fds_two (int               fd,
                                           const int        *fds,
                                           int               n_fds);
 
-dbus_bool_t _dbus_socket_is_invalid (int              fd);
-
-int _dbus_connect_tcp_socket  (const char     *host,
-                               const char     *port,
-                               const char     *family,
-                               DBusError      *error);
-int _dbus_connect_tcp_socket_with_nonce  (const char     *host,
-                                          const char     *port,
-                                          const char     *family,
-                                          const char     *noncefile,
-                                          DBusError      *error);
+DBusSocket _dbus_connect_tcp_socket  (const char     *host,
+                                      const char     *port,
+                                      const char     *family,
+                                      DBusError      *error);
+DBusSocket _dbus_connect_tcp_socket_with_nonce  (const char     *host,
+                                                 const char     *port,
+                                                 const char     *family,
+                                                 const char     *noncefile,
+                                                 DBusError      *error);
 int _dbus_listen_tcp_socket   (const char     *host,
                                const char     *port,
                                const char     *family,
                                DBusString     *retport,
-                               int           **fds_p,
+                               const char    **retfamily,
+                               DBusSocket    **fds_p,
                                DBusError      *error);
-int _dbus_accept              (int             listen_fd);
+DBusSocket _dbus_accept       (DBusSocket      listen_fd);
 
-
-dbus_bool_t _dbus_read_credentials_socket (int               client_fd,
+dbus_bool_t _dbus_read_credentials_socket (DBusSocket        client_fd,
                                            DBusCredentials  *credentials,
                                            DBusError        *error);
-dbus_bool_t _dbus_send_credentials_socket (int              server_fd,
+dbus_bool_t _dbus_send_credentials_socket (DBusSocket       server_fd,
                                            DBusError       *error);
 
-dbus_bool_t _dbus_credentials_add_from_user            (DBusCredentials  *credentials,
-                                                        const DBusString *username);
+typedef enum
+{
+  DBUS_CREDENTIALS_ADD_FLAGS_USER_DATABASE = (1 << 0),
+  DBUS_CREDENTIALS_ADD_FLAGS_NONE = 0
+} DBusCredentialsAddFlags;
+
+dbus_bool_t _dbus_credentials_add_from_user            (DBusCredentials         *credentials,
+                                                        const DBusString        *username,
+                                                        DBusCredentialsAddFlags  flags,
+                                                        DBusError               *error);
+
 dbus_bool_t _dbus_credentials_add_from_current_process (DBusCredentials  *credentials);
+DBUS_PRIVATE_EXPORT
 dbus_bool_t _dbus_append_user_from_current_process     (DBusString        *str);
 
 dbus_bool_t _dbus_parse_unix_user_from_config   (const DBusString  *username,
@@ -213,7 +310,7 @@ dbus_bool_t _dbus_daemon_publish_session_bus_address (const char* address, const
 
 void _dbus_daemon_unpublish_session_bus_address (void);
 
-dbus_bool_t _dbus_socket_can_pass_unix_fd(int fd);
+dbus_bool_t _dbus_socket_can_pass_unix_fd(DBusSocket fd);
 
 /** Opaque type representing an atomically-modifiable integer
  * that can be used from multiple threads.
@@ -232,49 +329,95 @@ struct DBusAtomic
 #endif
 };
 
-/* The value we get from autofoo is in the form of a cpp expression;
- * convert that to a conventional defined/undef switch. (We can't get
- * the conventional defined/undef because of multiarch builds only running
- * ./configure once, on Darwin.) */
-#if DBUS_HAVE_ATOMIC_INT_COND
-#   define DBUS_HAVE_ATOMIC_INT 1
-#else
-#   undef DBUS_HAVE_ATOMIC_INT
-#endif
-
+DBUS_PRIVATE_EXPORT
 dbus_int32_t _dbus_atomic_inc (DBusAtomic *atomic);
+DBUS_PRIVATE_EXPORT
 dbus_int32_t _dbus_atomic_dec (DBusAtomic *atomic);
+DBUS_PRIVATE_EXPORT
+dbus_int32_t _dbus_atomic_get (DBusAtomic *atomic);
 
+#ifdef DBUS_WIN
 
-/* AIX uses different values for poll */
+/* On Windows, you can only poll sockets. We emulate Unix poll() using
+ * select(), so it doesn't matter what precise type we put in DBusPollFD;
+ * use DBusSocket so that the compiler can check we are doing it right.
+ */
+typedef DBusSocket DBusPollable;
+# define DBUS_POLLABLE_FORMAT "Iu"
+
+static inline DBusPollable
+_dbus_socket_get_pollable (DBusSocket s) { return s; }
+
+static inline SOCKET
+_dbus_pollable_printable (DBusPollable p) { return p.sock; }
+
+static inline dbus_bool_t
+_dbus_pollable_is_valid (DBusPollable p) { return _dbus_socket_is_valid (p); }
+
+static inline void
+_dbus_pollable_invalidate (DBusPollable *p) { _dbus_socket_invalidate (p); }
+
+static inline dbus_bool_t
+_dbus_pollable_equals (DBusPollable a, DBusPollable b) { return a.sock == b.sock; }
+
+#else /* !DBUS_WIN */
+
+/* On Unix, you can poll sockets, pipes, etc., and we must put exactly
+ * "int" in DBusPollFD because we're relying on its layout exactly matching
+ * struct pollfd. (This is silly, and one day we should use a better
+ * abstraction.)
+ */
+typedef int DBusPollable;
+# define DBUS_POLLABLE_FORMAT "d"
+
+static inline DBusPollable
+_dbus_socket_get_pollable (DBusSocket s) { return s.fd; }
+
+static inline int
+_dbus_pollable_printable (DBusPollable p) { return p; }
+
+static inline dbus_bool_t
+_dbus_pollable_is_valid (DBusPollable p) { return p >= 0; }
+
+static inline void
+_dbus_pollable_invalidate (DBusPollable *p) { *p = -1; }
+
+static inline dbus_bool_t
+_dbus_pollable_equals (DBusPollable a, DBusPollable b) { return a == b; }
+
+#endif /* !DBUS_WIN */
+
+#if defined(HAVE_POLL) && !defined(BROKEN_POLL)
+/**
+ * A portable struct pollfd wrapper, or an emulation of struct pollfd
+ * on platforms where poll() is missing or broken.
+ */
+typedef struct pollfd DBusPollFD;
 
-#ifdef _AIX
 /** There is data to read */
-#define _DBUS_POLLIN      0x0001
+#define _DBUS_POLLIN      POLLIN
 /** There is urgent data to read */
-#define _DBUS_POLLPRI     0x0004
+#define _DBUS_POLLPRI     POLLPRI
 /** Writing now will not block */
-#define _DBUS_POLLOUT     0x0002
+#define _DBUS_POLLOUT     POLLOUT
 /** Error condition */
-#define _DBUS_POLLERR     0x4000
+#define _DBUS_POLLERR     POLLERR
 /** Hung up */
-#define _DBUS_POLLHUP     0x2000
+#define _DBUS_POLLHUP     POLLHUP
 /** Invalid request: fd not open */
-#define _DBUS_POLLNVAL    0x8000
-#elif defined(__HAIKU__)
-/** There is data to read */
-#define _DBUS_POLLIN      0x0001
-/** Writing now will not block */
-#define _DBUS_POLLOUT     0x0002
-/** Error condition */
-#define _DBUS_POLLERR     0x0004
-/** There is urgent data to read */
-#define _DBUS_POLLPRI     0x0020
-/** Hung up */
-#define _DBUS_POLLHUP     0x0080
-/** Invalid request: fd not open */
-#define _DBUS_POLLNVAL    0x1000
+#define _DBUS_POLLNVAL    POLLNVAL
 #else
+/* Emulate poll() via select(). Because we aren't really going to call
+ * poll(), any similarly-shaped struct is acceptable, and any power of 2
+ * will do for the events/revents; these values happen to match Linux
+ * and *BSD. */
+typedef struct
+{
+  DBusPollable fd;   /**< File descriptor */
+  short events;      /**< Events to poll for */
+  short revents;     /**< Events that occurred */
+} DBusPollFD;
+
 /** There is data to read */
 #define _DBUS_POLLIN      0x0001
 /** There is urgent data to read */
@@ -289,44 +432,50 @@ dbus_int32_t _dbus_atomic_dec (DBusAtomic *atomic);
 #define _DBUS_POLLNVAL    0x0020
 #endif
 
-/**
- * A portable struct pollfd wrapper. 
- */
-typedef struct
-{
-  int fd;            /**< File descriptor */
-  short events;      /**< Events to poll for */
-  short revents;     /**< Events that occurred */
-} DBusPollFD;
-
+DBUS_PRIVATE_EXPORT
 int _dbus_poll (DBusPollFD *fds,
                 int         n_fds,
                 int         timeout_milliseconds);
 
+DBUS_PRIVATE_EXPORT
 void _dbus_sleep_milliseconds (int milliseconds);
 
-void _dbus_get_current_time (long *tv_sec,
-                             long *tv_usec);
+DBUS_PRIVATE_EXPORT
+void _dbus_get_monotonic_time (long *tv_sec,
+                               long *tv_usec);
+
+DBUS_PRIVATE_EXPORT
+void _dbus_get_real_time (long *tv_sec,
+                          long *tv_usec);
 
 /**
  * directory interface
  */
+DBUS_PRIVATE_EXPORT
 dbus_bool_t    _dbus_create_directory        (const DBusString *filename,
                                               DBusError        *error);
+DBUS_PRIVATE_EXPORT
+dbus_bool_t    _dbus_ensure_directory        (const DBusString *filename,
+                                              DBusError        *error);
+DBUS_PRIVATE_EXPORT
 dbus_bool_t    _dbus_delete_directory        (const DBusString *filename,
 					      DBusError        *error);
 
+DBUS_PRIVATE_EXPORT
 dbus_bool_t _dbus_concat_dir_and_file (DBusString       *dir,
                                        const DBusString *next_component);
 dbus_bool_t _dbus_string_get_dirname  (const DBusString *filename,
                                        DBusString       *dirname);
+DBUS_PRIVATE_EXPORT
 dbus_bool_t _dbus_path_is_absolute    (const DBusString *filename);
 
 dbus_bool_t _dbus_get_standard_session_servicedirs (DBusList **dirs);
 dbus_bool_t _dbus_get_standard_system_servicedirs (DBusList **dirs);
+dbus_bool_t _dbus_set_up_transient_session_servicedirs (DBusList  **dirs,
+                                                        DBusError  *error);
 
-dbus_bool_t _dbus_append_system_config_file  (DBusString *str);
-dbus_bool_t _dbus_append_session_config_file (DBusString *str);
+dbus_bool_t _dbus_get_system_config_file  (DBusString *str);
+dbus_bool_t _dbus_get_session_config_file (DBusString *str);
 
 /** Opaque type for reading a directory listing */
 typedef struct DBusDirIter DBusDirIter;
@@ -341,41 +490,55 @@ void         _dbus_directory_close         (DBusDirIter      *iter);
 dbus_bool_t  _dbus_check_dir_is_private_to_user    (DBusString *dir,
                                                     DBusError *error);
 
-void _dbus_fd_set_close_on_exec (intptr_t fd);
-
+DBUS_PRIVATE_EXPORT
 const char* _dbus_get_tmpdir      (void);
 
 /**
  * Random numbers 
  */
-void        _dbus_generate_pseudorandom_bytes_buffer (char *buffer,
-                                                      int   n_bytes);
-void        _dbus_generate_random_bytes_buffer (char       *buffer,
-                                                int         n_bytes);
+_DBUS_WARN_UNUSED_RESULT
+dbus_bool_t _dbus_generate_random_bytes_buffer (char       *buffer,
+                                                int         n_bytes,
+                                                DBusError  *error);
 dbus_bool_t _dbus_generate_random_bytes        (DBusString *str,
-                                                int         n_bytes);
+                                                int         n_bytes,
+                                                DBusError  *error);
+DBUS_PRIVATE_EXPORT
 dbus_bool_t _dbus_generate_random_ascii        (DBusString *str,
-                                                int         n_bytes);
+                                                int         n_bytes,
+                                                DBusError  *error);
 
+DBUS_PRIVATE_EXPORT
 const char* _dbus_error_from_errno (int error_number);
+DBUS_PRIVATE_EXPORT
 const char* _dbus_error_from_system_errno (void);
 
+int         _dbus_get_low_level_socket_errno         (void);
+
+int         _dbus_save_socket_errno                  (void);
+void        _dbus_restore_socket_errno               (int saved_errno);
 void        _dbus_set_errno_to_zero                  (void);
-dbus_bool_t _dbus_get_is_errno_nonzero               (void);
-dbus_bool_t _dbus_get_is_errno_eagain_or_ewouldblock (void);
-dbus_bool_t _dbus_get_is_errno_enomem                (void);
-dbus_bool_t _dbus_get_is_errno_eintr                 (void);
-dbus_bool_t _dbus_get_is_errno_epipe                 (void);
+dbus_bool_t _dbus_get_is_errno_eagain_or_ewouldblock (int e);
+dbus_bool_t _dbus_get_is_errno_enomem                (int e);
+dbus_bool_t _dbus_get_is_errno_eintr                 (int e);
+dbus_bool_t _dbus_get_is_errno_epipe                 (int e);
+dbus_bool_t _dbus_get_is_errno_etoomanyrefs          (int e);
+DBUS_PRIVATE_EXPORT
 const char* _dbus_strerror_from_errno                (void);
 
 void _dbus_disable_sigpipe (void);
 
-
+DBUS_PRIVATE_EXPORT
 void _dbus_exit (int code) _DBUS_GNUC_NORETURN;
 
+DBUS_PRIVATE_EXPORT
 int _dbus_printf_string_upper_bound (const char *format,
-                                     va_list args);
+                                     va_list args) _DBUS_GNUC_PRINTF (1, 0);
 
+#ifdef DBUS_ENABLE_VERBOSE_MODE
+DBUS_PRIVATE_EXPORT
+void _dbus_print_thread (void);
+#endif
 
 /**
  * Portable struct with stat() results
@@ -395,11 +558,13 @@ typedef struct
 dbus_bool_t _dbus_stat             (const DBusString *filename,
                                     DBusStat         *statbuf,
                                     DBusError        *error);
-dbus_bool_t _dbus_full_duplex_pipe (int              *fd1,
-                                    int              *fd2,
-                                    dbus_bool_t       blocking,
-                                    DBusError        *error);
+DBUS_PRIVATE_EXPORT
+dbus_bool_t _dbus_socketpair (DBusSocket       *fd1,
+                              DBusSocket       *fd2,
+                              dbus_bool_t       blocking,
+                              DBusError        *error);
 
+DBUS_PRIVATE_EXPORT
 void        _dbus_print_backtrace  (void);
 
 dbus_bool_t _dbus_become_daemon   (const DBusString *pidfile,
@@ -421,39 +586,33 @@ dbus_bool_t _dbus_command_for_pid (unsigned long  pid,
                                    int            max_len,
                                    DBusError     *error);
 
-/** A UNIX signal handler */
-typedef void (* DBusSignalHandler) (int sig);
-
-void _dbus_set_signal_handler (int               sig,
-                               DBusSignalHandler handler);
-
 dbus_bool_t _dbus_user_at_console (const char *username,
                                    DBusError  *error);
 
-void _dbus_init_system_log (void);
+typedef enum {
+  DBUS_LOG_FLAGS_STDERR = (1 << 0),
+  DBUS_LOG_FLAGS_SYSTEM_LOG = (1 << 1)
+} DBusLogFlags;
+
+DBUS_PRIVATE_EXPORT
+void _dbus_init_system_log (const char   *tag,
+                            DBusLogFlags  flags);
 
 typedef enum {
   DBUS_SYSTEM_LOG_INFO,
+  DBUS_SYSTEM_LOG_WARNING,
   DBUS_SYSTEM_LOG_SECURITY,
-  DBUS_SYSTEM_LOG_FATAL
+  DBUS_SYSTEM_LOG_ERROR
 } DBusSystemLogSeverity;
 
-void _dbus_system_log (DBusSystemLogSeverity severity, const char *msg, ...) _DBUS_GNUC_PRINTF (2, 3);
-void _dbus_system_logv (DBusSystemLogSeverity severity, const char *msg, va_list args);
-
-/* Define DBUS_VA_COPY() to do the right thing for copying va_list variables.
- * config.h may have already defined DBUS_VA_COPY as va_copy or __va_copy.
- */
-#if !defined (DBUS_VA_COPY)
-#  if defined (__GNUC__) && defined (__PPC__) && (defined (_CALL_SYSV) || defined (_WIN32))
-#    define DBUS_VA_COPY(ap1, ap2)   (*(ap1) = *(ap2))
-#  elif defined (DBUS_VA_COPY_AS_ARRAY)
-#    define DBUS_VA_COPY(ap1, ap2)   memcpy ((ap1), (ap2), sizeof (va_list))
-#  else /* va_list is a pointer */
-#    define DBUS_VA_COPY(ap1, ap2)   ((ap1) = (ap2))
-#  endif /* va_list is a pointer */
-#endif /* !DBUS_VA_COPY */
-
+DBUS_PRIVATE_EXPORT
+void _dbus_log  (DBusSystemLogSeverity  severity,
+                 const char            *msg,
+                 ...) _DBUS_GNUC_PRINTF (2, 3);
+DBUS_PRIVATE_EXPORT
+void _dbus_logv (DBusSystemLogSeverity  severity,
+                 const char            *msg,
+                 va_list args) _DBUS_GNUC_PRINTF (2, 0);
 
 /**
  * Casts a primitive C type to a byte array and then indexes
@@ -489,6 +648,7 @@ dbus_bool_t _dbus_lookup_session_address (dbus_bool_t *supported,
  */
 typedef union DBusGUID DBusGUID;
 
+DBUS_PRIVATE_EXPORT
 dbus_bool_t _dbus_read_local_machine_uuid   (DBusGUID         *machine_id,
                                              dbus_bool_t       create_if_not_found,
                                              DBusError        *error);
@@ -500,6 +660,17 @@ dbus_bool_t _dbus_read_local_machine_uuid   (DBusGUID         *machine_id,
  */
 dbus_bool_t _dbus_threads_init_platform_specific (void);
 
+/**
+ * Lock a static mutex used to protect _dbus_threads_init_platform_specific().
+ */
+void _dbus_threads_lock_platform_specific (void);
+
+/**
+ * Undo _dbus_threads_lock_platform_specific().
+ */
+void _dbus_threads_unlock_platform_specific (void);
+
+DBUS_PRIVATE_EXPORT
 dbus_bool_t _dbus_split_paths_and_append (DBusString *dirs, 
                                           const char *suffix, 
                                           DBusList **dir_list);
@@ -510,25 +681,66 @@ unsigned long _dbus_pid_for_log (void);
  * the PID file handling just needs a little more abstraction
  * in the bus daemon first.
  */
+DBUS_PRIVATE_EXPORT
 dbus_pid_t    _dbus_getpid (void);
 
-dbus_bool_t _dbus_change_to_daemon_user (const char *user,
-                                         DBusError  *error);
+DBUS_PRIVATE_EXPORT
+dbus_uid_t    _dbus_getuid (void);
 
+DBUS_PRIVATE_EXPORT
 void _dbus_flush_caches (void);
 
-/*
- * replaces the term DBUS_PREFIX in configure_time_path by the
- * current dbus installation directory. On unix this function is a noop
+dbus_bool_t _dbus_replace_install_prefix (DBusString *path);
+
+/* Do not set this too high: it is a denial-of-service risk.
+ * See <https://bugs.freedesktop.org/show_bug.cgi?id=82820>
  *
- * @param configure_time_path
- * @return real path
+ * (This needs to be in the non-Unix-specific header so that
+ * the config-parser can use it.)
  */
-const char *
-_dbus_replace_install_prefix (const char *configure_time_path);
+#define DBUS_DEFAULT_MESSAGE_UNIX_FDS 16
+
+typedef struct DBusRLimit DBusRLimit;
+
+DBusRLimit     *_dbus_rlimit_save_fd_limit                 (DBusError    *error);
+dbus_bool_t     _dbus_rlimit_raise_fd_limit_if_privileged  (unsigned int  desired,
+                                                            DBusError    *error);
+dbus_bool_t     _dbus_rlimit_restore_fd_limit              (DBusRLimit   *saved,
+                                                            DBusError    *error);
+void            _dbus_rlimit_free                          (DBusRLimit   *lim);
+
+void            _dbus_daemon_report_ready                  (void);
+void            _dbus_daemon_report_reloading              (void);
+void            _dbus_daemon_report_reloaded               (void);
+void            _dbus_daemon_report_stopping               (void);
+
+dbus_bool_t _dbus_inet_sockaddr_to_string (const void *sockaddr_pointer,
+                                           size_t len,
+                                           char *string,
+                                           size_t string_len,
+                                           const char **family_name,
+                                           dbus_uint16_t *port,
+                                           DBusError *error);
+void _dbus_set_error_with_inet_sockaddr (DBusError *error,
+                                         const void *sockaddr_pointer,
+                                         size_t len,
+                                         const char *description,
+                                         int saved_errno);
+void _dbus_combine_tcp_errors (DBusList **sources,
+                               const char *summary,
+                               const char *host,
+                               const char *port,
+                               DBusError *dest);
+
+void _dbus_disable_crash_handling (void);
 
 /** @} */
 
 DBUS_END_DECLS
 
+
+#ifdef DBUS_WIN
+#include "dbus-sysdeps-win.h"
+#endif
+
 #endif /* DBUS_SYSDEPS_H */
