@@ -21,14 +21,15 @@
  *
  */
 
+#include <config.h>
 #include "dbus-internals.h"
 #include "dbus-connection-internal.h"
+#include "dbus-nonce.h"
 #include "dbus-transport-socket.h"
 #include "dbus-transport-protected.h"
 #include "dbus-watch.h"
 #include "dbus-credentials.h"
 
-
 /**
  * @defgroup DBusTransportSocket DBusTransport implementations for sockets
  * @ingroup  DBusInternals
@@ -48,7 +49,7 @@ typedef struct DBusTransportSocket DBusTransportSocket;
 struct DBusTransportSocket
 {
   DBusTransport base;                   /**< Parent instance */
-  int fd;                               /**< File descriptor. */
+  DBusSocket fd;                        /**< File descriptor. */
   DBusWatch *read_watch;                /**< Watch for readability. */
   DBusWatch *write_watch;               /**< Watch for writability. */
 
@@ -72,7 +73,7 @@ free_watches (DBusTransport *transport)
 {
   DBusTransportSocket *socket_transport = (DBusTransportSocket*) transport;
 
-  _dbus_verbose ("%s start\n", _DBUS_FUNCTION_NAME);
+  _dbus_verbose ("start\n");
   
   if (socket_transport->read_watch)
     {
@@ -94,7 +95,7 @@ free_watches (DBusTransport *transport)
       socket_transport->write_watch = NULL;
     }
 
-  _dbus_verbose ("%s end\n", _DBUS_FUNCTION_NAME);
+  _dbus_verbose ("end\n");
 }
 
 static void
@@ -102,7 +103,7 @@ socket_finalize (DBusTransport *transport)
 {
   DBusTransportSocket *socket_transport = (DBusTransportSocket*) transport;
 
-  _dbus_verbose ("%s\n", _DBUS_FUNCTION_NAME);
+  _dbus_verbose ("\n");
   
   free_watches (transport);
 
@@ -134,7 +135,7 @@ check_write_watch (DBusTransport *transport)
   
   _dbus_transport_ref (transport);
 
-  if (_dbus_transport_get_is_authenticated (transport))
+  if (_dbus_transport_try_to_authenticate (transport))
     needed = _dbus_connection_has_messages_to_send_unlocked (transport->connection);
   else
     {
@@ -158,9 +159,9 @@ check_write_watch (DBusTransport *transport)
         }
     }
 
-  _dbus_verbose ("check_write_watch(): needed = %d on connection %p watch %p fd = %d outgoing messages exist %d\n",
+  _dbus_verbose ("check_write_watch(): needed = %d on connection %p watch %p fd = %" DBUS_SOCKET_FORMAT " outgoing messages exist %d\n",
                  needed, transport->connection, socket_transport->write_watch,
-                 socket_transport->fd,
+                 _dbus_socket_printable (socket_transport->fd),
                  _dbus_connection_has_messages_to_send_unlocked (transport->connection));
 
   _dbus_connection_toggle_watch_unlocked (transport->connection,
@@ -176,8 +177,8 @@ check_read_watch (DBusTransport *transport)
   DBusTransportSocket *socket_transport = (DBusTransportSocket*) transport;
   dbus_bool_t need_read_watch;
 
-  _dbus_verbose ("%s: fd = %d\n",
-                 _DBUS_FUNCTION_NAME, socket_transport->fd);
+  _dbus_verbose ("fd = %" DBUS_SOCKET_FORMAT "\n",
+                 _dbus_socket_printable (socket_transport->fd));
   
   if (transport->connection == NULL)
     return;
@@ -190,9 +191,10 @@ check_read_watch (DBusTransport *transport)
   
   _dbus_transport_ref (transport);
 
-  if (_dbus_transport_get_is_authenticated (transport))
+  if (_dbus_transport_try_to_authenticate (transport))
     need_read_watch =
-      _dbus_counter_get_value (transport->live_messages_size) < transport->max_live_messages_size;
+      (_dbus_counter_get_size_value (transport->live_messages) < transport->max_live_messages_size) &&
+      (_dbus_counter_get_unix_fd_value (transport->live_messages) < transport->max_live_messages_unix_fds);
   else
     {
       if (transport->receive_credentials_pending)
@@ -246,16 +248,17 @@ read_data_into_auth (DBusTransport *transport,
   DBusTransportSocket *socket_transport = (DBusTransportSocket*) transport;
   DBusString *buffer;
   int bytes_read;
-  
+  int saved_errno;
+
   *oom = FALSE;
 
   _dbus_auth_get_buffer (transport->auth, &buffer);
   
   bytes_read = _dbus_read_socket (socket_transport->fd,
                                   buffer, socket_transport->max_bytes_read_per_iteration);
+  saved_errno = _dbus_save_socket_errno ();
 
-  _dbus_auth_return_buffer (transport->auth, buffer,
-                            bytes_read > 0 ? bytes_read : 0);
+  _dbus_auth_return_buffer (transport->auth, buffer);
 
   if (bytes_read > 0)
     {
@@ -267,16 +270,16 @@ read_data_into_auth (DBusTransport *transport,
     {
       /* EINTR already handled for us */
 
-      if (_dbus_get_is_errno_enomem ())
+      if (_dbus_get_is_errno_enomem (saved_errno))
         {
           *oom = TRUE;
         }
-      else if (_dbus_get_is_errno_eagain_or_ewouldblock ())
+      else if (_dbus_get_is_errno_eagain_or_ewouldblock (saved_errno))
         ; /* do nothing, just return FALSE below */
       else
         {
           _dbus_verbose ("Error reading from remote app: %s\n",
-                         _dbus_strerror_from_errno ());
+                         _dbus_strerror (saved_errno));
           do_io_error (transport);
         }
 
@@ -299,6 +302,7 @@ write_data_from_auth (DBusTransport *transport)
 {
   DBusTransportSocket *socket_transport = (DBusTransportSocket*) transport;
   int bytes_written;
+  int saved_errno;
   const DBusString *buffer;
 
   if (!_dbus_auth_get_bytes_to_send (transport->auth,
@@ -308,6 +312,7 @@ write_data_from_auth (DBusTransport *transport)
   bytes_written = _dbus_write_socket (socket_transport->fd,
                                       buffer,
                                       0, _dbus_string_get_length (buffer));
+  saved_errno = _dbus_save_socket_errno ();
 
   if (bytes_written > 0)
     {
@@ -318,12 +323,12 @@ write_data_from_auth (DBusTransport *transport)
     {
       /* EINTR already handled for us */
       
-      if (_dbus_get_is_errno_eagain_or_ewouldblock ())
+      if (_dbus_get_is_errno_eagain_or_ewouldblock (saved_errno))
         ;
       else
         {
           _dbus_verbose ("Error writing to remote app: %s\n",
-                         _dbus_strerror_from_errno ());
+                         _dbus_strerror (saved_errno));
           do_io_error (transport);
         }
     }
@@ -403,7 +408,7 @@ do_authentication (DBusTransport *transport,
 
   oom = FALSE;
   
-  orig_auth_state = _dbus_transport_get_is_authenticated (transport);
+  orig_auth_state = _dbus_transport_try_to_authenticate (transport);
 
   /* This is essential to avoid the check_write_watch() at the end,
    * we don't want to add a write watch in do_iteration before
@@ -418,7 +423,7 @@ do_authentication (DBusTransport *transport,
   
   _dbus_transport_ref (transport);
   
-  while (!_dbus_transport_get_is_authenticated (transport) &&
+  while (!_dbus_transport_try_to_authenticate (transport) &&
          _dbus_transport_get_is_connected (transport))
     {      
       if (!exchange_credentials (transport, do_reading, do_writing))
@@ -471,12 +476,17 @@ do_authentication (DBusTransport *transport,
           _dbus_verbose (" %s auth state: authenticated\n",
                          TRANSPORT_SIDE (transport));
           break;
+
+        case DBUS_AUTH_STATE_INVALID:
+          /* fall through */
+        default:
+          _dbus_assert_not_reached ("invalid auth state");
         }
     }
 
  out:
   if (auth_completed)
-    *auth_completed = (orig_auth_state != _dbus_transport_get_is_authenticated (transport));
+    *auth_completed = (orig_auth_state != _dbus_transport_try_to_authenticate (transport));
   
   check_read_watch (transport);
   check_write_watch (transport);
@@ -497,7 +507,7 @@ do_writing (DBusTransport *transport)
   dbus_bool_t oom;
   
   /* No messages without authentication! */
-  if (!_dbus_transport_get_is_authenticated (transport))
+  if (!_dbus_transport_try_to_authenticate (transport))
     {
       _dbus_verbose ("Not authenticated, not writing anything\n");
       return TRUE;
@@ -510,9 +520,9 @@ do_writing (DBusTransport *transport)
     }
 
 #if 1
-  _dbus_verbose ("do_writing(), have_messages = %d, fd = %d\n",
+  _dbus_verbose ("do_writing(), have_messages = %d, fd = %" DBUS_SOCKET_FORMAT "\n",
                  _dbus_connection_has_messages_to_send_unlocked (transport->connection),
-                 socket_transport->fd);
+                 _dbus_socket_printable (socket_transport->fd));
 #endif
   
   oom = FALSE;
@@ -527,6 +537,7 @@ do_writing (DBusTransport *transport)
       const DBusString *body;
       int header_len, body_len;
       int total_bytes_to_write;
+      int saved_errno;
       
       if (total > socket_transport->max_bytes_written_per_iteration)
         {
@@ -551,6 +562,9 @@ do_writing (DBusTransport *transport)
 
       if (_dbus_auth_needs_encoding (transport->auth))
         {
+          /* Does fd passing even make sense with encoded data? */
+          _dbus_assert(!DBUS_TRANSPORT_CAN_SEND_UNIX_FD(transport));
+
           if (_dbus_string_get_length (&socket_transport->encoded_outgoing) == 0)
             {
               if (!_dbus_auth_encode_data (transport->auth,
@@ -581,6 +595,7 @@ do_writing (DBusTransport *transport)
                                 &socket_transport->encoded_outgoing,
                                 socket_transport->message_bytes_written,
                                 total_bytes_to_write - socket_transport->message_bytes_written);
+          saved_errno = _dbus_save_socket_errno ();
         }
       else
         {
@@ -588,27 +603,56 @@ do_writing (DBusTransport *transport)
 
 #if 0
           _dbus_verbose ("message is %d bytes\n",
-                         total_bytes_to_write);          
+                         total_bytes_to_write);
 #endif
-          
-          if (socket_transport->message_bytes_written < header_len)
+
+#ifdef HAVE_UNIX_FD_PASSING
+          if (socket_transport->message_bytes_written <= 0 && DBUS_TRANSPORT_CAN_SEND_UNIX_FD(transport))
             {
+              /* Send the fds along with the first byte of the message */
+              const int *unix_fds;
+              unsigned n;
+
+              _dbus_message_get_unix_fds(message, &unix_fds, &n);
+
               bytes_written =
-                _dbus_write_socket_two (socket_transport->fd,
-                                        header,
-                                        socket_transport->message_bytes_written,
-                                        header_len - socket_transport->message_bytes_written,
-                                        body,
-                                        0, body_len);
+                _dbus_write_socket_with_unix_fds_two (socket_transport->fd,
+                                                      header,
+                                                      socket_transport->message_bytes_written,
+                                                      header_len - socket_transport->message_bytes_written,
+                                                      body,
+                                                      0, body_len,
+                                                      unix_fds,
+                                                      n);
+              saved_errno = _dbus_save_socket_errno ();
+
+              if (bytes_written > 0 && n > 0)
+                _dbus_verbose("Wrote %i unix fds\n", n);
             }
           else
+#endif
             {
-              bytes_written =
-                _dbus_write_socket (socket_transport->fd,
-                                    body,
-                                    (socket_transport->message_bytes_written - header_len),
-                                    body_len -
-                                    (socket_transport->message_bytes_written - header_len));
+              if (socket_transport->message_bytes_written < header_len)
+                {
+                  bytes_written =
+                    _dbus_write_socket_two (socket_transport->fd,
+                                            header,
+                                            socket_transport->message_bytes_written,
+                                            header_len - socket_transport->message_bytes_written,
+                                            body,
+                                            0, body_len);
+                }
+              else
+                {
+                  bytes_written =
+                    _dbus_write_socket (socket_transport->fd,
+                                        body,
+                                        (socket_transport->message_bytes_written - header_len),
+                                        body_len -
+                                        (socket_transport->message_bytes_written - header_len));
+                }
+
+              saved_errno = _dbus_save_socket_errno ();
             }
         }
 
@@ -616,16 +660,48 @@ do_writing (DBusTransport *transport)
         {
           /* EINTR already handled for us */
           
-          /* For some discussion of why we also ignore EPIPE here, see
+          /* If the other end closed the socket with close() or shutdown(), we
+           * receive EPIPE here but we must not close the socket yet: there
+           * might still be some data to read. See:
            * http://lists.freedesktop.org/archives/dbus/2008-March/009526.html
            */
           
-          if (_dbus_get_is_errno_eagain_or_ewouldblock () || _dbus_get_is_errno_epipe ())
+          if (_dbus_get_is_errno_eagain_or_ewouldblock (saved_errno) || _dbus_get_is_errno_epipe (saved_errno))
             goto out;
+
+          /* Since Linux commit 25888e (from 2.6.37-rc4, Nov 2010), sendmsg()
+           * on Unix sockets returns -1 errno=ETOOMANYREFS when the passfd
+           * mechanism (SCM_RIGHTS) is used recursively with a recursion level
+           * of maximum 4. The kernel does not have an API to check whether
+           * the passed fds can be forwarded and it can change asynchronously.
+           * See:
+           * https://bugs.freedesktop.org/show_bug.cgi?id=80163
+           */
+
+          else if (_dbus_get_is_errno_etoomanyrefs (saved_errno))
+            {
+              /* We only send fds in the first byte of the message.
+               * ETOOMANYREFS cannot happen after.
+               */
+              _dbus_assert (socket_transport->message_bytes_written == 0);
+
+              _dbus_verbose (" discard message of %d bytes due to ETOOMANYREFS\n",
+                             total_bytes_to_write);
+
+              socket_transport->message_bytes_written = 0;
+              _dbus_string_set_length (&socket_transport->encoded_outgoing, 0);
+              _dbus_string_compact (&socket_transport->encoded_outgoing, 2048);
+
+              /* The message was not actually sent but it needs to be removed
+               * from the outgoing queue
+               */
+              _dbus_connection_message_sent_unlocked (transport->connection,
+                                                      message);
+            }
           else
             {
               _dbus_verbose ("Error writing to remote app: %s\n",
-                             _dbus_strerror_from_errno ());
+                             _dbus_strerror (saved_errno));
               do_io_error (transport);
               goto out;
             }
@@ -647,8 +723,8 @@ do_writing (DBusTransport *transport)
               _dbus_string_set_length (&socket_transport->encoded_outgoing, 0);
               _dbus_string_compact (&socket_transport->encoded_outgoing, 2048);
 
-              _dbus_connection_message_sent (transport->connection,
-                                             message);
+              _dbus_connection_message_sent_unlocked (transport->connection,
+                                                      message);
             }
         }
     }
@@ -669,12 +745,13 @@ do_reading (DBusTransport *transport)
   int bytes_read;
   int total;
   dbus_bool_t oom;
+  int saved_errno;
 
-  _dbus_verbose ("%s: fd = %d\n", _DBUS_FUNCTION_NAME,
-                 socket_transport->fd);
+  _dbus_verbose ("fd = %" DBUS_SOCKET_FORMAT "\n",
+                 _dbus_socket_printable (socket_transport->fd));
   
   /* No messages without authentication! */
-  if (!_dbus_transport_get_is_authenticated (transport))
+  if (!_dbus_transport_try_to_authenticate (transport))
     return TRUE;
 
   oom = FALSE;
@@ -704,6 +781,9 @@ do_reading (DBusTransport *transport)
   
   if (_dbus_auth_needs_decoding (transport->auth))
     {
+      /* Does fd passing even make sense with encoded data? */
+      _dbus_assert(!DBUS_TRANSPORT_CAN_SEND_UNIX_FD(transport));
+
       if (_dbus_string_get_length (&socket_transport->encoded_incoming) > 0)
         bytes_read = _dbus_string_get_length (&socket_transport->encoded_incoming);
       else
@@ -711,34 +791,32 @@ do_reading (DBusTransport *transport)
                                         &socket_transport->encoded_incoming,
                                         socket_transport->max_bytes_read_per_iteration);
 
+      saved_errno = _dbus_save_socket_errno ();
+
       _dbus_assert (_dbus_string_get_length (&socket_transport->encoded_incoming) ==
                     bytes_read);
       
       if (bytes_read > 0)
         {
-          int orig_len;
-          
           _dbus_message_loader_get_buffer (transport->loader,
-                                           &buffer);
+                                           &buffer,
+                                           NULL,
+                                           NULL);
 
-          orig_len = _dbus_string_get_length (buffer);
-          
           if (!_dbus_auth_decode_data (transport->auth,
                                        &socket_transport->encoded_incoming,
                                        buffer))
             {
               _dbus_verbose ("Out of memory decoding incoming data\n");
               _dbus_message_loader_return_buffer (transport->loader,
-                                              buffer,
-                                              _dbus_string_get_length (buffer) - orig_len);
+                                              buffer);
 
               oom = TRUE;
               goto out;
             }
 
           _dbus_message_loader_return_buffer (transport->loader,
-                                              buffer,
-                                              _dbus_string_get_length (buffer) - orig_len);
+                                              buffer);
 
           _dbus_string_set_length (&socket_transport->encoded_incoming, 0);
           _dbus_string_compact (&socket_transport->encoded_incoming, 2048);
@@ -746,33 +824,70 @@ do_reading (DBusTransport *transport)
     }
   else
     {
+      int max_to_read = DBUS_MAXIMUM_MESSAGE_LENGTH;
+      dbus_bool_t may_read_unix_fds = TRUE;
+
       _dbus_message_loader_get_buffer (transport->loader,
-                                       &buffer);
-      
-      bytes_read = _dbus_read_socket (socket_transport->fd,
-                                      buffer, socket_transport->max_bytes_read_per_iteration);
-      
+                                       &buffer,
+                                       &max_to_read,
+                                       &may_read_unix_fds);
+
+      if (max_to_read > socket_transport->max_bytes_read_per_iteration)
+        max_to_read = socket_transport->max_bytes_read_per_iteration;
+
+#ifdef HAVE_UNIX_FD_PASSING
+      if (DBUS_TRANSPORT_CAN_SEND_UNIX_FD(transport) && may_read_unix_fds)
+        {
+          int *fds;
+          unsigned int n_fds;
+
+          if (!_dbus_message_loader_get_unix_fds(transport->loader, &fds, &n_fds))
+            {
+              _dbus_verbose ("Out of memory reading file descriptors\n");
+              _dbus_message_loader_return_buffer (transport->loader, buffer);
+              oom = TRUE;
+              goto out;
+            }
+
+          bytes_read = _dbus_read_socket_with_unix_fds(socket_transport->fd,
+                                                       buffer,
+                                                       max_to_read,
+                                                       fds, &n_fds);
+          saved_errno = _dbus_save_socket_errno ();
+
+          if (bytes_read >= 0 && n_fds > 0)
+            _dbus_verbose("Read %i unix fds\n", n_fds);
+
+          _dbus_message_loader_return_unix_fds(transport->loader, fds, bytes_read < 0 ? 0 : n_fds);
+        }
+      else
+#endif
+        {
+          bytes_read = _dbus_read_socket (socket_transport->fd,
+                                          buffer, max_to_read);
+          saved_errno = _dbus_save_socket_errno ();
+        }
+
       _dbus_message_loader_return_buffer (transport->loader,
-                                          buffer,
-                                          bytes_read < 0 ? 0 : bytes_read);
+                                          buffer);
     }
-  
+
   if (bytes_read < 0)
     {
       /* EINTR already handled for us */
 
-      if (_dbus_get_is_errno_enomem ())
+      if (_dbus_get_is_errno_enomem (saved_errno))
         {
           _dbus_verbose ("Out of memory in read()/do_reading()\n");
           oom = TRUE;
           goto out;
         }
-      else if (_dbus_get_is_errno_eagain_or_ewouldblock ())
+      else if (_dbus_get_is_errno_eagain_or_ewouldblock (saved_errno))
         goto out;
       else
         {
           _dbus_verbose ("Error reading from remote app: %s\n",
-                         _dbus_strerror_from_errno ());
+                         _dbus_strerror (saved_errno));
           do_io_error (transport);
           goto out;
         }
@@ -910,8 +1025,8 @@ socket_handle_watch (DBusTransport *transport,
         _dbus_verbose ("asked to handle write watch with non-write condition 0x%x\n",
                        flags);
       else
-        _dbus_verbose ("asked to handle watch %p on fd %d that we don't recognize\n",
-                       watch, dbus_watch_get_socket (watch));
+        _dbus_verbose ("asked to handle watch %p on fd %" DBUS_SOCKET_FORMAT " that we don't recognize\n",
+                       watch, _dbus_socket_printable (_dbus_watch_get_socket (watch)));
     }
 #endif /* DBUS_ENABLE_VERBOSE_MODE */
 
@@ -923,12 +1038,12 @@ socket_disconnect (DBusTransport *transport)
 {
   DBusTransportSocket *socket_transport = (DBusTransportSocket*) transport;
 
-  _dbus_verbose ("%s\n", _DBUS_FUNCTION_NAME);
+  _dbus_verbose ("\n");
   
   free_watches (transport);
   
   _dbus_close_socket (socket_transport->fd, NULL);
-  socket_transport->fd = -1;
+  _dbus_socket_invalidate (&socket_transport->fd);
 }
 
 static dbus_bool_t
@@ -979,13 +1094,13 @@ socket_do_iteration (DBusTransport *transport,
   int poll_res;
   int poll_timeout;
 
-  _dbus_verbose (" iteration flags = %s%s timeout = %d read_watch = %p write_watch = %p fd = %d\n",
+  _dbus_verbose (" iteration flags = %s%s timeout = %d read_watch = %p write_watch = %p fd = %" DBUS_SOCKET_FORMAT "\n",
                  flags & DBUS_ITERATION_DO_READING ? "read" : "",
                  flags & DBUS_ITERATION_DO_WRITING ? "write" : "",
                  timeout_milliseconds,
                  socket_transport->read_watch,
                  socket_transport->write_watch,
-                 socket_transport->fd);
+                 _dbus_socket_printable (socket_transport->fd));
   
   /* the passed in DO_READING/DO_WRITING flags indicate whether to
    * read/write messages, but regardless of those we may need to block
@@ -993,10 +1108,10 @@ socket_do_iteration (DBusTransport *transport,
    * we don't want to read any messages yet if not given DO_READING.
    */
 
-  poll_fd.fd = socket_transport->fd;
+  poll_fd.fd = _dbus_socket_get_pollable (socket_transport->fd);
   poll_fd.events = 0;
   
-  if (_dbus_transport_get_is_authenticated (transport))
+  if (_dbus_transport_try_to_authenticate (transport))
     {
       /* This is kind of a hack; if we have stuff to write, then try
        * to avoid the poll. This is probably about a 5% speedup on an
@@ -1046,6 +1161,8 @@ socket_do_iteration (DBusTransport *transport,
 
   if (poll_fd.events)
     {
+      int saved_errno;
+
       if (flags & DBUS_ITERATION_BLOCK)
 	poll_timeout = timeout_milliseconds;
       else
@@ -1058,19 +1175,20 @@ socket_do_iteration (DBusTransport *transport,
        */
       if (flags & DBUS_ITERATION_BLOCK)
         {
-          _dbus_verbose ("unlock %s pre poll\n", _DBUS_FUNCTION_NAME);
+          _dbus_verbose ("unlock pre poll\n");
           _dbus_connection_unlock (transport->connection);
         }
       
     again:
       poll_res = _dbus_poll (&poll_fd, 1, poll_timeout);
+      saved_errno = _dbus_save_socket_errno ();
 
-      if (poll_res < 0 && _dbus_get_is_errno_eintr ())
+      if (poll_res < 0 && _dbus_get_is_errno_eintr (saved_errno))
 	goto again;
 
       if (flags & DBUS_ITERATION_BLOCK)
         {
-          _dbus_verbose ("lock %s post poll\n", _DBUS_FUNCTION_NAME);
+          _dbus_verbose ("lock post poll\n");
           _dbus_connection_lock (transport->connection);
         }
       
@@ -1108,7 +1226,7 @@ socket_do_iteration (DBusTransport *transport,
       else
         {
           _dbus_verbose ("Error from _dbus_poll(): %s\n",
-                         _dbus_strerror_from_errno ());
+                         _dbus_strerror (saved_errno));
         }
     }
 
@@ -1139,7 +1257,7 @@ socket_live_messages_changed (DBusTransport *transport)
 
 static dbus_bool_t
 socket_get_socket_fd (DBusTransport *transport,
-                      int           *fd_p)
+                      DBusSocket    *fd_p)
 {
   DBusTransportSocket *socket_transport = (DBusTransportSocket*) transport;
   
@@ -1170,41 +1288,50 @@ static const DBusTransportVTable socket_vtable = {
  * @returns the new transport, or #NULL if no memory.
  */
 DBusTransport*
-_dbus_transport_new_for_socket (int               fd,
+_dbus_transport_new_for_socket (DBusSocket        fd,
                                 const DBusString *server_guid,
                                 const DBusString *address)
 {
   DBusTransportSocket *socket_transport;
-  
+  DBusString invalid = _DBUS_STRING_INIT_INVALID;
+
   socket_transport = dbus_new0 (DBusTransportSocket, 1);
   if (socket_transport == NULL)
     return NULL;
 
+  /* So they can be "freed" without error */
+  socket_transport->encoded_outgoing = invalid;
+  socket_transport->encoded_incoming = invalid;
+
   if (!_dbus_string_init (&socket_transport->encoded_outgoing))
-    goto failed_0;
+    goto failed;
 
   if (!_dbus_string_init (&socket_transport->encoded_incoming))
-    goto failed_1;
+    goto failed;
   
-  socket_transport->write_watch = _dbus_watch_new (fd,
+  socket_transport->write_watch = _dbus_watch_new (_dbus_socket_get_pollable (fd),
                                                  DBUS_WATCH_WRITABLE,
                                                  FALSE,
                                                  NULL, NULL, NULL);
   if (socket_transport->write_watch == NULL)
-    goto failed_2;
+    goto failed;
   
-  socket_transport->read_watch = _dbus_watch_new (fd,
+  socket_transport->read_watch = _dbus_watch_new (_dbus_socket_get_pollable (fd),
                                                 DBUS_WATCH_READABLE,
                                                 FALSE,
                                                 NULL, NULL, NULL);
   if (socket_transport->read_watch == NULL)
-    goto failed_3;
+    goto failed;
 
   if (!_dbus_transport_init_base (&socket_transport->base,
                                   &socket_vtable,
                                   server_guid, address))
-    goto failed_4;
-  
+    goto failed;
+
+#ifdef HAVE_UNIX_FD_PASSING
+  _dbus_auth_set_unix_fd_possible(socket_transport->base.auth, _dbus_socket_can_pass_unix_fd(fd));
+#endif
+
   socket_transport->fd = fd;
   socket_transport->message_bytes_written = 0;
   
@@ -1214,15 +1341,21 @@ _dbus_transport_new_for_socket (int               fd,
   
   return (DBusTransport*) socket_transport;
 
- failed_4:
-  _dbus_watch_unref (socket_transport->read_watch);
- failed_3:
-  _dbus_watch_unref (socket_transport->write_watch);
- failed_2:
+failed:
+  if (socket_transport->read_watch != NULL)
+    {
+      _dbus_watch_invalidate (socket_transport->read_watch);
+      _dbus_watch_unref (socket_transport->read_watch);
+    }
+
+  if (socket_transport->write_watch != NULL)
+    {
+      _dbus_watch_invalidate (socket_transport->write_watch);
+      _dbus_watch_unref (socket_transport->write_watch);
+    }
+
   _dbus_string_free (&socket_transport->encoded_incoming);
- failed_1:
   _dbus_string_free (&socket_transport->encoded_outgoing);
- failed_0:
   dbus_free (socket_transport);
   return NULL;
 }
@@ -1234,6 +1367,7 @@ _dbus_transport_new_for_socket (int               fd,
  * @param host the host to connect to
  * @param port the port to connect to
  * @param family the address family to connect to
+ * @param noncefile path to nonce file
  * @param error location to store reason for failure.
  * @returns a new transport, or #NULL on failure.
  */
@@ -1241,9 +1375,10 @@ DBusTransport*
 _dbus_transport_new_for_tcp_socket (const char     *host,
                                     const char     *port,
                                     const char     *family,
+                                    const char     *noncefile,
                                     DBusError      *error)
 {
-  int fd;
+  DBusSocket fd;
   DBusTransport *transport;
   DBusString address;
   
@@ -1258,7 +1393,7 @@ _dbus_transport_new_for_tcp_socket (const char     *host,
   if (host == NULL)
     host = "localhost";
 
-  if (!_dbus_string_append (&address, "tcp:"))
+  if (!_dbus_string_append (&address, noncefile ? "nonce-tcp:" : "tcp:"))
     goto error;
 
   if (!_dbus_string_append (&address, "host=") ||
@@ -1270,20 +1405,23 @@ _dbus_transport_new_for_tcp_socket (const char     *host,
     goto error;
 
   if (family != NULL &&
-      (!_dbus_string_append (&address, "family=") ||
+      (!_dbus_string_append (&address, ",family=") ||
        !_dbus_string_append (&address, family)))
     goto error;
 
-  fd = _dbus_connect_tcp_socket (host, port, family, error);
-  if (fd < 0)
+  if (noncefile != NULL &&
+      (!_dbus_string_append (&address, ",noncefile=") ||
+       !_dbus_string_append (&address, noncefile)))
+    goto error;
+
+  fd = _dbus_connect_tcp_socket_with_nonce (host, port, family, noncefile, error);
+  if (!_dbus_socket_is_valid (fd))
     {
       _DBUS_ASSERT_ERROR_IS_SET (error);
       _dbus_string_free (&address);
       return NULL;
     }
 
-  _dbus_fd_set_close_on_exec (fd);
-  
   _dbus_verbose ("Successfully connected to tcp socket %s:%s\n",
                  host, port);
   
@@ -1293,7 +1431,7 @@ _dbus_transport_new_for_tcp_socket (const char     *host,
     {
       dbus_set_error (error, DBUS_ERROR_NO_MEMORY, NULL);
       _dbus_close_socket (fd, NULL);
-      fd = -1;
+      _dbus_socket_invalidate (&fd);
     }
 
   return transport;
@@ -1318,23 +1456,34 @@ _dbus_transport_open_socket(DBusAddressEntry  *entry,
                             DBusError         *error)
 {
   const char *method;
+  dbus_bool_t isTcp;
+  dbus_bool_t isNonceTcp;
   
   method = dbus_address_entry_get_method (entry);
   _dbus_assert (method != NULL);
 
-  if (strcmp (method, "tcp") == 0)
+  isTcp = strcmp (method, "tcp") == 0;
+  isNonceTcp = strcmp (method, "nonce-tcp") == 0;
+
+  if (isTcp || isNonceTcp)
     {
       const char *host = dbus_address_entry_get_value (entry, "host");
       const char *port = dbus_address_entry_get_value (entry, "port");
       const char *family = dbus_address_entry_get_value (entry, "family");
+      const char *noncefile = dbus_address_entry_get_value (entry, "noncefile");
+
+      if ((isNonceTcp == TRUE) != (noncefile != NULL)) {
+          _dbus_set_bad_address (error, method, "noncefile", NULL);
+          return DBUS_TRANSPORT_OPEN_BAD_ADDRESS;
+      }
 
       if (port == NULL)
         {
-          _dbus_set_bad_address (error, "tcp", "port", NULL);
+          _dbus_set_bad_address (error, method, "port", NULL);
           return DBUS_TRANSPORT_OPEN_BAD_ADDRESS;
         }
 
-      *transport_p = _dbus_transport_new_for_tcp_socket (host, port, family, error);
+      *transport_p = _dbus_transport_new_for_tcp_socket (host, port, family, noncefile, error);
       if (*transport_p == NULL)
         {
           _DBUS_ASSERT_ERROR_IS_SET (error);
