@@ -1,12 +1,16 @@
 
+#include <config.h>
+#include <dbus/dbus-valgrind-internal.h>
 #include "../test-utils.h"
 
 static DBusLoop *loop;
 
+static void die (const char *message) _DBUS_GNUC_NORETURN;
+
 static void
 die (const char *message)
 {
-  fprintf (stderr, "*** test-shutdown: %s", message);
+  printf ("Bail out! test-shutdown: %s", message);
   exit (1);
 }
 
@@ -35,8 +39,7 @@ open_destroy_shared_session_bus_connection (void)
   if (loop == NULL)
     die ("No memory\n");
   
-  if (!test_connection_setup (loop, connection))
-    die ("No memory\n");
+  test_connection_setup (loop, connection);
 
   test_connection_shutdown (loop, connection);
  
@@ -45,23 +48,34 @@ open_destroy_shared_session_bus_connection (void)
   dbus_connection_unref (connection); 
 }
 
+/* This test outputs TAP syntax: http://testanything.org/ */
 int
 main (int    argc,
       char **argv)
 {
-  open_destroy_shared_session_bus_connection ();
+  int test_num = 0;
 
-  dbus_shutdown ();
+  if (RUNNING_ON_VALGRIND)
+    {
+      printf ("1..0 # SKIP Not ready to run under valgrind yet\n");
+      return 0;
+    }
 
   open_destroy_shared_session_bus_connection ();
 
   dbus_shutdown ();
+  printf ("ok %d\n", ++test_num);
 
   open_destroy_shared_session_bus_connection ();
 
   dbus_shutdown ();
+  printf ("ok %d\n", ++test_num);
 
-  _dbus_verbose ("*** Test shutdown exiting\n");
-  
+  open_destroy_shared_session_bus_connection ();
+
+  dbus_shutdown ();
+  printf ("ok %d\n", ++test_num);
+
+  printf ("1..%d\n", test_num);
   return 0;
 }
