@@ -5,22 +5,16 @@
 #include <signal.h>
 #endif
 
-#ifdef HAVE_SETRLIMIT
-#include <sys/resource.h>
-#endif
+#include "dbus/dbus-sysdeps.h"
 
 int
 main (int argc, char **argv)
 {
   char *p;  
 
-#if HAVE_SETRLIMIT
-  struct rlimit r = { 0, };
-  
-  getrlimit (RLIMIT_CORE, &r);
-  r.rlim_cur = 0;
-  setrlimit (RLIMIT_CORE, &r);
-  
+  _dbus_disable_crash_handling ();
+
+#ifdef HAVE_RAISE
   raise (SIGSEGV);
 #endif
   p = NULL;
